@@ -17,6 +17,7 @@
 #include "dialog.h"
 #include "strutil.h"
 #include "guiutil.h"
+#include "odmultibutton.h"
 #include "xmlutil.h"
 #include "cmdseq.h"
 #include "cmdseq_dialog.h"
@@ -78,7 +79,7 @@ static P_Buttons	the_page;
 
 /* ----------------------------------------------------------------------------------------- */
 
-static gint	evt_button_clicked(GtkWidget *wid, gpointer wuser, guint index, gpointer suser);
+static gint	evt_button_clicked(GtkWidget *wid, gpointer user);
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -89,7 +90,7 @@ static gint	evt_button_clicked(GtkWidget *wid, gpointer wuser, guint index, gpoi
 */
 static void set_button_state(GtkWidget *btn, gboolean state)
 {
-	od_emil_button_config_select(OD_EMIL_BUTTON(btn), state);
+	od_multibutton_set_config_selected(OD_MULTIBUTTON(btn), state);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -177,7 +178,7 @@ static void populate_scwin(P_Buttons *page)
 	if((page->shvbox = btn_buttonsheet_build(page->min, &page->buttons, page->cur_sheet, TRUE, GTK_SIGNAL_FUNC(evt_button_clicked), page)) != NULL)
 	{
 		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(page->scwin), page->shvbox);
-		gtk_widget_show(page->shvbox);
+		gtk_widget_show_all(page->shvbox);
 	}
 	reset_widgets(page);
 }
@@ -213,10 +214,11 @@ static void set_mode_selected(P_Buttons *page, GtkWidget *wid)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-15 -	One of the buttons in the sheet was clicked. */
-static gint evt_button_clicked(GtkWidget *wid, gpointer wuser, guint index, gpointer suser)
+static gint evt_button_clicked(GtkWidget *wid, gpointer user)
 {
 	Button		*but = btn_button_get(wid);
-	P_Buttons	*page = suser;
+	P_Buttons	*page = user;
+	gint		index = od_multibutton_get_index(OD_MULTIBUTTON(wid));
 
 	switch(page->mode)
 	{
@@ -244,7 +246,6 @@ static gint evt_button_clicked(GtkWidget *wid, gpointer wuser, guint index, gpoi
 			populate_scwin(page);
 			break;
 	}
-
 	return TRUE;
 }
 
@@ -588,11 +589,12 @@ static void col_bg_changed(GdkColor *color, gpointer user)
 /* 1999-05-02 -	User clicked the background color multi-faced button. Either bring up a dialog,
 **		letting user pick a new color, or reset the button's color to the default.
 */
-static void evt_col_bg_clicked(GtkWidget *wid, gpointer wuser, guint function, gpointer suser)
+static void evt_col_bg_clicked(GtkWidget *wid, gpointer user)
 {
-	BtnFace		tface = (BtnFace) wuser;
-	P_Buttons	*page = suser;
+	BtnFace		function = od_multibutton_get_index(OD_MULTIBUTTON(wid));
+	P_Buttons	*page = user;
 	Button		*btn;
+	gint		tface = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wid), "face"));
 
 	if((page != NULL) && ((btn = btn_button_get(page->button)) != NULL))
 	{
@@ -610,7 +612,7 @@ static void evt_col_bg_clicked(GtkWidget *wid, gpointer wuser, guint function, g
 			else
 				btn_button_set_color_bg_widget(page->button, tface, op);
 		}
-		else				/* Time to reset color. */
+		else		/* Time to reset color to system default. */
 		{
 			btn_button_set_color_bg_widget(page->button, tface, NULL);
 			page->modified = TRUE;
@@ -628,14 +630,15 @@ static void col_fg_changed(GdkColor *color, gpointer user)
 	btn_button_set_color_fg_widget(page->button, face, color);
 }
 
-/* 1999-05-02 -	User clicked the foreground color multi-faced button. Either bring up a dialog,
+/* 2008-11-28 -	User clicked the foreground color multi-faced button. Either bring up a dialog,
 **		letting user pick a new color, or reset the button's color to the default.
 */
-static void evt_col_fg_clicked(GtkWidget *wid, gpointer wuser, guint function, gpointer suser)
+static void evt_col_fg_clicked(GtkWidget *wid, gpointer user)
 {
-	BtnFace		tface = (BtnFace) wuser;
-	P_Buttons	*page = suser;
+	BtnFace		function = od_multibutton_get_index(OD_MULTIBUTTON(wid));
+	P_Buttons	*page = user;
 	Button		*btn;
+	gint		tface = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wid), "face"));
 
 	if((page != NULL) && ((btn = btn_button_get(page->button)) != NULL))
 	{
@@ -653,7 +656,7 @@ static void evt_col_fg_clicked(GtkWidget *wid, gpointer wuser, guint function, g
 			else
 				btn_button_set_color_fg_widget(page->button, tface, op);
 		}
-		else				/* Time to reset color. */
+		else		/* Time to reset color to system default. */
 		{
 			btn_button_set_color_fg_widget(page->button, tface, NULL);
 			page->modified = TRUE;
@@ -676,18 +679,18 @@ static void init_bdef(MainInfo *min, P_Buttons *page, BDef *def, const gchar *la
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
 	def->label = gtk_entry_new_with_max_length(BTN_LABEL_SIZE - 1);
 	gtk_object_set_data(GTK_OBJECT(def->label), "face", GUINT_TO_POINTER(face));
-	def->label_sig = gtk_signal_connect(GTK_OBJECT(def->label), "changed", GTK_SIGNAL_FUNC(evt_label_changed), page);
+	def->label_sig = g_signal_connect(G_OBJECT(def->label), "changed", G_CALLBACK(evt_label_changed), page);
 	gtk_table_attach(GTK_TABLE(table), def->label, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
 	def->cmdseq = gtk_entry_new();
 	gtk_object_set_data(GTK_OBJECT(def->cmdseq), "face", GUINT_TO_POINTER(face));
-	def->cmdseq_sig = gtk_signal_connect(GTK_OBJECT(def->cmdseq), "changed", GTK_SIGNAL_FUNC(evt_cmdseq_changed), page);
+	def->cmdseq_sig = g_signal_connect(G_OBJECT(def->cmdseq), "changed", G_CALLBACK(evt_cmdseq_changed), page);
 	gtk_table_attach(GTK_TABLE(table), def->cmdseq, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
 	btn = gui_details_button_new(min->gui->window->window);
 	gtk_object_set_data(GTK_OBJECT(btn), "face", GUINT_TO_POINTER(face));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_cmdseq_pick_clicked), page);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_cmdseq_pick_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 1, 2,  GTK_FILL,0,0,0);
 
 	label = gtk_label_new(_("Key"));
@@ -695,26 +698,29 @@ static void init_bdef(MainInfo *min, P_Buttons *page, BDef *def, const gchar *la
 	def->key = gtk_entry_new();
 	gtk_entry_set_editable(GTK_ENTRY(def->key), FALSE);
 	gtk_object_set_data(GTK_OBJECT(def->key), "face", GUINT_TO_POINTER(face));
-	gtk_signal_connect(GTK_OBJECT(def->key), "key_press_event", GTK_SIGNAL_FUNC(evt_key_press), page);
+	g_signal_connect(G_OBJECT(def->key), "key_press_event", G_CALLBACK(evt_key_press), page);
 	gtk_table_attach(GTK_TABLE(table), def->key, 1, 2, 2, 3,  GTK_EXPAND|GTK_FILL,0,0,0);
 	btn = gtk_button_new_with_label(_("Clr"));
 	gtk_object_set_data(GTK_OBJECT(btn), "face", GUINT_TO_POINTER(face));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_key_clear_clicked), page);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_key_clear_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 2, 3,  GTK_FILL,0,0,0);
 
 	label = gtk_label_new(_("Colors"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,  0,0,0,0);
 	hbox = gtk_hbox_new(FALSE, 0);
-	def->col_bg = od_emil_button_new();
-	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_bg), OD_EMIL_BUTTON_PRIMARY,   _("Background..."), NULL, NULL, GUINT_TO_POINTER(face));
-	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_bg), OD_EMIL_BUTTON_SECONDARY, _("Reset to Default"),  NULL, NULL, GUINT_TO_POINTER(face));
-	gtk_signal_connect(GTK_OBJECT(def->col_bg), "activate", GTK_SIGNAL_FUNC(evt_col_bg_clicked), page);
+	def->col_bg = od_multibutton_new();
+	od_multibutton_set_text(OD_MULTIBUTTON(def->col_bg), 0, _("Background..."),    NULL, NULL, GUINT_TO_POINTER(face));
+	od_multibutton_set_text(OD_MULTIBUTTON(def->col_bg), 1, _("Reset to Default"), NULL, NULL, GUINT_TO_POINTER(face));
+	g_object_set_data(G_OBJECT(def->col_bg), "face", GINT_TO_POINTER(0));
+	g_signal_connect(G_OBJECT(def->col_bg), "clicked", G_CALLBACK(evt_col_bg_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), def->col_bg, TRUE, TRUE, 0);
-	def->col_fg = od_emil_button_new();
-	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_fg), OD_EMIL_BUTTON_PRIMARY,   _("Foreground..."), NULL, NULL, GUINT_TO_POINTER(face));
-	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_fg), OD_EMIL_BUTTON_SECONDARY, _("Reset to Default"),  NULL, NULL, GUINT_TO_POINTER(face));
-	gtk_signal_connect(GTK_OBJECT(def->col_fg), "activate", GTK_SIGNAL_FUNC(evt_col_fg_clicked), page);
+	def->col_fg = od_multibutton_new();
+	od_multibutton_set_text(OD_MULTIBUTTON(def->col_fg), 0, _("Foreground..."),    NULL, NULL, GUINT_TO_POINTER(face));
+	od_multibutton_set_text(OD_MULTIBUTTON(def->col_fg), 1, _("Reset to Default"), NULL, NULL, GUINT_TO_POINTER(face));
+	g_object_set_data(G_OBJECT(def->col_fg), "face", GINT_TO_POINTER(0));
+	g_signal_connect(G_OBJECT(def->col_fg), "clicked", G_CALLBACK(evt_col_fg_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), def->col_fg, TRUE, TRUE, 0);
+
 	gtk_table_attach(GTK_TABLE(table), hbox, 1, 3, 3, 4,  GTK_EXPAND|GTK_FILL,0,0,0);
 	
 	gtk_container_add(GTK_CONTAINER(def->frame), table);
@@ -776,18 +782,18 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	label = gtk_label_new(_("Tooltip"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	page->dtooltip = gtk_entry_new_with_max_length(BTN_TOOLTIP_SIZE - 1);
-	gtk_signal_connect(GTK_OBJECT(page->dtooltip), "changed", GTK_SIGNAL_FUNC(evt_dtooltip_changed), page);
+	g_signal_connect(G_OBJECT(page->dtooltip), "changed", G_CALLBACK(evt_dtooltip_changed), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dtooltip, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(page->dvbox), hbox, FALSE, FALSE, 0);
 
 	frame = gtk_frame_new(_("Flags"));
 	hbox = gtk_hbox_new(FALSE, 0);
 	page->dnarrow = gtk_check_button_new_with_label(_("Narrow?"));
-	page->dnarsig = gtk_signal_connect(GTK_OBJECT(page->dnarrow), "clicked", GTK_SIGNAL_FUNC(evt_dnarrow_clicked), page);
+	page->dnarsig = g_signal_connect(G_OBJECT(page->dnarrow), "clicked", G_CALLBACK(evt_dnarrow_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dnarrow, TRUE, TRUE, 0);
 
 	page->dshowtip = gtk_check_button_new_with_label(_("Show Tooltip?"));
-	gtk_signal_connect(GTK_OBJECT(page->dshowtip), "clicked", GTK_SIGNAL_FUNC(evt_dshowtip_clicked), page);
+	g_signal_connect(G_OBJECT(page->dshowtip), "clicked", G_CALLBACK(evt_dshowtip_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dshowtip, TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
 
@@ -799,7 +805,7 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	{
 		page->bcmd[i] = gtk_button_new_with_label(_(blab[i]));
 		gtk_object_set_user_data(GTK_OBJECT(page->bcmd[i]), GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->bcmd[i]), "clicked", GTK_SIGNAL_FUNC(evt_bcmd_clicked), page);
+		g_signal_connect(G_OBJECT(page->bcmd[i]), "clicked", G_CALLBACK(evt_bcmd_clicked), page);
 		gtk_box_pack_start(GTK_BOX(hbox), page->bcmd[i], TRUE, TRUE, 0);
 	}
 	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 5);
@@ -815,7 +821,7 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 		else
 			page->rcmd[i] = gtk_button_new_with_label(_(rlab[i]));
 		gtk_object_set_user_data(GTK_OBJECT(page->rcmd[i]), GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->rcmd[i]), "clicked", GTK_SIGNAL_FUNC(evt_rcmd_clicked), page);
+		g_signal_connect(G_OBJECT(page->rcmd[i]), "clicked", G_CALLBACK(evt_rcmd_clicked), page);
 		expand = (i == 0) || (i == sizeof page->rcmd / sizeof page->rcmd[0] - 1);
 		gtk_box_pack_start(GTK_BOX(hbox), page->rcmd[i], expand, TRUE, expand ? 5 : 1);
 	}
