@@ -35,22 +35,21 @@ typedef struct {
 	GtkWidget	*vbox;			/* Root page container. Very standard. */
 
 	GtkWidget	*kframe;		/* Keyboard frame. */
-
-	GtkWidget	*kscwin;		/* Scrolling window for keyboard shortcuts. */
-	GtkWidget	*kclist;		/* Actual list holding the keyboard shortcuts. */
+	GtkListStore	*kstore;		/* Stores keyboard shortcuts. */
+	GtkWidget	*kview;			/* A tree view for the keyboard shortcuts. */
 	GtkWidget	*kdtable;		/* Key definition table. */
 	GtkWidget	*kdkey;			/* Definition key name (entry). */
 	GtkWidget	*kdcmd;			/* Definition cmdsequence (entry). */
 	GtkWidget	*kdcmdpick;		/* Definition command pick button. */
-	GtkWidget	*khbox;			/* Horizontal box for keyboard add & del commands. */
 	GtkWidget	*kadd;			/* Keyboard add button. */
 	GtkWidget	*kdel;			/* Keyboard delete button. */
 
 	GtkWidget	*mframe;		/* Dirpane mouse button config frame. */
-	GtkWidget	*mscwin;
-	GtkWidget	*mclist;		/* List of defined mouse buttons. */
+	GtkListStore	*mstore;		/* Stores mouse button configs. */
+	GtkWidget	*mview;			/* A tree view for mouse button settings. */
+	GtkWidget	*mscwin;		/* Scrolling window (obsolete). */
 	GtkWidget	*mdtable;		/* Definition table. */
-	GtkWidget	*mdmenu;		/* Option menu showing mouse button for mapping. */
+	GtkWidget	*mdcombo;		/* Combobox showing mouse button for mapping. */
 	GtkWidget	*mdcmd;
 	GtkWidget	*mdcmdpick;
 	GtkWidget	*mhbox;
@@ -58,60 +57,43 @@ typedef struct {
 
 	GdkPixmap	*mpix[5];		/* Mouse icon pixmaps and masks. */
 	GdkBitmap	*mmsk[5];
+	GdkPixbuf	*mpbuf[5];		/* Mouse icon pixbufs. */
 
 	GtkWidget	*cmccmd;		/* Entry widget for Click-M-Click command. */
 	GtkWidget	*cmcdelay;		/* HScale widget for cmc delay. */
-	
+
 	GtkWidget	*pathcmd;		/* Entry widget for path-rmb command. */
 
 	GtkWidget	*nonumlock;		/* Ignore numlock check button. */
 
 	MainInfo	*min;			/* This is handy sometimes. */
-	gboolean	modified;		/* Indicates that a change has been made. */
 	CtrlInfo	*ctrlinfo;		/* The control info we're editing. */
-	CtrlKey		*curr_key;		/* Current key (if selected). */
-	gint		curr_key_row;		/* Row of current key in list. */
-	CtrlMouse	*curr_mouse;		/* Current mouse button binding (if any). */
-	gint		curr_mouse_row;		/* List row number for currently selected mouse row. */
+
+	gboolean	modified;		/* Indicates that a change has been made. */
 } P_Controls;
 
+enum {
+	KEY_COLUMN_KEYNAME = 0,
+	KEY_COLUMN_CMDSEQ,
+	KEY_COLUMN_KEY,
+
+	KEY_COLUMN_COUNT
+};
+
+enum {
+	MOUSE_COLUMN_ICON = 0,
+	MOUSE_COLUMN_BUTTON,
+	MOUSE_COLUMN_STATE,
+	MOUSE_COLUMN_CMDSEQ,
+	MOUSE_COLUMN_MOUSE,
+
+	MOUSE_COLUMN_COUNT
+};
+
 static P_Controls	the_page;
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-03-16 -	Set key definition widgets to non-idle state. */
-static void set_key_widgets(P_Controls *page, CtrlKey *key)
-{
-	if(page != NULL)
-	{
-
-		if(key != NULL)
-		{
-			const gchar	*cmdseq;
-
-			gtk_entry_set_text(GTK_ENTRY(page->kdkey), ctrl_key_get_keyname(key));
-			if((cmdseq = ctrl_key_get_cmdseq(key)) != NULL)
-				gtk_entry_set_text(GTK_ENTRY(page->kdcmd), cmdseq);
-			gtk_widget_set_sensitive(page->kdtable, TRUE);
-			gtk_widget_set_sensitive(page->kdel, TRUE);
-		}
-	}
-}
-
-/* 1999-06-13 -	Set state of mouse binding def. widgets to match selected binding. */
-static void set_mouse_widgets(P_Controls *page, CtrlMouse *mouse)
-{
-	if((page != NULL) && (mouse != NULL))
-	{
-		gtk_option_menu_set_history(GTK_OPTION_MENU(page->mdmenu), ctrl_mouse_get_button(mouse) - 1);
-		gtk_entry_set_text(GTK_ENTRY(page->mdcmd), ctrl_mouse_get_cmdseq(mouse));
-		gtk_widget_set_sensitive(page->mdtable, TRUE);
-		gtk_widget_set_sensitive(page->mdel, TRUE);
-	}
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1999-03-16 -	Reset the key definition widgets to their idle state. */
 static void reset_key_widgets(P_Controls *page)
 {
@@ -125,7 +107,7 @@ static void reset_key_widgets(P_Controls *page)
 /* 1999-06-13 -	Reset mouse editing widgets. */
 static void reset_mouse_widgets(P_Controls *page)
 {
-	gtk_option_menu_set_history(GTK_OPTION_MENU(page->mdmenu), 0);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(page->mdcombo), -1);
 	gtk_entry_set_text(GTK_ENTRY(page->mdcmd), "");
 	gtk_widget_set_sensitive(page->mdtable, FALSE);
 	gtk_widget_set_sensitive(page->madd, TRUE);
@@ -137,8 +119,6 @@ static void reset_widgets(P_Controls *page)
 {
 	const gchar	*cmd;
 
-	page->curr_key   = NULL;
-	page->curr_mouse = NULL;
 	reset_key_widgets(page);
 	reset_mouse_widgets(page);
 	cmd = ctrl_clickmclick_get_cmdseq(page->ctrlinfo);
@@ -151,89 +131,99 @@ static void reset_widgets(P_Controls *page)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-03-16 -	Populate keyboard clist. */
-static void populate_key_clist(P_Controls *page)
+static void list_store_append_key(P_Controls *page, const CtrlKey *key, GtkTreeIter *iter)
 {
-	gchar	*crow[] = { NULL, NULL };
-	GSList	*list, *iter;
+	GtkTreeIter	myiter;
 
-	gtk_clist_clear(GTK_CLIST(page->kclist));
-	gtk_clist_freeze(GTK_CLIST(page->kclist));
-	if((list = ctrl_keys_get_list(page->ctrlinfo)) != NULL)
-	{
-		gint	row;
+	/* If caller didn't provide an iter to use, use our local. */
+	if(iter == NULL)
+		iter = &myiter;
+	gtk_list_store_append(page->kstore, iter);
+	gtk_list_store_set(page->kstore, iter,
+			KEY_COLUMN_KEYNAME, ctrl_key_get_keyname(key),
+			KEY_COLUMN_CMDSEQ,  ctrl_key_get_cmdseq(key),
+			KEY_COLUMN_KEY,     key,
+			-1);
+}
 
-		for(iter = list; iter != NULL; iter = g_slist_next(iter))
-		{
-			crow[0] = (gchar *) ctrl_key_get_keyname(iter->data);
-			crow[1] = (gchar *) ctrl_key_get_cmdseq(iter->data);
-			row = gtk_clist_append(GTK_CLIST(page->kclist), crow);
-			gtk_clist_set_row_data(GTK_CLIST(page->kclist), row, iter->data);
-		}
-	}
-	gtk_clist_thaw(GTK_CLIST(page->kclist));
+/* 1999-03-16 -	Populate keyboard clist. */
+static void populate_key_list(P_Controls *page)
+{
+	const GSList	*iter;
+
+	gtk_list_store_clear(page->kstore);
+	for(iter = ctrl_keys_get_list(page->ctrlinfo); iter != NULL; iter = g_slist_next(iter))
+		list_store_append_key(page, iter->data, NULL);
+}
+
+static void list_store_append_mouse(P_Controls *page, const CtrlMouse *mouse, GtkTreeIter *iter)
+{
+	gchar   	*bname[] = { N_("Left"), N_("Middle"), N_("Right"), N_("Wheel Up"), N_("Wheel Down") };
+	guint		btn;
+	GtkTreeIter	myiter;
+
+	/* If caller didn't provide an iter to use, use our local. */
+	if(iter == NULL)
+		iter = &myiter;
+	gtk_list_store_append(page->mstore, iter);
+	btn = ctrl_mouse_get_button(mouse) - 1;
+	gtk_list_store_set(page->mstore, iter,
+			MOUSE_COLUMN_ICON,   page->mpbuf[btn],
+			MOUSE_COLUMN_BUTTON, _(bname[btn]),
+			MOUSE_COLUMN_STATE,  gtk_accelerator_name(0U, ctrl_mouse_get_state(mouse)),
+			MOUSE_COLUMN_CMDSEQ, ctrl_mouse_get_cmdseq(mouse),
+			MOUSE_COLUMN_MOUSE,  mouse,
+			-1);
 }
 
 /* 1999-06-13 -	Populate the mouse command mapping list. */
-static void populate_mouse_clist(P_Controls *page)
+static void populate_mouse_list(P_Controls *page)
 {
-	gchar	*bname[] = { N_("Left"), N_("Middle"), N_("Right"), N_("Wheel Up"), N_("Wheel Down") },
-		*crow[] = { NULL, NULL, NULL, NULL };
-	gint	row;
-	guint	btn;
-	GSList	*list, *iter;
+	const GSList	*iter;
 
-	gtk_clist_freeze(GTK_CLIST(page->mclist));
-	gtk_clist_clear(GTK_CLIST(page->mclist));
-	for(list = ctrl_mouse_get_list(page->ctrlinfo), iter = list; iter != NULL; iter = g_slist_next(iter))
-	{
-		btn = ctrl_mouse_get_button(iter->data) - 1;
-		crow[1] = _(bname[btn]);
-		crow[2] = gtk_accelerator_name(0U, ctrl_mouse_get_state(iter->data));
-		crow[3] = (gchar *) ctrl_mouse_get_cmdseq(iter->data);
-		row = gtk_clist_append(GTK_CLIST(page->mclist), crow);
-		gtk_clist_set_row_data(GTK_CLIST(page->mclist), row, iter->data);
-		gtk_clist_set_pixmap(GTK_CLIST(page->mclist), row, 0, page->mpix[btn], page->mmsk[btn]);
-	}
-	gtk_clist_thaw(GTK_CLIST(page->mclist));
+	gtk_list_store_clear(page->mstore);
+	for(iter = ctrl_mouse_get_list(page->ctrlinfo); iter != NULL; iter = g_slist_next(iter))
+		list_store_append_mouse(page, iter->data, NULL);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-03-16 -	Keyboard clist selection signal handler. */
-static void evt_kclist_select(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
+/* 2009-03-14 -	Keyboard list selection changed. Updated widgetry for editing. */
+static void evt_kselection_changed(GtkTreeSelection *ts, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeModel	*model;
+	GtkTreeIter	iter;
+	CtrlKey		*key = NULL;
 
-	set_key_widgets(page, gtk_clist_get_row_data(GTK_CLIST(page->kclist), row));
-	page->curr_key = gtk_clist_get_row_data(GTK_CLIST(page->kclist), row);
-	page->curr_key_row = row;
+	if(gtk_tree_selection_get_selected(ts, &model, &iter))
+		gtk_tree_model_get(model, &iter, KEY_COLUMN_KEY, &key, -1);
+	gtk_entry_set_text(GTK_ENTRY(page->kdkey), key != NULL ? ctrl_key_get_keyname(key) : "");
+	gtk_entry_set_text(GTK_ENTRY(page->kdcmd), key != NULL ? ctrl_key_get_cmdseq(key) : "");
+	gtk_widget_set_sensitive(page->kdtable, key != NULL);
+	gtk_widget_set_sensitive(page->kdel, key != NULL);
 }
 
-/* 1999-03-16 -	Keyboard clist unselection handler. */
-static void evt_kclist_unselect(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
-{
-	P_Controls	*page = user;
-
-	page->curr_key = NULL;
-	page->curr_key_row = -1;
-	reset_key_widgets(page);
-}
-
-/* 1999-03-16 -	User pressed something in the key definition entry. Convert to name and set it. */
+/* 1999-03-16 -	User pressed something in the key definition entry. Convert to name and set it.
+** 2009-03-14 -	Adapted for new GTK+ 2.0 list selection management. 10 years is a good interval. :)
+ */
 static gint evt_kkey_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeModel	*model;
+	GtkTreeIter	iter;
+	CtrlKey		*key = NULL;
 
-	if((page != NULL) && (page->curr_key != NULL))
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->kview)), &model, &iter))
 	{
 		gchar	*keyname = gtk_accelerator_name(evt->keyval, evt->state);
 
-		page->modified = TRUE;
-		ctrl_key_set_keyname(page->ctrlinfo, page->curr_key, keyname);
-		gtk_clist_set_text(GTK_CLIST(page->kclist), page->curr_key_row, 0, keyname);
-		set_key_widgets(page, page->curr_key);
+		gtk_tree_model_get(model, &iter, 2, &key, -1);
+		ctrl_key_set_keyname(page->ctrlinfo, key, keyname);
+		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, keyname, -1);
+		gtk_entry_set_text(GTK_ENTRY(page->kdkey), keyname);
 		gtk_signal_emit_stop_by_name(GTK_OBJECT(wid), "key_press_event");
+		page->modified = TRUE;
 	}
 	return TRUE;
 }
@@ -242,16 +232,20 @@ static gint evt_kkey_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 static gint evt_kcmd_changed(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeModel	*model;
+	GtkTreeIter	iter;
+	CtrlKey		*key = NULL;
 
-	if((page != NULL) && (page->curr_key != NULL))
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->kview)), &model, &iter))
 	{
-		gchar	*cmdseq = gtk_entry_get_text(GTK_ENTRY(wid));
+		const gchar	*cmdseq = gtk_entry_get_text(GTK_ENTRY(wid));
 
-		if(!ctrl_key_has_cmdseq(page->ctrlinfo, page->curr_key, cmdseq))
+		gtk_tree_model_get(model, &iter, 2, &key, -1);
+		if(!ctrl_key_has_cmdseq(page->ctrlinfo, key, cmdseq))
 		{
+			ctrl_key_set_cmdseq(page->ctrlinfo, key, cmdseq);
+			gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, cmdseq, -1);
 			page->modified = TRUE;
-			ctrl_key_set_cmdseq(page->ctrlinfo, page->curr_key, cmdseq);
-			gtk_clist_set_text(GTK_CLIST(page->kclist), page->curr_key_row, 1, cmdseq);
 		}
 	}
 	return TRUE;
@@ -278,22 +272,25 @@ static gint evt_kcmdpick_clicked(GtkWidget *wid, gpointer user)
 static gint evt_kadd_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	CtrlKey		*key;
+	GtkTreeIter	iter;
+	GtkTreePath	*path;
 
-	if(page != NULL)
-	{
-		gchar	*crow[] = { NULL, NULL };
-		gint	row;
-		CtrlKey	*key;
+	key = ctrl_key_add_unique(page->ctrlinfo);
+	ctrl_key_set_cmdseq(page->ctrlinfo, key, "");
+	gtk_list_store_append(page->kstore, &iter);
+	gtk_list_store_set(page->kstore, &iter,
+				0, ctrl_key_get_keyname(key),
+				1, ctrl_key_get_cmdseq(key),
+				2, key,
+				-1);
+	gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->kview)), &iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(page->kstore), &iter);
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(page->kview), path, NULL, TRUE, 0.5f, 0.0f);
+	gtk_tree_path_free(path);
+	gtk_widget_grab_focus(page->kdkey);
+	page->modified = TRUE;
 
-		key = ctrl_key_add_unique(page->ctrlinfo);
-		crow[0] = (gchar *) ctrl_key_get_keyname(key);
-		crow[1] = (gchar *) ctrl_key_get_cmdseq(key);
-		row = gtk_clist_append(GTK_CLIST(page->kclist), crow);
-		gtk_clist_set_row_data(GTK_CLIST(page->kclist), row, key);
-		gtk_clist_select_row(GTK_CLIST(page->kclist), row, -1);
-		gtk_widget_grab_focus(page->kdkey);
-		page->modified = TRUE;
-	}
 	return TRUE;
 }
 
@@ -301,16 +298,15 @@ static gint evt_kadd_clicked(GtkWidget *wid, gpointer user)
 static gint evt_kdel_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeModel	*model;
+	GtkTreeIter	iter;
+	CtrlKey		*key = NULL;
 
-	if((page != NULL) && (page->curr_key != NULL))
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->kview)), &model, &iter))
 	{
-		gint	row = page->curr_key_row;
-
-		ctrl_key_remove(page->ctrlinfo, page->curr_key);
-		gtk_clist_remove(GTK_CLIST(page->kclist), row);
-		while(row >= GTK_CLIST(page->kclist)->rows)
-			row--;
-		gtk_clist_select_row(GTK_CLIST(page->kclist), row, -1);
+		gtk_tree_model_get(model, &iter, 2, &key, -1);
+		ctrl_key_remove(page->ctrlinfo, key);
+		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 		page->modified = TRUE;
 	}
 	return TRUE;
@@ -318,50 +314,69 @@ static gint evt_kdel_clicked(GtkWidget *wid, gpointer user)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-06-13 -	A mouse binding row has been selected. */
-static void evt_mclist_select_row(GtkWidget *wid, gint row, gint col, GdkEventButton *evt, gpointer user)
+/* 2009-03-14 -	The selection in the mouse binding list has changed. Deal with it. */
+static void evt_mselection_changed(GtkTreeSelection *ts, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeModel	*model;
+	GtkTreeIter	iter;
+	CtrlMouse	*mouse = NULL;
 
-	set_mouse_widgets(page, gtk_clist_get_row_data(GTK_CLIST(wid), row));
-	page->curr_mouse     = gtk_clist_get_row_data(GTK_CLIST(wid), row);
-	page->curr_mouse_row = row;
+	if(gtk_tree_selection_get_selected(ts, &model, &iter))
+		gtk_tree_model_get(model, &iter, MOUSE_COLUMN_MOUSE, &mouse, -1);
+	gtk_entry_set_text(GTK_ENTRY(page->mdcmd), mouse != NULL ? ctrl_mouse_get_cmdseq(mouse) : "");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(page->mdcombo), ctrl_mouse_get_button(mouse) - 1);
+	gtk_widget_set_sensitive(page->mdtable, mouse != NULL);
+	gtk_widget_set_sensitive(page->mdel, mouse != NULL);
 }
 
-/* 1999-06-13 -	We lost the selected row in the mouse CList. Clear other widgets. */
-static void evt_mclist_unselect_row(GtkWidget *wid, gint row, gint col, GdkEventButton *evt, gpointer user)
+static void evt_mbutton_changed(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
-
-	page->curr_mouse = NULL;
-	page->curr_mouse_row = -1;
-	reset_mouse_widgets(page);
-}
-
-static void evt_mbutton_activated(GtkWidget *wid, gpointer user)
-{
-	P_Controls	*page = user;
-	gchar		*blabel[] = { N_("Left"), N_("Middle"), N_("Right"), N_("Wheel Up"), N_("Wheel Down") };
+	GtkTreeSelection *ts;
+	GtkTreeIter	ti, biter;
+	CtrlMouse	*mouse;
 	guint		btn;
 
-	btn = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(wid)));
-	ctrl_mouse_set_button(page->curr_mouse, btn + 1);
-	gtk_clist_set_pixmap(GTK_CLIST(page->mclist), page->curr_mouse_row, 0, page->mpix[btn], page->mmsk[btn]);
-	gtk_clist_set_text(GTK_CLIST(page->mclist), page->curr_mouse_row, 1, _(blabel[btn]));
+	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview));
+	if(ts == NULL || !gtk_tree_selection_get_selected(ts, NULL, &ti))
+		return;
+	gtk_tree_model_get(GTK_TREE_MODEL(page->mstore), &ti, MOUSE_COLUMN_MOUSE, &mouse, -1);
+	btn = gtk_combo_box_get_active(GTK_COMBO_BOX(page->mdcombo));
+	ctrl_mouse_set_button(mouse, btn + 1);
+
+	/* Almost-too-clever: grab the translated mouse button label from the GtkComboBox's model. */
+	if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(page->mdcombo), &biter))
+	{
+		gchar	*button;
+
+		gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(page->mdcombo)), &biter, 1, &button, -1);
+
+		gtk_list_store_set(GTK_LIST_STORE(page->mstore), &ti,
+					MOUSE_COLUMN_ICON, page->mpbuf[btn],
+					MOUSE_COLUMN_BUTTON, button,
+					-1);
+		g_free(button);
+	}
 }
 
 /* 1999-06-13 -	User just hit the "Edit Modifiers..." button. Bring up a little dialog. */
 static void evt_mstate_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeSelection *ts;
+	GtkTreeIter	ti;
+	CtrlMouse	*mouse;
 	Dialog		*dlg;
 	guint		i, x, y,
 			mask[] = { GDK_SHIFT_MASK, GDK_CONTROL_MASK, GDK_MOD1_MASK, GDK_MOD2_MASK,
 				   GDK_MOD3_MASK, GDK_MOD4_MASK, GDK_MOD5_MASK };
 	GtkWidget	*table, *label, *check[sizeof mask / sizeof mask[0]];
 
-	if((page == NULL) || (page->curr_mouse == NULL))
+	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview));
+	if(ts == NULL || !gtk_tree_selection_get_selected(ts, NULL, &ti))
 		return;
+	gtk_tree_model_get(GTK_TREE_MODEL(page->mstore), &ti, MOUSE_COLUMN_MOUSE, &mouse, -1);
 
 	table = gtk_table_new(1 + (sizeof mask / sizeof *mask) / 2 + 1, 2, FALSE);
 	label = gtk_label_new(_("The following modifier key(s) must\n"
@@ -371,7 +386,7 @@ static void evt_mstate_clicked(GtkWidget *wid, gpointer user)
 	for(i = 0, x = 0, y = 1; i < sizeof mask / sizeof mask[0]; i++, y++)
 	{
 		check[i] = gtk_check_button_new_with_label(gtk_accelerator_name(0, mask[i]));
-		if(ctrl_mouse_get_state(page->curr_mouse) & mask[i])
+		if(ctrl_mouse_get_state(mouse) & mask[i])
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check[i]), TRUE);
 		gtk_table_attach(GTK_TABLE(table), check[i], x, x + 1, y, y + 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 		if(i == (sizeof mask / sizeof *mask) / 2)
@@ -387,8 +402,8 @@ static void evt_mstate_clicked(GtkWidget *wid, gpointer user)
 			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check[i])))
 				ns |= mask[i];
 		}
-		ctrl_mouse_set_state(page->curr_mouse, ns);
-		gtk_clist_set_text(GTK_CLIST(page->mclist), page->curr_mouse_row, 2, gtk_accelerator_name(0, ns));
+		ctrl_mouse_set_state(mouse, ns);
+		gtk_list_store_set(GTK_LIST_STORE(page->mstore), &ti, MOUSE_COLUMN_STATE, gtk_accelerator_name(0, ns), -1);
 		page->modified = TRUE;
 	}
 	dlg_dialog_sync_destroy(dlg);
@@ -398,17 +413,20 @@ static void evt_mstate_clicked(GtkWidget *wid, gpointer user)
 static void evt_mcmd_changed(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeSelection *ts;
+	GtkTreeIter	ti;
+	CtrlMouse	*mouse;
+	const gchar	*cmdseq;
 
-	if((page != NULL) && (page->curr_mouse != NULL))
+	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview));
+	if(ts == NULL || !gtk_tree_selection_get_selected(ts, NULL, &ti))
+		return;
+	gtk_tree_model_get(GTK_TREE_MODEL(page->mstore), &ti, MOUSE_COLUMN_MOUSE, &mouse, -1);
+	if((cmdseq = gtk_entry_get_text(GTK_ENTRY(page->mdcmd))) != NULL)
 	{
-		gchar	*cmdseq;
-
-		if((cmdseq = gtk_entry_get_text(GTK_ENTRY(page->mdcmd))) != NULL)
-		{
-			ctrl_mouse_set_cmdseq(page->curr_mouse, cmdseq);
-			gtk_clist_set_text(GTK_CLIST(page->mclist), page->curr_mouse_row, 3, cmdseq);
-			page->modified = TRUE;
-		}
+		ctrl_mouse_set_cmdseq(mouse, cmdseq);
+		gtk_list_store_set(page->mstore, &ti, MOUSE_COLUMN_CMDSEQ, cmdseq, -1);
+		page->modified = TRUE;
 	}
 }
 
@@ -416,27 +434,28 @@ static void evt_mcmd_changed(GtkWidget *wid, gpointer user)
 static void evt_mcmdpick_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	const gchar	*cmd;
 
-	if((page != NULL) && (page->curr_mouse != NULL))
-	{
-		const gchar	*cmd;
-
-		if((cmd = csq_dialog_sync_new_wait(page->min, ccs_get_current())) != NULL)
-			gtk_entry_set_text(GTK_ENTRY(page->mdcmd), cmd);
-	}
+	if((cmd = csq_dialog_sync_new_wait(page->min, ccs_get_current())) != NULL)
+		gtk_entry_set_text(GTK_ENTRY(page->mdcmd), cmd);
 }
 
 /* 1999-06-20 -	User clicked the "Add" button for mouse bindings, so add one. */
 static void evt_madd_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	CtrlMouse	*mouse;
+	GtkTreeIter	iter;
+	GtkTreePath	*path;
 
-	if(page != NULL)
+	mouse = ctrl_mouse_add(page->ctrlinfo, 1U, 0U, "");
+	if(mouse != NULL)
 	{
-		ctrl_mouse_add(page->ctrlinfo, 1U, 0U, "");
-		populate_mouse_clist(page);
-		gtk_clist_select_row(GTK_CLIST(page->mclist), 0, -1);
-		gtk_clist_moveto(GTK_CLIST(page->mclist), 0, -1, 0.0f, -1.0f);
+		list_store_append_mouse(page, mouse, &iter);
+		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview)), &iter);
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(page->mstore), &iter);
+		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(page->mview), path, NULL, TRUE, 0.5f, 0.0f);
+		gtk_tree_path_free(path);
 		page->modified = TRUE;
 	}
 }
@@ -445,19 +464,15 @@ static void evt_madd_clicked(GtkWidget *wid, gpointer user)
 static void evt_mdel_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Controls	*page = user;
+	GtkTreeModel	*model;
+	GtkTreeIter	iter;
+	CtrlMouse	*mouse;
 
-	if((page != NULL) && (page->curr_mouse != NULL))
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview)), &model, &iter))
 	{
-		gint	row = page->curr_mouse_row, nr;
-
-		ctrl_mouse_remove(page->ctrlinfo, page->curr_mouse);
-		gtk_clist_unselect_row(GTK_CLIST(page->mclist), row, 0);
-		page->curr_mouse = NULL;
-		page->curr_mouse_row = -1;
-		populate_mouse_clist(page);
-		nr = (row >= GTK_CLIST(page->mclist)->rows) ? GTK_CLIST(page->mclist)->rows - 1 : row;
-		gtk_clist_select_row(GTK_CLIST(page->mclist), nr, 0);
-		gtk_clist_moveto(GTK_CLIST(page->mclist), nr, -1, 1.0f, -1.0f);
+		gtk_tree_model_get(model, &iter, MOUSE_COLUMN_MOUSE, &mouse, -1);
+		ctrl_mouse_remove(page->ctrlinfo, mouse);
+		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 		page->modified = TRUE;
 	}
 }
@@ -548,35 +563,50 @@ static void build_mouse_menu(P_Controls *page)
 {
 	gchar		**gfx[]  = { icon_mouse1_xpm, icon_mouse2_xpm, icon_mouse3_xpm, icon_mouse4_xpm, icon_mouse5_xpm },
 			*label[] = { N_("Left"), N_("Middle"), N_("Right"), N_("Wheel Up"), N_("Wheel Down") };
-	GtkWidget	*menu, *item, *ihbox, *pmap, *lab;
 	guint		i;
+	GtkListStore	*mmodel;
+	GtkCellRenderer	*cr;
 
-	menu = gtk_menu_new();
-	for(i = 0; i < 5; i++)
+	for(i = 0; i < sizeof gfx / sizeof *gfx; i++)
+		page->mpbuf[i] = gdk_pixbuf_new_from_xpm_data((const gchar **) gfx[i]);
+
+	/* Now build a full TreeModel for the GtkOptionMenu replacement. Pfew!
+	** NOTE: Unlike key and mouse true models, we don't store the button index
+	** in the model; instead we just use get/set active() on the GtkComboBox.
+	 */
+	mmodel = gtk_list_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
+	for(i = 0; i < sizeof gfx / sizeof *gfx; i++)
 	{
-		item  = gtk_menu_item_new();
-		ihbox = gtk_hbox_new(FALSE, 0);
-		page->mpix[i] = gdk_pixmap_colormap_create_from_xpm_d(NULL, gtk_widget_get_default_colormap(), &page->mmsk[i], NULL, gfx[i]);
-		pmap = gtk_pixmap_new(page->mpix[i], page->mmsk[i]);
-		gtk_box_pack_start(GTK_BOX(ihbox), pmap, FALSE, FALSE, 0);
-		lab = gtk_label_new(_(label[i]));
-		gtk_box_pack_start(GTK_BOX(ihbox), lab, FALSE, FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(item), ihbox);
-		gtk_menu_append(GTK_MENU(menu), item);
-		gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_mbutton_activated), page);
-		gtk_widget_show_all(item);
+		GtkTreeIter	iter;
+
+		gtk_list_store_append(mmodel, &iter);
+		gtk_list_store_set(mmodel, &iter, 0, page->mpbuf[i], 1, _(label[i]), -1);
 	}
-	page->mdmenu = gtk_option_menu_new();
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(page->mdmenu), menu);
-	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdmenu, 1, 2, 0, 1,  GTK_FILL|GTK_EXPAND,0,0,0);
+	page->mdcombo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(mmodel));
+	cr = gtk_cell_renderer_pixbuf_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(page->mdcombo), cr, FALSE);
+	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(page->mdcombo), cr,
+	                                "pixbuf", 0,
+	                                NULL);
+
+	cr = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(page->mdcombo), cr, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT(page->mdcombo), cr,
+	                                "text", 1,
+	                                NULL);
+
+	g_signal_connect(G_OBJECT(page->mdcombo), "changed", G_CALLBACK(evt_mbutton_changed), page);
+	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcombo, 1, 2, 0, 1,  GTK_FILL|GTK_EXPAND,0,0,0);
 }
 
 static GtkWidget * cct_init(MainInfo *min, gchar **name)
 {
 	P_Controls	*page = &the_page;
-	GtkWidget	*vbox, *frame, *label, *wid, *hbox;
+	GtkWidget	*vbox, *frame, *label, *wid, *hbox, *scwin;
 	GtkObject	*adj;
+	GtkCellRenderer	*cr;
+	GtkTreeViewColumn *vc;
+	GtkTreeSelection *ts;
 
 	if(name == NULL)
 		return NULL;
@@ -589,79 +619,95 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 
 	page->kframe = gtk_frame_new(_("Global Keyboard Shortcuts"));
 	vbox = gtk_vbox_new(FALSE, 0);
-	page->kscwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->kscwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	page->kclist = gtk_clist_new(2);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(page->kclist), 0, TRUE);
-	gtk_clist_set_column_min_width(GTK_CLIST(page->kclist), 0, 80);
-	gtk_signal_connect(GTK_OBJECT(page->kclist), "select_row", GTK_SIGNAL_FUNC(evt_kclist_select), page);
-	gtk_signal_connect(GTK_OBJECT(page->kclist), "unselect_row", GTK_SIGNAL_FUNC(evt_kclist_unselect), page);
-	gtk_container_add(GTK_CONTAINER(page->kscwin), page->kclist);
-	gtk_box_pack_start(GTK_BOX(vbox), page->kscwin, TRUE, TRUE, 0);
+	page->kstore = gtk_list_store_new(KEY_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+	page->kview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(page->kstore));
+	cr = gtk_cell_renderer_text_new();
+	vc = gtk_tree_view_column_new_with_attributes("(Key)", cr, "text", 0, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->kview), vc);
+	vc = gtk_tree_view_column_new_with_attributes("(Command)", cr, "text", 1, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->kview), vc);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(page->kview), FALSE);
+	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->kview));
+	gtk_tree_selection_set_mode(ts, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(ts), "changed", G_CALLBACK(evt_kselection_changed), page);
+	scwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scwin), page->kview);
+	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
+
 	page->kdtable = gtk_table_new(2, 3, FALSE);
 	label = gtk_label_new(_("Key"));
 	gtk_table_attach(GTK_TABLE(page->kdtable), label, 0, 1, 0, 1,  0,0,0,0);
 	page->kdkey = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(page->kdkey), "key_press_event", GTK_SIGNAL_FUNC(evt_kkey_press), page);
+	g_signal_connect(G_OBJECT(page->kdkey), "key_press_event", G_CALLBACK(evt_kkey_press), page);
 	gtk_entry_set_editable(GTK_ENTRY(page->kdkey), FALSE);
 	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdkey, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(page->kdtable), label, 0, 1, 1, 2,  0,0,0,0);
 	page->kdcmd = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(page->kdcmd), "changed", GTK_SIGNAL_FUNC(evt_kcmd_changed), page);
+	g_signal_connect(G_OBJECT(page->kdcmd), "changed", G_CALLBACK(evt_kcmd_changed), page);
 	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdcmd, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
 	page->kdcmdpick = gui_details_button_new(min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(page->kdcmdpick), "clicked", GTK_SIGNAL_FUNC(evt_kcmdpick_clicked), page);
+	g_signal_connect(G_OBJECT(page->kdcmdpick), "clicked", G_CALLBACK(evt_kcmdpick_clicked), page);
 	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdcmdpick, 2, 3, 1, 2,  0,0,0,0);
 	gtk_box_pack_start(GTK_BOX(vbox), page->kdtable, FALSE, FALSE, 0);
-	page->khbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_hbox_new(FALSE, 0);
 	page->kadd = gtk_button_new_with_label(_("Add"));
-	gtk_signal_connect(GTK_OBJECT(page->kadd), "clicked", GTK_SIGNAL_FUNC(evt_kadd_clicked), page);
-	gtk_box_pack_start(GTK_BOX(page->khbox), page->kadd, TRUE, TRUE, 5);
+	g_signal_connect(G_OBJECT(page->kadd), "clicked", G_CALLBACK(evt_kadd_clicked), page);
+	gtk_box_pack_start(GTK_BOX(hbox), page->kadd, TRUE, TRUE, 5);
 	page->kdel = gtk_button_new_with_label(_("Delete"));
-	gtk_signal_connect(GTK_OBJECT(page->kdel), "clicked", GTK_SIGNAL_FUNC(evt_kdel_clicked), page);
-	gtk_box_pack_start(GTK_BOX(page->khbox), page->kdel, TRUE, TRUE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), page->khbox, FALSE, FALSE, 5);
+	g_signal_connect(G_OBJECT(page->kdel), "clicked", G_CALLBACK(evt_kdel_clicked), page);
+	gtk_box_pack_start(GTK_BOX(hbox), page->kdel, TRUE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(page->kframe), vbox);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->kframe, TRUE, TRUE, 0);
 
 	page->mframe = gtk_frame_new(_("Dirpane Mouse Buttons"));
 	vbox = gtk_vbox_new(FALSE, 0);
-	page->mscwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->mscwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	page->mclist = gtk_clist_new(4);
-	gtk_clist_set_column_width(GTK_CLIST(page->mclist), 0, 16);
-	gtk_clist_set_column_justification(GTK_CLIST(page->mclist), 0, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_width(GTK_CLIST(page->mclist), 1, 80);
-	gtk_clist_set_column_justification(GTK_CLIST(page->mclist), 1, GTK_JUSTIFY_LEFT);
-	gtk_clist_set_column_width(GTK_CLIST(page->mclist), 2, 160);
-	gtk_clist_set_column_justification(GTK_CLIST(page->mclist), 2, GTK_JUSTIFY_LEFT);
-	gtk_signal_connect(GTK_OBJECT(page->mclist), "select_row", GTK_SIGNAL_FUNC(evt_mclist_select_row), page);
-	gtk_signal_connect(GTK_OBJECT(page->mclist), "unselect_row", GTK_SIGNAL_FUNC(evt_mclist_unselect_row), page);
-	gtk_container_add(GTK_CONTAINER(page->mscwin), page->mclist);
-	gtk_box_pack_start(GTK_BOX(vbox), page->mscwin, TRUE, TRUE, 0);
+
+	page->mstore = gtk_list_store_new(MOUSE_COLUMN_COUNT, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+	page->mview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(page->mstore));
+	cr = gtk_cell_renderer_pixbuf_new();
+	vc = gtk_tree_view_column_new_with_attributes("(BIcon)", cr, "pixbuf", 0, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->mview), vc);
+	cr = gtk_cell_renderer_text_new();
+	vc = gtk_tree_view_column_new_with_attributes("(Button)", cr, "text", 1, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->mview), vc);
+	vc = gtk_tree_view_column_new_with_attributes("(Modifier)", cr, "text", 2, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->mview), vc);
+	vc = gtk_tree_view_column_new_with_attributes("(Command)", cr, "text", 3, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->mview), vc);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(page->mview), FALSE);
+	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview));
+	gtk_tree_selection_set_mode(ts, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(ts), "changed", G_CALLBACK(evt_mselection_changed), page);
+	scwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scwin), page->mview);
+	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
+
 	page->mdtable = gtk_table_new(2, 4, FALSE);
 	label = gtk_label_new(_("Button"));
 	gtk_table_attach(GTK_TABLE(page->mdtable), label, 0, 1, 0, 1,  0,0,0,0);
 	build_mouse_menu(page);
 	wid = gtk_button_new_with_label(_("Edit Modifiers..."));
-	gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_mstate_clicked), page);
+	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_mstate_clicked), page);
 	gtk_table_attach(GTK_TABLE(page->mdtable), wid, 2, 3, 0, 1, GTK_FILL|GTK_EXPAND,0,5,0);
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(page->mdtable), label, 0, 1, 1, 2,  0,0,0,0);
 	page->mdcmd = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(page->mdcmd), "changed", GTK_SIGNAL_FUNC(evt_mcmd_changed), page);
+	g_signal_connect(G_OBJECT(page->mdcmd), "changed", G_CALLBACK(evt_mcmd_changed), page);
 	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcmd, 1, 3, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
 	page->mdcmdpick = gui_details_button_new(min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(page->mdcmdpick), "clicked", GTK_SIGNAL_FUNC(evt_mcmdpick_clicked), page);
+	g_signal_connect(G_OBJECT(page->mdcmdpick), "clicked", G_CALLBACK(evt_mcmdpick_clicked), page);
 	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcmdpick, 3, 4, 1, 2,  0,0,0,0);
 	gtk_box_pack_start(GTK_BOX(vbox), page->mdtable, FALSE, FALSE, 0);
 	page->mhbox = gtk_hbox_new(FALSE, 0);
 	page->madd = gtk_button_new_with_label(_("Add"));
-	gtk_signal_connect(GTK_OBJECT(page->madd), "clicked", GTK_SIGNAL_FUNC(evt_madd_clicked), page);
+	g_signal_connect(G_OBJECT(page->madd), "clicked", G_CALLBACK(evt_madd_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->mhbox), page->madd, TRUE, TRUE, 5);
 	page->mdel = gtk_button_new_with_label(_("Delete"));
-	gtk_signal_connect(GTK_OBJECT(page->mdel), "clicked", GTK_SIGNAL_FUNC(evt_mdel_clicked), page);
+	g_signal_connect(G_OBJECT(page->mdel), "clicked", G_CALLBACK(evt_mdel_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->mhbox), page->mdel, TRUE, TRUE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox), page->mhbox, FALSE, FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(page->mframe), vbox);
@@ -672,15 +718,15 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	label = gtk_label_new(_("Command"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	page->cmccmd = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(page->cmccmd), "changed", GTK_SIGNAL_FUNC(evt_clickmclick_changed), page);
+	g_signal_connect(G_OBJECT(page->cmccmd), "changed", G_CALLBACK(evt_clickmclick_changed), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->cmccmd, TRUE, TRUE, 0);
 	wid = gui_details_button_new(page->min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_clickmclick_pick_clicked), page);
+	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_clickmclick_pick_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), wid, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Time Limit"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 	adj = gtk_adjustment_new(0.400f, 0.100f, 1.5f, 0.1f, 0.1f, 0.0f);
-	gtk_signal_connect(adj, "value_changed", GTK_SIGNAL_FUNC(evt_clickmclick_delay_changed), page);
+	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_clickmclick_delay_changed), page);
 	page->cmcdelay = gtk_hscale_new(GTK_ADJUSTMENT(adj));
 	gtk_scale_set_value_pos(GTK_SCALE(page->cmcdelay), GTK_POS_RIGHT);
 	gtk_scale_set_digits(GTK_SCALE(page->cmcdelay), 3);
@@ -693,16 +739,16 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	label = gtk_label_new(_("Command"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	page->pathcmd = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(page->pathcmd), "changed", GTK_SIGNAL_FUNC(evt_pathcmd_changed), page);
+	g_signal_connect(G_OBJECT(page->pathcmd), "changed", G_CALLBACK(evt_pathcmd_changed), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->pathcmd, TRUE, TRUE, 0);
 	wid = gui_details_button_new(page->min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_pathcmd_pick_clicked), page);
+	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_pathcmd_pick_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), wid, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
 	gtk_box_pack_start(GTK_BOX(page->vbox), frame, FALSE, FALSE, 0);
-	
+
 	page->nonumlock = gtk_check_button_new_with_label(_("Ignore Num Lock For All Bindings?"));
-	gtk_signal_connect(GTK_OBJECT(page->nonumlock), "clicked", GTK_SIGNAL_FUNC(evt_nonumlock_clicked), page);
+	g_signal_connect(G_OBJECT(page->nonumlock), "clicked", G_CALLBACK(evt_nonumlock_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->nonumlock, FALSE, FALSE, 0);
 
 	gtk_widget_show_all(page->vbox);
@@ -715,8 +761,8 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 static void cct_update(MainInfo *min)
 {
 	the_page.ctrlinfo = ctrl_copy(min->cfg.ctrlinfo);
-	populate_key_clist(&the_page);
-	populate_mouse_clist(&the_page);
+	populate_key_list(&the_page);
+	populate_mouse_list(&the_page);
 	reset_widgets(&the_page);
 }
 
@@ -924,7 +970,7 @@ static void cct_load(MainInfo *min, const XmlNode *node)
 	}
 }
 
-/* ----------------------------------------------------------------------------------------- */	
+/* ----------------------------------------------------------------------------------------- */
 
 const CfgModule * cct_describe(MainInfo *min)
 {
