@@ -21,8 +21,6 @@
 
 #include <stdlib.h>
 
-#include "odemilbutton.h"
-
 #include "dirpane.h"
 #include "guiutil.h"
 #include "xmlutil.h"
@@ -30,6 +28,7 @@
 #include "strutil.h"
 #include "dpformat.h"
 #include "color_dialog.h"
+#include "odmultibutton.h"
 #include "window.h"
 
 #include "configure.h"
@@ -72,8 +71,7 @@ typedef struct {			/* Pane definition widgets. */
 	P_DirPane	*page;
 	DPFormat	edit;
 	guint		index;
-	gint		aclist_row,	/* The last and previous (to last) rows that were clicked in aclist. */
-			aclist_row2;
+	gint		aclist_row;	/* The last row that were clicked in aclist. */
 	gint		sclist_row;	/* Last clicked row in selected content list. */
 	MainInfo	*min;		/* Handy to have around. */
 } PDef;
@@ -84,7 +82,6 @@ typedef struct {
 	GtkWidget	*widget[DPCOL_NUM];
 
 	DPColors	edit;
-	DpCol		index;		/* This is just used when editing, to record which color is being edited. */
 } PColors;
 
 typedef struct {
@@ -237,7 +234,7 @@ static void set_pane_widgets(PDef *def)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(def->spos[(guint)  def->edit.sbar_pos]), TRUE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(def->mctrl[(guint) def->edit.ctrl_mode]), TRUE);
 
-	def->aclist_row = def->aclist_row2 = -1;
+	def->aclist_row = -1;
 	def->sclist_row = -1;
 }
 
@@ -247,7 +244,7 @@ static void set_color_widgets(PColors *col)
 	guint	i;
 
 	for(i = 0; i < sizeof col->widget / sizeof col->widget[0]; i++)
-		od_emil_button_set_colors(OD_EMIL_BUTTON(col->widget[i]), OD_EMIL_BUTTON_PRIMARY, NULL, &col->edit.color[i]);
+		gtk_color_button_set_color(GTK_COLOR_BUTTON(col->widget[i]), &col->edit.color[i]);
 }
 
 static void set_paning_widgets(PPaning *p)
@@ -276,7 +273,7 @@ static void set_paning_widgets(PPaning *p)
 static void set_history_widgets(PHistory *p)
 {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p->select), p->edit.select);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p->save),   p->edit.save);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p->save), p->edit.save);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -294,21 +291,14 @@ static void add_column(PDef *def, guint content)
 	}
 }
 
-/* 1999-05-15 -	User clicked a row in the available clist (leftmost). If double-click, add
-**		row to the selected (rightmost) list.
-*/
-static void evt_aclist_button_press(GtkWidget *wid, GdkEventButton *evt, gpointer user)
+/* 2008-07-28 -	Rewritten, now listens to a different signal and works a lot cleaner than before. */
+static void evt_aclist_select_row(GtkWidget *clist, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
 	PDef	*def = user;
-	gint	row;
 
-	if(gtk_clist_get_selection_info(GTK_CLIST(wid), (gint) evt->x, (gint) evt->y, &row, NULL))
-	{
-		if((row == def->aclist_row2) && (evt->type == GDK_2BUTTON_PRESS))
-			add_column(def, row);
-		def->aclist_row2 = def->aclist_row;
-		def->aclist_row = row;
-	}
+	if(row == def->aclist_row && evt->type == GDK_2BUTTON_PRESS)
+		add_column(def, row);
+	def->aclist_row = row;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -386,28 +376,28 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 			{
 				trb[i] = rb = gtk_radio_button_new_with_label(group, _(slab[i]));
 				gtk_object_set_data(GTK_OBJECT(rb), "type", GINT_TO_POINTER(stype[i]));
-				gtk_signal_connect(GTK_OBJECT(rb), "toggled", GTK_SIGNAL_FUNC(evt_size_type_toggled), edit);
+				g_signal_connect(G_OBJECT(rb), "toggled", G_CALLBACK(evt_size_type_toggled), edit);
 				gtk_table_attach(GTK_TABLE(table), rb, 0, 2, i, i + 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 				if(stype[i] == fmt->extra.size.type)
 					ti = i;
 				group = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
 			}
 			gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
-	
+
 			edit->extra.size.options = gtk_table_new(2, 2, FALSE);
 			edit->extra.size.ticks = gtk_check_button_new_with_label(_("Place Tick Every 3 Digits?"));
 			gtk_table_attach(GTK_TABLE(edit->extra.size.options), edit->extra.size.ticks, 0, 1, 0, 1,  0,0,0,0);
 			edit->extra.size.tick = gtk_entry_new_with_max_length(1);
-			gtk_widget_set_usize(edit->extra.size.tick, 48, -1);
+			gtk_entry_set_width_chars(GTK_ENTRY(edit->extra.size.tick), 1);
 			tickstr[0] = fmt->extra.size.tick;
 			gtk_entry_set_text(GTK_ENTRY(edit->extra.size.tick), tickstr);
 			gtk_table_attach(GTK_TABLE(edit->extra.size.options), edit->extra.size.tick, 1, 2, 0, 1,  GTK_SHRINK,0,0,0);
-			gtk_signal_connect(GTK_OBJECT(edit->extra.size.ticks), "toggled", GTK_SIGNAL_FUNC(evt_size_ticks_toggled), edit);
+			g_signal_connect(G_OBJECT(edit->extra.size.ticks), "toggled", G_CALLBACK(evt_size_ticks_toggled), edit);
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(edit->extra.size.ticks), fmt->extra.size.ticks);
 
 			label = gtk_label_new(_("Digits of Precision"));
 			gtk_table_attach(GTK_TABLE(edit->extra.size.options), label, 0, 1, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-			adj = gtk_adjustment_new(0.0, 0.0, 6.0, 1.0, 1.0, 1.0);
+			adj = gtk_adjustment_new(0.0, 0.0, 6.0, 1.0, 1.0, 0.0);
 			edit->extra.size.digits = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(edit->extra.size.digits), fmt->extra.size.digits);
 			gtk_table_attach(GTK_TABLE(edit->extra.size.options), edit->extra.size.digits, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
@@ -416,7 +406,7 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(trb[ti]), TRUE);
 			gtk_widget_set_sensitive(edit->extra.size.options, ti != 0);
 
-			sep = gtk_hseparator_new();	
+			sep = gtk_hseparator_new();
 			gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
 			edit->extra.size.dir_show_fs_size = gtk_check_button_new_with_label(_("Show Dir's File System Size?"));
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(edit->extra.size.dir_show_fs_size), fmt->extra.size.dir_show_fs_size);
@@ -512,7 +502,7 @@ static gint evt_edit_justification(GtkWidget *wid, gpointer user)
 /* 2002-07-18 -	This used to be a button event handler. Now remade to just store accepted values. */
 static void accept_format(P_Edit *edit)
 {
-	gchar		*nfmt = "";
+	const gchar	*nfmt = "";
 	DpCFmt		*fmt = &edit->def->edit.format[edit->def->sclist_row];
 
 	strcpy(fmt->title, gtk_entry_get_text(GTK_ENTRY(edit->title)));
@@ -644,7 +634,7 @@ static gint evt_sclist_edit_clicked(GtkWidget *wid, gpointer user)
 	gtk_option_menu_set_history(GTK_OPTION_MENU(edit.just), edit.curr_just);
 	gtk_table_attach_defaults(GTK_TABLE(table), edit.just, 1, 2, 2, 3);
 
-	edit.wadj  = gtk_adjustment_new(fmt->width, 10.0f, 800.0f, 1.0f, 50.0f, 20.0f);
+	edit.wadj  = gtk_adjustment_new(fmt->width, 10.0, 800.0, 1.0f, 50.0, 0.0);
 	edit.width = gtk_spin_button_new(GTK_ADJUSTMENT(edit.wadj), 0, 0);
 	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(edit.width), FALSE);
 	gtk_table_attach_defaults(GTK_TABLE(table), edit.width, 1, 2, 3, 4);
@@ -832,10 +822,10 @@ static void evt_default_path_grab(GtkWidget *wid, gpointer user)
 	gtk_entry_set_text(GTK_ENTRY(def->dentry), page->min->gui->pane[def->index].dir.path);
 }
 
-/* 2004-02-25 -	Set default path to "take from history". Not totally beatiful, but still. */
+/* 2004-02-25 - Set default path to "take from history". Not totally beatiful, but still. */
 static void evt_default_path_history(GtkWidget *wid, gpointer user)
 {
-	PDef	*def = user;
+	PDef    *def = user;
 
 	gtk_entry_set_text(GTK_ENTRY(def->dentry), "@history[0]");
 }
@@ -936,17 +926,17 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	PDef		*def;
 	GtkWidget	*label, *scwin, *vbox, *frame, *hbox, *table, *button, *sep, *menu;
 	gchar		*atitle[] = { N_("Default Title"), N_("Content") },
-			*stitle[] = { N_("Title"), N_("Content") }, buf[64];
+				*stitle[] = { N_("Title"), N_("Content") }, buf[64];
 	const gchar	*smitem[] = { N_("Directories First"), N_("Directories Last"), N_("Directories Mixed"), NULL },
-			*spos[] = { N_("System Default"), N_("Left of List"), N_("Right of List") },
-			*mctrl[] = { N_("System Default"), N_("Always Set"), "(Always Clear)", N_("Toggle") },
-			*tfmt[] = { N_("Copy To %s"), N_("Copy From %s"), N_("Swap With %s") };
+				*spos[] = { N_("System Default"), N_("Left of List"), N_("Right of List") },
+				*mctrl[] = { N_("System Default"), N_("Always Set"), "(Always Clear)", N_("Toggle") },
+				*tfmt[] = { N_("Copy To %s"), N_("Copy From %s"), N_("Swap With %s") };
 	gchar		*row[2];
 	guint		i;
 	GtkSignalFunc	scfunc[] = { GTK_SIGNAL_FUNC(evt_sclist_edit_clicked), GTK_SIGNAL_FUNC(evt_sclist_remove_clicked),
-				     GTK_SIGNAL_FUNC(evt_sclist_up_clicked), GTK_SIGNAL_FUNC(evt_sclist_down_clicked) },
-			tool[] = { GTK_SIGNAL_FUNC(evt_copyto_clicked), GTK_SIGNAL_FUNC(evt_copyfrom_clicked),
-				    GTK_SIGNAL_FUNC(evt_swap_clicked) };
+								 GTK_SIGNAL_FUNC(evt_sclist_up_clicked), GTK_SIGNAL_FUNC(evt_sclist_down_clicked) },
+				     tool[] = { GTK_SIGNAL_FUNC(evt_copyto_clicked), GTK_SIGNAL_FUNC(evt_copyfrom_clicked),
+				    		    GTK_SIGNAL_FUNC(evt_swap_clicked) };
 
 	def = &page->pane[index];
 	def->page  = page;
@@ -967,7 +957,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	for(i = 0; i < sizeof atitle / sizeof atitle[0]; i++)
 		atitle[i] = _(atitle[i]);
 	def->aclist = gtk_clist_new_with_titles(2, atitle);
-	gtk_signal_connect(GTK_OBJECT(def->aclist), "button_press_event", GTK_SIGNAL_FUNC(evt_aclist_button_press), def);
+	g_signal_connect(G_OBJECT(def->aclist), "select_row", G_CALLBACK(evt_aclist_select_row), def);
 	gtk_clist_set_column_width(GTK_CLIST(def->aclist), 0, 70);
 	gtk_clist_column_titles_passive(GTK_CLIST(def->aclist));
 	for(i = 0; i < DPC_NUM_TYPES; i++)
@@ -993,9 +983,9 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	for(i = 0; i < sizeof stitle / sizeof stitle[0]; i++)
 		stitle[i] = _(stitle[i]);
 	def->sclist = gtk_clist_new_with_titles(2, stitle);
-	gtk_signal_connect(GTK_OBJECT(def->sclist), "select_row", GTK_SIGNAL_FUNC(evt_sclist_select_row), def);
-	gtk_signal_connect(GTK_OBJECT(def->sclist), "unselect_row", GTK_SIGNAL_FUNC(evt_sclist_unselect_row), def);
-	gtk_signal_connect(GTK_OBJECT(def->sclist), "row_move", GTK_SIGNAL_FUNC(evt_sclist_move_row), def);
+	g_signal_connect(G_OBJECT(def->sclist), "select_row", G_CALLBACK(evt_sclist_select_row), def);
+	g_signal_connect(G_OBJECT(def->sclist), "unselect_row", G_CALLBACK(evt_sclist_unselect_row), def);
+	g_signal_connect(G_OBJECT(def->sclist), "row_move", G_CALLBACK(evt_sclist_move_row), def);
 	gtk_clist_set_column_width(GTK_CLIST(def->sclist), 0, 70);
 	gtk_clist_column_titles_passive(GTK_CLIST(def->sclist));
 	gtk_clist_set_reorderable(GTK_CLIST(def->sclist), TRUE);
@@ -1008,7 +998,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	def->sbutton[3] = gui_arrow_button_new(GTK_ARROW_DOWN);
 	for(i = 0; i < sizeof def->sbutton / sizeof def->sbutton[0]; i++)
 	{
-		gtk_signal_connect(GTK_OBJECT(def->sbutton[i]), "clicked", GTK_SIGNAL_FUNC(scfunc[i]), def);
+		g_signal_connect(G_OBJECT(def->sbutton[i]), "clicked", G_CALLBACK(scfunc[i]), def);
 		gtk_table_attach_defaults(GTK_TABLE(table), def->sbutton[i], i, i + 1, 0, 1);
 	}
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
@@ -1023,7 +1013,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	label = gtk_label_new(_("Sort On"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	def->scmenu = gtk_option_menu_new();
-	menu = dpf_get_content_menu(evt_sort_set_content, def);
+	menu = dpf_get_content_menu(G_CALLBACK(evt_sort_set_content), def);
 	gtk_widget_show_all(menu);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(def->scmenu), menu);
 	gtk_box_pack_start(GTK_BOX(hbox), def->scmenu, TRUE, TRUE, 0);
@@ -1035,13 +1025,13 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	hbox = gtk_hbox_new(FALSE, 0);
 	def->sinvert = gtk_check_button_new_with_label(_("Inverse Sorting?"));
-	gtk_signal_connect(GTK_OBJECT(def->sinvert), "clicked", GTK_SIGNAL_FUNC(evt_sort_invert_clicked), def);
+	g_signal_connect(G_OBJECT(def->sinvert), "clicked", G_CALLBACK(evt_sort_invert_clicked), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->sinvert, TRUE, FALSE, 0);
 	def->snocase = gtk_check_button_new_with_label(_("Ignore Case?"));
-	gtk_signal_connect(GTK_OBJECT(def->snocase), "clicked", GTK_SIGNAL_FUNC(evt_sort_nocase_clicked), def);
+	g_signal_connect(G_OBJECT(def->snocase), "clicked", G_CALLBACK(evt_sort_nocase_clicked), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->snocase, TRUE, FALSE, 0);
 	def->snumerical = gtk_check_button_new_with_label(_("Numerical Mode?"));
-	gtk_signal_connect(GTK_OBJECT(def->snumerical), "clicked", GTK_SIGNAL_FUNC(evt_sort_numerical_clicked), def);
+	g_signal_connect(G_OBJECT(def->snumerical), "clicked", G_CALLBACK(evt_sort_numerical_clicked), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->snumerical, TRUE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -1051,16 +1041,16 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	frame = gtk_frame_new(_("Default Directory"));
 	hbox = gtk_hbox_new(FALSE, 0);
 	def->dentry = gtk_entry_new_with_max_length(PATH_MAX - 1);
-	gtk_signal_connect(GTK_OBJECT(def->dentry), "changed", GTK_SIGNAL_FUNC(evt_default_path_changed), def);
+	g_signal_connect(G_OBJECT(def->dentry), "changed", G_CALLBACK(evt_default_path_changed), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->dentry, TRUE, TRUE, 0);
 	button = gtk_button_new_with_label(_("Grab Current"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(evt_default_path_grab), def);
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(evt_default_path_grab), def);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(def->vbox), frame, FALSE, FALSE, 0);
 	button = gtk_button_new_with_label(_("From History"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(evt_default_path_history), def);
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(evt_default_path_history), def);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	gtk_box_pack_start(GTK_BOX(def->vbox), frame, FALSE, FALSE, 0);
 
 	/* General section below default directory (flags, scrollbar, and 'ctrl' modification. */
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -1070,19 +1060,19 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	table = gtk_table_new(3, 2, FALSE);
 
 	def->fabove = gtk_check_button_new_with_label(_("Path Above?"));
-	gtk_signal_connect(GTK_OBJECT(def->fabove), "clicked", GTK_SIGNAL_FUNC(evt_flag_above_clicked), def);
+	g_signal_connect(G_OBJECT(def->fabove), "clicked", G_CALLBACK(evt_flag_above_clicked), def);
 	gtk_table_attach_defaults(GTK_TABLE(table), def->fabove, 0, 1, 0, 1);
 
 	def->fhide = gtk_check_button_new_with_label(_("Hide Allowed?"));
-	gtk_signal_connect(GTK_OBJECT(def->fhide), "clicked", GTK_SIGNAL_FUNC(evt_flag_hide_clicked), def);
+	g_signal_connect(G_OBJECT(def->fhide), "clicked", G_CALLBACK(evt_flag_hide_clicked), def);
 	gtk_table_attach_defaults(GTK_TABLE(table), def->fhide, 0, 1, 1, 2);
 
 	def->fscroll = gtk_check_button_new_with_label(_("Scrollbar Always?"));
-	gtk_signal_connect(GTK_OBJECT(def->fscroll), "clicked", GTK_SIGNAL_FUNC(evt_flag_scrollbar_clicked), def);
+	g_signal_connect(G_OBJECT(def->fscroll), "clicked", G_CALLBACK(evt_flag_scrollbar_clicked), def);
 	gtk_table_attach_defaults(GTK_TABLE(table), def->fscroll, 0, 1, 2, 3);
 
 	def->fhparent = gtk_check_button_new_with_label(_("Huge Parent Button?"));
-	gtk_signal_connect(GTK_OBJECT(def->fhparent), "clicked", GTK_SIGNAL_FUNC(evt_flag_hugeparent_clicked), def);
+	g_signal_connect(G_OBJECT(def->fhparent), "clicked", G_CALLBACK(evt_flag_hugeparent_clicked), def);
 	gtk_table_attach_defaults(GTK_TABLE(table), def->fhparent, 1, 2, 0, 1);
 
 	gtk_container_add(GTK_CONTAINER(frame), table);
@@ -1094,7 +1084,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	def->splist = gui_radio_group_new(sizeof def->spos / sizeof def->spos[0], spos, def->spos);
 	for(i = 0; i < sizeof def->spos / sizeof def->spos[0]; i++)
 	{
-		gtk_signal_connect(GTK_OBJECT(def->spos[i]), "clicked", GTK_SIGNAL_FUNC(evt_scrollbarpos_clicked), def);
+		g_signal_connect(G_OBJECT(def->spos[i]), "clicked", G_CALLBACK(evt_scrollbarpos_clicked), def);
 		gtk_box_pack_start(GTK_BOX(vbox), def->spos[i], FALSE, FALSE, 0);
 	}
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -1108,7 +1098,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	{
 		if(mctrl[i][0] == '(')	/* Don't pack the hidden button. A rather ugly hack. */
 			continue;
-		gtk_signal_connect(GTK_OBJECT(def->mctrl[i]), "clicked", GTK_SIGNAL_FUNC(evt_controlmode_clicked), def);
+		g_signal_connect(G_OBJECT(def->mctrl[i]), "clicked", G_CALLBACK(evt_controlmode_clicked), def);
 		gtk_box_pack_start(GTK_BOX(vbox), def->mctrl[i], FALSE, FALSE, 0);
 	}
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -1121,7 +1111,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	{
 		g_snprintf(buf, sizeof buf, _(tfmt[i]), other);
 		button = gtk_button_new_with_label(buf);
-		gtk_signal_connect(GTK_OBJECT(button), "clicked", tool[i], def);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(tool[i]), def);
 		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
 	}
 	gtk_box_pack_start(GTK_BOX(def->vbox), hbox, FALSE, FALSE, 5);
@@ -1130,28 +1120,13 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-05-16 -	This gets called by the color_dialog when the color changes. */
-static void color_changed(GdkColor *color, gpointer user)
+/* 2008-11-28 -	User changed a color. Copy it from button. */
+static void evt_color_set(GtkWidget *wid, gpointer user)
 {
-	PColors	*col = user;
+	PColors		*col = user;
+	gint		index = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wid), "index"));
 
-	col->edit.color[col->index] = *color;
-	set_color_widgets(col);
-}
-
-/* 1999-05-16 -	User clicked a color. Pop up the good 'ol color dialog. */
-static void evt_color_clicked(GtkWidget *wid, gpointer wuser, guint index, gpointer suser)
-{
-	PColors		*col = suser;
-	GdkColor	initial;
-
-	col->index = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(wid)));
-	initial = col->edit.color[col->index];
-	if(cdl_dialog_sync_new_wait(NULL, color_changed, &initial, col) != DLG_POSITIVE)	/* Cancel? */
-	{
-		col->edit.color[col->index] = initial;
-		set_color_widgets(col);
-	}
+	gtk_color_button_get_color(GTK_COLOR_BUTTON(wid), &col->edit.color[index]);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -1172,10 +1147,11 @@ static void init_color_page(P_DirPane *page)
 	{
 		label = gtk_label_new(_(ltxt[i]));
 		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, i + 1,  0,0,0,0);
-		col->widget[i] = od_emil_button_new();
+		col->widget[i] = gtk_color_button_new();
 		gtk_object_set_user_data(GTK_OBJECT(col->widget[i]), GUINT_TO_POINTER((DpCol) i));
-		gtk_signal_connect(GTK_OBJECT(col->widget[i]), "activate", GTK_SIGNAL_FUNC(evt_color_clicked), col);
-		gtk_table_attach(GTK_TABLE(table), col->widget[i], 1, 2, i, i + 1, GTK_EXPAND|GTK_FILL,0,5,2);
+		g_object_set_data(G_OBJECT(col->widget[i]), "index", GINT_TO_POINTER(i));
+		g_signal_connect(G_OBJECT(col->widget[i]), "color_set", G_CALLBACK(evt_color_set), col);
+		gtk_table_attach(GTK_TABLE(table), col->widget[i], 1, 2, i, i + 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	}
 	gtk_box_pack_start(GTK_BOX(col->vbox), table, TRUE, TRUE, 0);
 }
@@ -1219,7 +1195,7 @@ static void init_paning_page(P_DirPane *page)
 				    NULL, _(orientname[i]));
 		if(i == 0)
 			gtk_object_set_data(GTK_OBJECT(p->orientation[i]), "horiz", GINT_TO_POINTER(1));
-		gtk_signal_connect(GTK_OBJECT(p->orientation[i]), "toggled", GTK_SIGNAL_FUNC(evt_pane_orientation_toggled), page);
+		g_signal_connect(G_OBJECT(p->orientation[i]), "toggled", G_CALLBACK(evt_pane_orientation_toggled), page);
 		gtk_box_pack_start(GTK_BOX(vbox), p->orientation[i], FALSE, FALSE, 0);
 	}
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -1251,7 +1227,7 @@ static void init_paning_page(P_DirPane *page)
 			gtk_box_pack_start(GTK_BOX(row), label, FALSE, FALSE, 5);
 		}
 		gtk_object_set_data(GTK_OBJECT(rb), "index", GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(rb), "toggled", GTK_SIGNAL_FUNC(evt_pane_mode_toggled), page);
+		g_signal_connect(G_OBJECT(rb), "toggled", G_CALLBACK(evt_pane_mode_toggled), page);
 		gtk_table_attach(GTK_TABLE(table), rb, 0, 1, i, i+1, GTK_FILL,0,0,0);
 		if(row != NULL)
 		{
@@ -1276,11 +1252,10 @@ static void init_history_page(P_DirPane *page)
 	page->history.vbox = gtk_vbox_new(FALSE, 0);
 
 	page->history.select = gtk_check_button_new_with_label(_("Remember Selected Rows?"));
-	gtk_signal_connect(GTK_OBJECT(page->history.select), "toggled", GTK_SIGNAL_FUNC(evt_history_toggled), &page->history.edit.select);
+	g_signal_connect(G_OBJECT(page->history.select), "toggled", G_CALLBACK(evt_history_toggled), &page->history.edit.select);
 	gtk_box_pack_start(GTK_BOX(page->history.vbox), page->history.select, FALSE, FALSE, 0);
-	
 	page->history.save = gtk_check_button_new_with_label(_("Save History Lists?"));
-	gtk_signal_connect(GTK_OBJECT(page->history.save), "toggled", GTK_SIGNAL_FUNC(evt_history_toggled), &page->history.edit.save);
+	g_signal_connect(G_OBJECT(page->history.save), "toggled", G_CALLBACK(evt_history_toggled), &page->history.edit.save);
 	gtk_box_pack_start(GTK_BOX(page->history.vbox), page->history.save, FALSE, FALSE, 0);
 }
 
@@ -1466,7 +1441,7 @@ static void save_sort(FILE *out, DPSort *sort)
 	xml_put_text(out, "content", dpf_get_content_mnemonic(sort->content));
 	xml_put_text(out, "mode", mode_name[sort->mode]);
 	xml_put_boolean(out, "invert", sort->invert);
-	xml_put_boolean(out, "nocase", sort->nocase);	
+	xml_put_boolean(out, "nocase", sort->nocase);
 	xml_put_boolean(out, "numerical", sort->numerical);
 	xml_put_node_close(out, "DPSort");
 }
@@ -1514,7 +1489,7 @@ static void save_history(FILE *out, DPHistory *p)
 {
 	xml_put_node_open(out, "DirPaneHistory");
 	xml_put_boolean(out, "select", p->select);
-	xml_put_boolean(out, "save",   p->save);
+	xml_put_boolean(out, "save", p->save);
 	xml_put_node_close(out, "DirPaneHistory");
 }
 
@@ -1701,7 +1676,7 @@ static void load_pane(const XmlNode *node, DPFormat *fmt)
 	if(xml_get_integer(node, "columns", (gint *) &fmt->num_columns))
 	{
 		xml_node_visit_children(node, load_column, fmt);
-		
+
 		if((data = xml_tree_search(node, "DPSort")) != NULL)
 			load_sort(data, &fmt->sort);
 		xml_get_text_copy(node, "defpath", fmt->def_path, sizeof fmt->def_path);
