@@ -50,11 +50,11 @@ static GtkWidget * cer_init(MainInfo *min, gchar **name)
 	page->vbox = gtk_vbox_new(FALSE, 0);
 
 	page->dialog = gtk_check_button_new_with_label(_("Use Dialog to Report Errors?"));
-	gtk_signal_connect(GTK_OBJECT(page->dialog), "toggled", GTK_SIGNAL_FUNC(evt_dialog_toggled), page);
+	g_signal_connect(G_OBJECT(page->dialog), "toggled", G_CALLBACK(evt_dialog_toggled), page);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->dialog, FALSE, FALSE, 0);
 
 	page->beep = gtk_check_button_new_with_label(_("Cause Console Beep on Error?"));
-	gtk_signal_connect(GTK_OBJECT(page->beep), "toggled", GTK_SIGNAL_FUNC(evt_beep_toggled), page);
+	g_signal_connect(G_OBJECT(page->beep), "toggled", G_CALLBACK(evt_beep_toggled), page);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->beep, FALSE, FALSE, 0);
 
 	gtk_widget_show_all(page->vbox);
