@@ -66,6 +66,9 @@ typedef struct {
 	GtkWidget	*vbox;
 	GtkWidget	*scwin;
 	GtkWidget	*tree;		/* Main style tree widget. */
+	gulong		sig_expand;	/* Signal handler for tree item expansion. */
+	gulong		sig_collapse;	/* Signal handler for tree item collapse. */
+	GtkTreeStore	*store;
 
 	GtkWidget	*dvbox;		/* Vbox holding definition widgets. */
 	GtkWidget	*dname;		/* Name of selected style. */
@@ -80,8 +83,6 @@ typedef struct {
 
 	MainInfo	*min;
 	StyleInfo	*si;
-	Style		*curr_style;
-	GtkWidget	*curr_style_widget;
 	const gchar	*curr_prop;	/* Current property, when editing one. */
 	gint		curr_arow;	/* Action property clist row. */
 	gboolean	modified;
@@ -186,7 +187,6 @@ static void reset_widgets_action(P_Styles *page)
 /* 1999-05-24 -	Reset widgets. Handy when there is no longer a selection. */
 static void reset_widgets(P_Styles *page)
 {
-	page->curr_style = NULL;
 	page->curr_prop  = NULL;
 	page->curr_arow  = -1;
 	gtk_entry_set_text(GTK_ENTRY(page->dname), "");
@@ -198,51 +198,85 @@ static void reset_widgets(P_Styles *page)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-05-27 -	This gets called when the user changes the style selection. Nice. */
-static void evt_style_selection_changed(GtkWidget *wid, gpointer user)
+static Style * style_get_selected(P_Styles *page, GtkTreeIter *iter)
+{
+	GtkTreeIter	myiter;
+
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->tree)), NULL, &myiter))
+	{
+		if(iter != NULL)
+			*iter = myiter;
+		return stl_styleinfo_get_style_iter(page->si, page->store, &myiter);
+	}
+	return NULL;
+}
+
+/* 2009-03-24 -	New-style selection tracking. */
+static void evt_style_selection_changed(GtkTreeSelection *sel, gpointer user)
 {
 	P_Styles	*page = user;
-	GList		*sel;
+	Style		*style;
 
-	if(page != NULL)
+	if((style = style_get_selected(page, NULL)) != NULL)
 	{
-		page->curr_style = NULL;
-		page->curr_style_widget = NULL;
-		if((sel = GTK_TREE_SELECTION(GTK_TREE(wid))) != NULL)
-		{
-			GtkWidget	*child = sel->data;
-			Style		*nc;
-
-			nc = stl_style_get(child);
-			set_widgets(page, nc);
-			reset_widgets_action(page);
-			page->curr_style = nc;
-			page->curr_style_widget = child;
-		}
-		else
-			reset_widgets(page);
+		set_widgets(page, style);
+		reset_widgets_action(page);
 	}
+	else
+		reset_widgets(page);
+}
+
+/* 2009-02-06 -	A row was expanded, update the underlying Style object. */
+static void evt_style_row_expanded(GtkTreeView *view, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data)
+{
+	GtkTreeModel	*m;
+	Style		*stl;
+
+	m = gtk_tree_view_get_model(view);
+	gtk_tree_model_get(m, iter, 1, &stl, -1);
+	stl_style_set_expand(stl, TRUE);
+}
+
+/* 2009-02-06 -	A row was collapsed, update the underlying Style object. */
+static void evt_style_row_collapsed(GtkTreeView *view, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data)
+{
+	GtkTreeModel	*m;
+	Style		*stl;
+
+	m = gtk_tree_view_get_model(view);
+	gtk_tree_model_get(m, iter, 1, &stl, -1);
+	stl_style_set_expand(stl, FALSE);
+}
+
+/* 2009-02-06 -	This is a gtk_tree_model_foreach() callback, that simply applies the collapsed/
+**		expanded status to a tree row showing a style. This is a property of the view,
+**		so it cannot be done from inside the styles module.
+*/
+static gboolean set_expand(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user)
+{
+	Style	*stl;
+
+	gtk_tree_model_get(model, iter, 1, &stl, -1);
+	if(stl_style_get_expand(stl))
+		gtk_tree_view_expand_row(GTK_TREE_VIEW(user), path, FALSE);
+	else
+		gtk_tree_view_collapse_row(GTK_TREE_VIEW(user), path);
+	return FALSE;
 }
 
 /* 1999-05-24 -	Repopulate the tree. */
 static void populate_tree(P_Styles *page)
 {
-	if(page->tree != NULL)
-	{
-		GList	*chlist, *iter;
-
-		chlist = gtk_container_children(GTK_CONTAINER(page->scwin));
-		for(iter = chlist; iter != NULL; iter = g_list_next(iter))
-			gtk_container_remove(GTK_CONTAINER(page->scwin), iter->data);
-		g_list_free(chlist);
-		page->tree = NULL;
-	}
 	if(page->si != NULL)
 	{
-		page->tree = GTK_WIDGET(stl_styleinfo_build(page->si, &page->modified));
-		gtk_signal_connect(GTK_OBJECT(page->tree), "selection_changed", evt_style_selection_changed, page);
-		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(page->scwin), page->tree);
-		gtk_widget_show(page->tree);
+		page->store = stl_styleinfo_build_partial2(page->si, NULL);
+		/* Block expand/collapse signals. */
+		g_signal_handler_block(page->tree, page->sig_collapse);
+		g_signal_handler_block(page->tree, page->sig_expand);
+		gtk_tree_view_set_model(GTK_TREE_VIEW(page->tree), GTK_TREE_MODEL(page->store));
+		gtk_tree_model_foreach(GTK_TREE_MODEL(page->store), set_expand, page->tree);
+		g_signal_handler_unblock(page->tree, page->sig_expand);
+		g_signal_handler_unblock(page->tree, page->sig_collapse);
 	}
 }
 
@@ -253,13 +287,17 @@ static void evt_style_name_changed(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
 	const gchar	*name;
+	Style		*style;
+	GtkTreeIter	iter;
 
-	if((page != NULL) && (page->curr_style != NULL) && (name = gtk_entry_get_text(GTK_ENTRY(wid))) != NULL)
+	if((style = style_get_selected(page, &iter)) != NULL)
 	{
-		ctp_replace_style(stl_style_get_name(page->curr_style), page->curr_style);
-		stl_style_set_name(page->curr_style, name);
-		stl_style_set_name_widget(page->curr_style_widget, name);
-		page->modified = TRUE;
+		if((name = gtk_entry_get_text(GTK_ENTRY(wid))) != NULL)
+		{
+			ctp_replace_style(stl_style_get_name(style), style);
+			stl_styleinfo_set_name_iter(page->si, page->store, &iter, name);
+			page->modified = TRUE;
+		}
 	}
 }
 
@@ -267,13 +305,17 @@ static void evt_style_name_changed(GtkWidget *wid, gpointer user)
 static void evt_style_parent_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 	Style		*np;
 
-	if((np = sdl_dialog_sync_new_wait(page->si, page->curr_style)) != NULL)	
+	if((style = style_get_selected(page, NULL)) != NULL)
 	{
-		stl_styleinfo_style_set_parent(page->si, page->curr_style, np);
-		populate_tree(page);
-		page->modified = TRUE;
+		if((np = sdl_dialog_sync_new_wait(page->si, style)) != NULL)
+		{
+			stl_styleinfo_style_set_parent(page->si, style, np);
+			populate_tree(page);
+			page->modified = TRUE;
+		}
 	}
 }
 
@@ -285,79 +327,84 @@ static void evt_style_parent_clicked(GtkWidget *wid, gpointer user)
 static void evt_vprop_override_toggled(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
+	const gchar	*pname;
+	gboolean	or;
 
-	if((page != NULL) && (page->curr_style != NULL))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	pname = gtk_object_get_user_data(GTK_OBJECT(wid));
+	or = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
+	if(or)
 	{
-		const gchar	*pname;
-		gboolean	or;
-
-		pname = gtk_object_get_user_data(GTK_OBJECT(wid));
-		or = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
-		if(or)
-		{
-			if(strcmp(pname, SPN_ICON_UNSEL) == 0)
-				stl_style_property_set_icon(page->curr_style, pname, stl_style_property_get_icon(page->curr_style, pname));
-			else
-			{
-				const GdkColor	*def;
-
-				/* FIXME: This assumes only background can be non-overridden in root. */
-				if((def = stl_style_property_get_color(page->curr_style, pname)) == NULL)
-					def = &(gtk_widget_get_style(GTK_WIDGET(page->min->gui->pane[0].list))->base[GTK_STATE_NORMAL]);
-				stl_style_property_set_color(page->curr_style, pname, def);
-			}
-		}
+		if(strcmp(pname, SPN_ICON_UNSEL) == 0)
+			stl_style_property_set_icon(style, pname, stl_style_property_get_icon(style, pname));
 		else
-			stl_style_property_remove(page->curr_style, pname);
-		set_widgets(page, page->curr_style);
-		page->modified = TRUE;
+		{
+			const GdkColor	*def;
+
+			/* FIXME: This assumes only background can be non-overridden in root. */
+			if((def = stl_style_property_get_color(style, pname)) == NULL)
+				def = &(gtk_widget_get_style(GTK_WIDGET(page->min->gui->pane[0].list))->base[GTK_STATE_NORMAL]);
+			stl_style_property_set_color(style, pname, def);
+		}
 	}
+	else
+		stl_style_property_remove(style, pname);
+	set_widgets(page, style);
+	page->modified = TRUE;
 }
 
 /* 1999-05-24 -	A color property has changed. Update previews and stuff. */
 static void evt_vprop_color_changed(GdkColor *color, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	stl_style_property_set_color(page->curr_style, page->curr_prop, color);
-	set_widgets(page, page->curr_style);
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	stl_style_property_set_color(style, page->curr_prop, color);
+	set_widgets(page, style);
 }
 
 /* 1999-05-24 -	User hit the "Edit..." (or, for the icon, "Pick...") button below override toggle. */
 static void evt_vprop_edit_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	if((page != NULL) && (page->curr_style != NULL))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	page->curr_prop = gtk_object_get_user_data(GTK_OBJECT(wid));
+
+	if(strcmp(page->curr_prop, SPN_ICON_UNSEL) == 0)
 	{
-		page->curr_prop = gtk_object_get_user_data(GTK_OBJECT(wid));
+		const gchar	*icon;
 
-		if(strcmp(page->curr_prop, SPN_ICON_UNSEL) == 0)
+		if((icon = idl_dialog_sync_new_wait(page->min, cpt_get_path(PTID_ICON), NULL,
+						  stl_style_property_get_icon(style, page->curr_prop),
+							TRUE)) != NULL)
 		{
-			const gchar	*icon;
-
-			if((icon = idl_dialog_sync_new_wait(page->min, cpt_get_path(PTID_ICON), NULL,
-							  stl_style_property_get_icon(page->curr_style, page->curr_prop),
-								TRUE)) != NULL)
-			{
-				stl_style_property_set_icon(page->curr_style, page->curr_prop, icon);
-				page->modified = TRUE;
-				set_widgets(page, page->curr_style);
-			}
+			stl_style_property_set_icon(style, page->curr_prop, icon);
+			page->modified = TRUE;
+			set_widgets(page, style);
 		}
+	}
+	else
+	{
+		const GdkColor	*col;
+		GdkColor	initial;
+
+		if((col = stl_style_property_get_color(style, page->curr_prop)) != NULL)
+			initial = *col;
+		if(cdl_dialog_sync_new_wait(_("Edit Color"), evt_vprop_color_changed, &initial, page) != DLG_POSITIVE)
+			stl_style_property_set_color(style, page->curr_prop, &initial);
 		else
-		{
-			const GdkColor	*col;
-			GdkColor	initial;
-
-			if((col = stl_style_property_get_color(page->curr_style, page->curr_prop)) != NULL)
-				initial = *col;
-			if(cdl_dialog_sync_new_wait(_("Edit Color"), evt_vprop_color_changed, &initial, page) != DLG_POSITIVE)
-				stl_style_property_set_color(page->curr_style, page->curr_prop, &initial);
-			else
-				page->modified = TRUE;
-			set_widgets(page, page->curr_style);
-		}
+			page->modified = TRUE;
+		set_widgets(page, style);
 	}
 }
 
@@ -367,11 +414,15 @@ static void evt_vprop_edit_clicked(GtkWidget *wid, gpointer user)
 static void action_set_delete_button(P_Styles *page)
 {
 	gboolean	unique, or;
+	Style		*style;
 
-	unique = stl_style_property_is_unique(page->curr_style, page->curr_prop);
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	unique = stl_style_property_is_unique(style, page->curr_prop);
 	gtk_label_set_text(GTK_LABEL(GTK_BIN(page->dpaction.adel)->child), unique ? _("Delete Action") :
 				_("Revert to Inherited Command"));
-	or = stl_style_property_get_override(page->curr_style, page->curr_prop);
+	or = stl_style_property_get_override(style, page->curr_prop);
 	gtk_entry_set_editable(GTK_ENTRY(page->dpaction.aname), or);
 	gtk_widget_set_sensitive(page->dpaction.adel, or);
 }
@@ -380,53 +431,58 @@ static void action_set_delete_button(P_Styles *page)
 static void evt_aprop_selected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
+	gchar		*prop;
 
-	if((page != NULL) && (page->curr_style != NULL))
-	{
-		gchar	*prop;
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
 
-		page->curr_prop = NULL;
-		page->curr_arow = -1;
-		gtk_clist_get_text(GTK_CLIST(page->dpaction.actions), row, 0, (gchar **) &prop);
-		gtk_entry_set_text(GTK_ENTRY(page->dpaction.aname), prop);
-		gtk_widget_set_sensitive(page->dpaction.aname, TRUE);
-		gtk_entry_set_text(GTK_ENTRY(page->dpaction.acmdseq), stl_style_property_get_action(page->curr_style, prop));
-		/* Force execution of cmdseq_changed() before we set curr_prop & curr_arow. */
-		gui_events_flush();
-		gtk_widget_set_sensitive(page->dpaction.acmdseq, TRUE);
-		gtk_widget_set_sensitive(page->dpaction.acmdpick, TRUE);
-		page->curr_prop = prop;
-		page->curr_arow = row;
-		action_set_delete_button(page);
-	}
+	page->curr_prop = NULL;
+	page->curr_arow = -1;
+	gtk_clist_get_text(GTK_CLIST(page->dpaction.actions), row, 0, (gchar **) &prop);
+	gtk_entry_set_text(GTK_ENTRY(page->dpaction.aname), prop);
+	gtk_widget_set_sensitive(page->dpaction.aname, TRUE);
+	gtk_entry_set_text(GTK_ENTRY(page->dpaction.acmdseq), stl_style_property_get_action(style, prop));
+	/* Force execution of cmdseq_changed() before we set curr_prop & curr_arow. */
+	gui_events_flush();
+	gtk_widget_set_sensitive(page->dpaction.acmdseq, TRUE);
+	gtk_widget_set_sensitive(page->dpaction.acmdpick, TRUE);
+	page->curr_prop = prop;
+	page->curr_arow = row;
+	action_set_delete_button(page);
 }
 
-/* 1999-05-25 -	An action was just deselected. Update widgetry. */
+/* 1999-05-25 -	An action was just unselected. Update widgets. */
 static void evt_aprop_unselected(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	if((page != NULL) && (page->curr_style != NULL))
-	{
-		page->curr_prop = NULL;
-		page->curr_arow = -1;
-		reset_widgets_action(page);
-	}
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	page->curr_prop = NULL;
+	page->curr_arow = -1;
+	reset_widgets_action(page);
 }
 
 /* 2004-02-24 -	Action was renamed. */
 static void evt_aprop_name_changed(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	if((page != NULL) && (page->curr_style != NULL) && (page->curr_prop != NULL) && (page->curr_arow >= 0))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+	if((page->curr_prop != NULL) && (page->curr_arow >= 0))
 	{
-		gchar	*oname, *nname;
+		gchar		*oname;
+		const gchar	*nname;
 
 		gtk_clist_get_text(GTK_CLIST(page->dpaction.actions), page->curr_arow, 0, &oname);
 		if((oname != NULL) && (nname = gtk_entry_get_text(GTK_ENTRY(wid))) != NULL)
 		{
-			if(stl_style_property_rename(page->curr_style, oname, nname))	/* Will fail on collision. */
+			if(stl_style_property_rename(style, oname, nname))	/* Will fail on collision. */
 				gtk_clist_set_text(GTK_CLIST(page->dpaction.actions), page->curr_arow, 0, nname);
 		}
 	}
@@ -436,27 +492,35 @@ static void evt_aprop_name_changed(GtkWidget *wid, gpointer user)
 static void evt_aprop_cmdseq_changed(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	if((page != NULL) && (page->curr_style != NULL) && (page->curr_prop != NULL) && (page->curr_arow >= 0))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	if((page->curr_prop != NULL) && (page->curr_arow >= 0))
 	{
 		const gchar	*text;
 
 		if((text = gtk_entry_get_text(GTK_ENTRY(wid))) != NULL)
 		{
 			gtk_clist_set_text(GTK_CLIST(page->dpaction.actions), page->curr_arow, 1, text);
-			stl_style_property_set_action(page->curr_style, page->curr_prop, text);
+			stl_style_property_set_action(style, page->curr_prop, text);
 			action_set_delete_button(page);
 			page->modified = TRUE;
 		}
 	}
 }
 
-/* 1999-05-25 -	User hit the details (mag. glass) button to the right of the Command field. Dialog-time. */
+/* 1999-05-25 -	User hit the details (magnifying glass) button to the right of the Command field. Dialog-time. */
 static void evt_aprop_cmdpick_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	if((page != NULL) && (page->curr_style != NULL) && (page->curr_prop != NULL) && (page->curr_arow >= 0))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	if((page->curr_prop != NULL) && (page->curr_arow >= 0))
 	{
 		const gchar	*cmd;
 
@@ -469,45 +533,49 @@ static void evt_aprop_cmdpick_clicked(GtkWidget *wid, gpointer user)
 static void evt_aprop_add_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
+	Dialog		*dlg;
+	GtkWidget	*hbox, *label, *entry;
 
-	if((page != NULL) && (page->curr_style != NULL))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	label = gtk_label_new(_("Name"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	entry = gtk_entry_new_with_max_length(STL_PROPERTY_NAME_SIZE - 1);
+	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+	dlg = dlg_dialog_sync_new(hbox, _("New Action Property"), NULL);
+	gtk_widget_grab_focus(entry);
+	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 	{
-		Dialog		*dlg;
-		GtkWidget	*hbox, *label, *entry;
+		const gchar	*name;
 
-		hbox = gtk_hbox_new(FALSE, 0);
-		label = gtk_label_new(_("Name"));
-		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-		entry = gtk_entry_new_with_max_length(STL_PROPERTY_NAME_SIZE - 1);
-		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-		dlg = dlg_dialog_sync_new(hbox, _("New Action Property"), NULL);
-		gtk_widget_grab_focus(entry);
-		if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
+		if(((name = gtk_entry_get_text(GTK_ENTRY(entry))) != NULL) && *name != '\0')
 		{
-			const gchar	*name;
-
-			if((name = gtk_entry_get_text(GTK_ENTRY(entry))) != NULL)
-			{
-				stl_style_property_set_action(page->curr_style, name, _("something"));
-				set_widgets_action(page, page->curr_style);
-				reset_widgets_action(page);
-				page->modified = TRUE;
-			}
+			stl_style_property_set_action(style, name, _("something"));
+			set_widgets_action(page, style);
+			reset_widgets_action(page);
+			page->modified = TRUE;
 		}
-		dlg_dialog_sync_destroy(dlg);
 	}
+	dlg_dialog_sync_destroy(dlg);
 }
 
 /* 1999-05-25 -	User clicked the "Delete" (or "Revert...") button. Do it. */
 static void evt_aprop_delete_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
 
-	if((page != NULL) && (page->curr_style != NULL) && (page->curr_prop != NULL) && (page->curr_arow >= 0))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	if((page->curr_prop != NULL) && (page->curr_arow >= 0))
 	{
-		stl_style_property_remove(page->curr_style, page->curr_prop);
+		stl_style_property_remove(style, page->curr_prop);
 		gtk_clist_unselect_all(GTK_CLIST(page->dpaction.actions));
-		set_widgets_action(page, page->curr_style);
+		set_widgets_action(page, style);
 		reset_widgets_action(page);
 		page->modified = TRUE;
 	}
@@ -524,18 +592,22 @@ static void evt_style_add_clicked(GtkWidget *wid, gpointer user)
 
 	if(page != NULL)
 	{
-		Style		*stl, *pstl;
-		GtkWidget	*child;
+		Style		*stl, *parent;
+		GtkTreeIter	iter;
 
 		stl = stl_style_new_unique_name(page->si);
-		if((pstl = page->curr_style) == NULL)
-			pstl = stl_styleinfo_style_root(page->si);
-		stl_styleinfo_style_add(page->si, pstl, stl);
+		/* Parent is either current selection, or root. */
+		if((parent = style_get_selected(page, &iter)) == NULL)
+			parent = stl_styleinfo_style_root(page->si);
+		stl_styleinfo_style_add(page->si, parent, stl);
+		stl_style_set_expand(parent, TRUE);
 		populate_tree(page);
-		if((child = stl_styleinfo_widget_find(page->si, GTK_TREE(page->tree), stl)) != NULL)
-			gtk_tree_select_child(GTK_TREE(page->tree), child);
-		gtk_entry_select_region(GTK_ENTRY(page->dname), 0, -1);
-		gtk_widget_grab_focus(page->dname);
+		if(stl_styleinfo_tree_find_style(page->si, page->store, stl, &iter))
+		{
+			gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->tree)), &iter);
+			gtk_entry_select_region(GTK_ENTRY(page->dname), 0, -1);
+			gtk_widget_grab_focus(page->dname);
+		}
 	}
 }
 
@@ -545,28 +617,28 @@ static void evt_style_add_clicked(GtkWidget *wid, gpointer user)
 static void evt_style_del_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Styles	*page = user;
+	Style		*style;
+	gint		ok = DLG_POSITIVE;
 
-	if((page != NULL) && (page->curr_style != NULL))
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+
+	if(stl_styleinfo_style_has_children(page->si, style))
+		ok = dlg_dialog_sync_new_simple_wait(_("Deleting this style will also delete\n"
+			"all its children. Are you sure?"), _("Confirm Delete"), _("_Delete|_Cancel"));
+	if(ok == DLG_POSITIVE)
 	{
-		gint	ok = DLG_POSITIVE;
+		GList	*chlist, *iter;
+		Style	*stl;
 
-		if(stl_styleinfo_style_has_children(page->si, page->curr_style))
-			ok = dlg_dialog_sync_new_simple_wait(_("Deleting this style will also delete\n"
-				"all its children. Are you sure?"), _("Confirm Delete"), _("_Delete|_Cancel"));
-		if(ok == DLG_POSITIVE)
-		{
-			GList	*chlist, *iter;
-			Style	*stl;
-
-			stl = stl_styleinfo_style_root(page->si);
-			chlist = stl_styleinfo_style_get_children(page->si, page->curr_style, TRUE);
-			for(iter = chlist; iter != NULL; iter = g_list_next(iter))
-				ctp_replace_style(stl_style_get_name(iter->data), stl);
-			g_list_free(chlist);
-			stl_styleinfo_style_remove(page->si, page->curr_style);
-			populate_tree(page);
-			reset_widgets(page);
-		}
+		stl = stl_styleinfo_style_root(page->si);
+		chlist = stl_styleinfo_style_get_children(page->si, style, TRUE);
+		for(iter = chlist; iter != NULL; iter = g_list_next(iter))
+			ctp_replace_style(stl_style_get_name(iter->data), stl);
+		g_list_free(chlist);
+		stl_styleinfo_style_remove(page->si, style);
+		populate_tree(page);
+		reset_widgets(page);
 	}
 }
 
@@ -603,11 +675,11 @@ static void build_pvisual(P_Styles *page)
 		gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 		pv->override[i] = gtk_check_button_new_with_label(_("Override Parent's?"));
 		gtk_object_set_user_data(GTK_OBJECT(pv->override[i]), (gpointer) vpname[i]);
-		gtk_signal_connect(GTK_OBJECT(pv->override[i]), "toggled", GTK_SIGNAL_FUNC(evt_vprop_override_toggled), page);
+		g_signal_connect(G_OBJECT(pv->override[i]), "toggled", G_CALLBACK(evt_vprop_override_toggled), page);
 		gtk_box_pack_start(GTK_BOX(vbox), pv->override[i], FALSE, FALSE, 0);
 		pv->edit[i] = gtk_button_new_with_label(i < 2 ? _("Edit...") : _("Pick..."));
 		gtk_object_set_user_data(GTK_OBJECT(pv->edit[i]), (gpointer) vpname[i]);
-		gtk_signal_connect(GTK_OBJECT(pv->edit[i]), "clicked", GTK_SIGNAL_FUNC(evt_vprop_edit_clicked), page);
+		g_signal_connect(G_OBJECT(pv->edit[i]), "clicked", G_CALLBACK(evt_vprop_edit_clicked), page);
 		gtk_box_pack_start(GTK_BOX(vbox), pv->edit[i], FALSE, FALSE, 0);
 		gtk_container_add(GTK_CONTAINER(frame), vbox);
 		gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 5);
@@ -626,7 +698,6 @@ static void build_paction(P_Styles *page)
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	pa->actions = gtk_clist_new(2);
-	gtk_widget_set_usize(pa->actions, -1, 48);
 	gtk_clist_set_column_justification(GTK_CLIST(pa->actions), 0, GTK_JUSTIFY_LEFT);
 	gtk_clist_set_column_justification(GTK_CLIST(pa->actions), 1, GTK_JUSTIFY_LEFT);
 	gtk_clist_set_column_auto_resize(GTK_CLIST(pa->actions), 0, TRUE);
@@ -640,8 +711,8 @@ static void build_paction(P_Styles *page)
 	gtk_clist_append(GTK_CLIST(pa->actions), at);
 	gtk_clist_append(GTK_CLIST(pa->actions), at);
 	gtk_clist_append(GTK_CLIST(pa->actions), at);
-	gtk_signal_connect(GTK_OBJECT(pa->actions), "select_row", GTK_SIGNAL_FUNC(evt_aprop_selected), page);
-	gtk_signal_connect(GTK_OBJECT(pa->actions), "unselect_row", GTK_SIGNAL_FUNC(evt_aprop_unselected), page);
+	g_signal_connect(G_OBJECT(pa->actions), "select_row", G_CALLBACK(evt_aprop_selected), page);
+	g_signal_connect(G_OBJECT(pa->actions), "unselect_row", G_CALLBACK(evt_aprop_unselected), page);
 	gtk_container_add(GTK_CONTAINER(scwin), pa->actions);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), scwin, TRUE, TRUE, 0);
 
@@ -650,24 +721,24 @@ static void build_paction(P_Styles *page)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
 	pa->aname = gtk_entry_new_with_max_length(STL_PROPERTY_NAME_SIZE - 1);
 	gtk_entry_set_editable(GTK_ENTRY(pa->aname), FALSE);
-	gtk_signal_connect(GTK_OBJECT(pa->aname), "changed", GTK_SIGNAL_FUNC(evt_aprop_name_changed), page);
+	g_signal_connect(G_OBJECT(pa->aname), "changed", G_CALLBACK(evt_aprop_name_changed), page);
 	gtk_table_attach(GTK_TABLE(table), pa->aname, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
 	pa->acmdseq = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(pa->acmdseq), "changed", GTK_SIGNAL_FUNC(evt_aprop_cmdseq_changed), page);
+	g_signal_connect(G_OBJECT(pa->acmdseq), "changed", G_CALLBACK(evt_aprop_cmdseq_changed), page);
 	gtk_table_attach(GTK_TABLE(table), pa->acmdseq, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
 	pa->acmdpick = gui_details_button_new(page->min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(pa->acmdpick), "clicked", GTK_SIGNAL_FUNC(evt_aprop_cmdpick_clicked), page);
+	g_signal_connect(G_OBJECT(pa->acmdpick), "clicked", G_CALLBACK(evt_aprop_cmdpick_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), pa->acmdpick, 2, 3, 1, 2,  0,0,0,0);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), table, FALSE, FALSE, 0);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	btn = gtk_button_new_with_label(_("Add Action..."));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_aprop_add_clicked), page);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_aprop_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 5);
 	pa->adel = gtk_button_new_with_label(_("Delete Action"));
-	gtk_signal_connect(GTK_OBJECT(pa->adel), "clicked", GTK_SIGNAL_FUNC(evt_aprop_delete_clicked), page);
+	g_signal_connect(G_OBJECT(pa->adel), "clicked", G_CALLBACK(evt_aprop_delete_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), pa->adel, TRUE, TRUE, 5);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), hbox, FALSE, FALSE, 5);
 }
@@ -677,15 +748,25 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 {
 	P_Styles	*page = &the_page;
 	GtkWidget	*table, *label, *frame, *hbox, *btn;
+	GtkCellRenderer	*cr;
+	GtkTreeViewColumn *vc;
 
 	page->min = min;
 	page->si  = NULL;
-	page->curr_style = NULL;
 	page->modified = FALSE;
-	page->tree = NULL;
+	page->store = NULL;
 
 	page->vbox = gtk_vbox_new(FALSE, 0);
 	page->scwin = gtk_scrolled_window_new(FALSE, FALSE);
+	page->tree = gtk_tree_view_new();
+	cr = gtk_cell_renderer_text_new();
+	vc = gtk_tree_view_column_new_with_attributes("(Styles)", cr, "text", 0, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(page->tree), vc);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(page->tree), FALSE);
+	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->tree))), "changed", G_CALLBACK(evt_style_selection_changed), page);
+	page->sig_expand   = g_signal_connect(G_OBJECT(page->tree), "row_expanded", G_CALLBACK(evt_style_row_expanded), page);
+	page->sig_collapse = g_signal_connect(G_OBJECT(page->tree), "row_collapsed", G_CALLBACK(evt_style_row_collapsed), page);
+	gtk_container_add(GTK_CONTAINER(page->scwin), page->tree);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
 
@@ -695,7 +776,7 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	label = gtk_label_new(_("Name"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
 	page->dname = gtk_entry_new_with_max_length(STL_STYLE_NAME_SIZE - 1);
-	gtk_signal_connect(GTK_OBJECT(page->dname), "changed", GTK_SIGNAL_FUNC(evt_style_name_changed), page);
+	g_signal_connect(G_OBJECT(page->dname), "changed", G_CALLBACK(evt_style_name_changed), page);
 	gtk_table_attach(GTK_TABLE(table), page->dname, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
 	label = gtk_label_new(_("Parent"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
@@ -703,7 +784,7 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	gtk_entry_set_editable(GTK_ENTRY(page->dparent), FALSE);
 	gtk_table_attach(GTK_TABLE(table), page->dparent, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
 	page->dreparent = gui_details_button_new(min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(page->dreparent), "clicked", GTK_SIGNAL_FUNC(evt_style_parent_clicked), page);
+	g_signal_connect(G_OBJECT(page->dreparent), "clicked", G_CALLBACK(evt_style_parent_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), page->dreparent, 2, 3, 1, 2,  0,0,0,0);
 	gtk_box_pack_start(GTK_BOX(page->dvbox), table, FALSE, FALSE, 0);
 
@@ -721,10 +802,10 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	btn = gtk_button_new_with_label(_("Add"));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_style_add_clicked), page);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_style_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 5);
 	page->del = gtk_button_new_with_label(_("Delete"));
-	gtk_signal_connect(GTK_OBJECT(page->del), "clicked", GTK_SIGNAL_FUNC(evt_style_del_clicked), page);
+	g_signal_connect(G_OBJECT(page->del), "clicked", G_CALLBACK(evt_style_del_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->del, TRUE, TRUE, 5);
 	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 5);
 
