@@ -5,6 +5,8 @@
 
 #include "gentoo.h"
 
+#include <stdlib.h>
+
 #include "types.h"
 #include "styles.h"
 #include "style_dialog.h"
@@ -33,8 +35,8 @@ typedef struct {		/* Just a little helper structure for the identification. */
 
 typedef struct {
 	GtkWidget	*vbox;		/* The usual mandatory root vbox. */
-	GtkWidget	*scwin;		/* Scrolled window wrapping main clist. */
-	GtkWidget	*clist;		/* Clist showing existing types. */
+	GtkListStore	*store;		/* Tree model holding existing types. */
+	GtkWidget	*view;		/* Tree view. */
 
 	GtkWidget	*ntable;	/* Table holding the type name widgetry. */
 	GtkWidget	*name;		/* Entry widget for type name. */
@@ -65,18 +67,58 @@ typedef struct {
 	MainInfo	*min;
 	GList		*type;		/* List of editing types. */
 	FType		*curr_type;	/* Currently selected type. */
-	gint		row;		/* Clist row index of currently selected type. */
 	gboolean	modified;	/* Did user change anything? */
 } P_Types;
 
+enum {
+	COLUMN_NAME,
+	COLUMN_TYPE,
+	COLUMN_COUNT
+};
+
 /* ----------------------------------------------------------------------------------------- */
 
 static P_Types	the_page;
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2009-03-15 -	Helper routine, aiming to remove the 'curr_type' field in the page since it
+**		feels a bit smelly. This just queries the tree view for the selected row,
+**		and extracts the type column which is returned. The iter to the selection is
+**		returned too, if non-NULL. If no selection exists, NULL is returned.
+*/
+static FType * get_current_type(P_Types *page, GtkTreeIter *iter)
+{
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view)), NULL, iter))
+	{
+		FType	*type = NULL;
+
+		gtk_tree_model_get(GTK_TREE_MODEL(page->store), iter, COLUMN_TYPE, &type, -1);
+		return type;
+	}
+	return NULL;
+}
+
+static void set_movement_widgets(P_Types *page, GtkTreeIter *iter)
+{
+	gchar	*istr;
+
+	/* Set Up/Down arrow states depending on index of current selection.
+	** Ask GTK+ for the numeric "address" of the current selection, then figure it out.
+	*/
+	if((istr = gtk_tree_model_get_string_from_iter(GTK_TREE_MODEL(page->store), iter)) != NULL)
+	{
+		gint	n = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(page->store), NULL), m;
+
+		m = strtol(istr, NULL, 10);
+		gtk_widget_set_sensitive(page->up, m > 0 && m < n - 1);
+		gtk_widget_set_sensitive(page->down, m < n - 2);
+		g_free(istr);
+	}
+}
+
 /* 1998-08-13 -	Set the widgets so they reflect the settings for the given file type. */
-static void set_widgets(P_Types *page, FType *tpe)
+static void set_widgets(P_Types *page, FType *tpe, GtkTreeIter *iter)
 {
 	gchar	*itext[3];
 	gint	iflags[] = { FTFL_REQSUFFIX, FTFL_NAMEMATCH, FTFL_FILEMATCH },
@@ -150,9 +192,8 @@ static void set_widgets(P_Types *page, FType *tpe)
 			gtk_label_set_text(GTK_LABEL(page->slabel), buf);
 		}
 		gtk_widget_set_sensitive(page->sframe, TRUE);
-		gtk_widget_set_sensitive(page->up, page->row != 0 && !TYP_IS_UNKNOWN(tpe));
-		gtk_widget_set_sensitive(page->down, !TYP_IS_UNKNOWN(tpe) && ((guint) page->row < g_list_length(page->type) - 2));
 		gtk_widget_set_sensitive(page->del, tpe->mode != 0);
+		set_movement_widgets(page, iter);
 	}
 }
 
@@ -162,7 +203,6 @@ static void reset_widgets(P_Types *page)
 	gint	i;
 
 	page->curr_type = NULL;
-	page->row	= -1;
 	gtk_entry_set_text(GTK_ENTRY(page->name), "");
 	gtk_widget_set_sensitive(page->ntable, FALSE);
 
@@ -197,26 +237,20 @@ static void reset_widgets(P_Types *page)
 }
 
 /* 1998-12-13 -	Broke out the actual list updating code, since it's now needed more. */
-static void update_clist(P_Types *page)
+static void update_list(P_Types *page)
 {
-	gchar	*text[] = { NULL };
-	gint	row;
-	GList	*iter;
+	const GList	*iter;
+	GtkTreeIter	ti;
 
-	gtk_clist_freeze(GTK_CLIST(page->clist));
-	gtk_clist_clear(GTK_CLIST(page->clist));
+	gtk_list_store_clear(page->store);
 	for(iter = page->type; iter != NULL; iter = g_list_next(iter))
 	{
-		text[0] = ((FType *) iter->data)->name;
-		row = gtk_clist_append(GTK_CLIST(page->clist), text);
-		gtk_clist_set_row_data(GTK_CLIST(page->clist), row, iter->data);
+		gtk_list_store_append(page->store, &ti);
+		gtk_list_store_set(page->store, &ti, COLUMN_NAME, ((FType *) iter->data)->name, COLUMN_TYPE, iter->data, -1);
 	}
-	gtk_clist_thaw(GTK_CLIST(page->clist));
 }
 
-/* 1998-08-13 -	Copy the given type (at <data>) and add it to the editing-copy list. Also
-**		append a row with the type's name to the clist. 
-*/
+/* 2009-03-15 -	Old code refactored a bit. Just copy an FType and insert in page's editing list. */
 static void copy_type(gpointer data, gpointer user)
 {
 	P_Types	*page = user;
@@ -229,43 +263,31 @@ static void copy_type(gpointer data, gpointer user)
 		page->type = typ_type_insert(page->type, NULL, nt);
 }
 
-/* 1998-08-13 -	Copy the current type definitions and display the copies in the main clist. */
-static void populate_clist(MainInfo *min, P_Types *page)
+/* 1998-08-13 -	Copy the current type definitions and display the copies in the main list. */
+static void populate_list(MainInfo *min, P_Types *page)
 {
-	gtk_clist_clear(GTK_CLIST(page->clist));
 	reset_widgets(page);
 
 	g_list_foreach(min->cfg.type, copy_type, page);
-	update_clist(page);
+	update_list(page);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-08-13 -	Something was selected in the main clist. Update editing widgets. */
-static gint evt_clist_select(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
+/* 2009-03-15 -	Selection changed in main type list, so update editing widgetry. */
+static void evt_selection_changed(GtkTreeSelection *ts, gpointer user)
 {
-	P_Types	*page = user;
-	FType	*tpe;
+	P_Types		*page = user;
+	GtkTreeIter	iter;
+	FType		*type = NULL;
 
-	if((tpe = gtk_clist_get_row_data(GTK_CLIST(wid), row)) != NULL)
+	if((type = get_current_type(page, &iter)) != NULL)
 	{
-		page->curr_type = tpe;
-		page->row	= row;
-		set_widgets(page, tpe);
+		set_widgets(page, type, &iter);
+		page->curr_type = type;
 	}
-	return TRUE;
-}
-
-/* 1998-08-13 -	This gets called when there is no longer a selection in the clist. */
-static gint evt_clist_unselect(GtkWidget *wid, gint row, gint column, GdkEventButton *evt, gpointer user)
-{
-	P_Types	*page = user;
-
-	page->curr_type = NULL;		/* Must be done _before_ reset_widgets()! Tricky! */
-	page->row = -1;
-	reset_widgets(page);	
-
-	return TRUE;
+	else
+		reset_widgets(page);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -275,15 +297,19 @@ static gint evt_clist_unselect(GtkWidget *wid, gint row, gint column, GdkEventBu
 */
 static gint evt_name_changed(GtkWidget *wid, gpointer user)
 {
-	P_Types	*page = user;
-	gchar	*text;
+	P_Types		*page = user;
+	const gchar	*text;
+	GtkTreeIter	iter;
+	FType		*type = NULL;
 
-	if(page->curr_type != NULL)
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view)), NULL, &iter))
+		gtk_tree_model_get(GTK_TREE_MODEL(page->store), &iter, COLUMN_TYPE, &type, -1);
+	if(type != NULL)
 	{
-		page->modified = TRUE;
 		text = gtk_entry_get_text(GTK_ENTRY(wid));
-		page->type = typ_type_set_name(page->type, page->curr_type, text);
-		gtk_clist_set_text(GTK_CLIST(page->clist), page->row, 0, text);
+		page->type = typ_type_set_name(page->type, type, text);
+		gtk_list_store_set(page->store, &iter, COLUMN_NAME, text, -1);
+		page->modified = TRUE;
 	}
 	return TRUE;
 }
@@ -321,7 +347,7 @@ static gint evt_id_reqperm_clicked(GtkWidget *wid, gpointer user)
 		page->curr_type->flags |= FTFL_REQPERM;
 	else
 		page->curr_type->flags &= ~FTFL_REQPERM;
-	
+
 	return TRUE;
 }
 
@@ -377,9 +403,9 @@ static gint evt_id_check_clicked(GtkWidget *wid, gpointer user)
 
 static gint evt_id_entry_changed(GtkWidget *wid, gpointer user)
 {
-	P_Types	*page = user;
-	gchar	*text;
-	gint	index = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid)));
+	P_Types		*page = user;
+	const gchar	*text;
+	gint		index = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid)));
 
 	if(page->curr_type == NULL)
 		return TRUE;
@@ -447,16 +473,19 @@ static gint evt_id_nocase_changed(GtkWidget *wid, gpointer user)
 /* 1999-05-27 -	Select style for current type. Rewritten, to work with new style config module. */
 static gint evt_style_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Types	*page = user;
+	P_Types		*page = user;
+	GtkTreeIter	iter;
+	FType		*type;
 
-	if((page != NULL) && (page->curr_type != NULL))
+	if((type = get_current_type(page, &iter)) != NULL)
 	{
 		Style	*stl;
 
 		if((stl = sdl_dialog_sync_new_wait(cst_get_styleinfo(), NULL)) != NULL)
 		{
-			page->curr_type->style = stl;
-			set_widgets(page, page->curr_type);
+			type->style = stl;
+			set_widgets(page, type, &iter);
+			page->modified = TRUE;
 		}
 	}
 	return TRUE;
@@ -471,19 +500,54 @@ static gint evt_add_clicked(GtkWidget *wid, gpointer user)
 {
 	P_Types	*page = user;
 	CfgInfo	*cfg  = gtk_object_get_user_data(GTK_OBJECT(wid));
-	FType	*tpe;
-	gint	row;
+	FType	*tpe, *otpe;
 
 	if((tpe = typ_type_new(cfg, _("(New Type)"), S_IFREG, 0, NULL, NULL, NULL)) != NULL)
 	{
-		page->type = typ_type_insert(page->type, page->curr_type, tpe);
-		page->type = typ_type_set_style(page->type, tpe, cst_get_styleinfo(), NULL);
-		row = g_list_index(page->type, tpe);
-		update_clist(page);
-		gtk_clist_select_row(GTK_CLIST(page->clist), row, 0);
-		gtk_clist_moveto(GTK_CLIST(page->clist), row, -1, 0.5f, 0.0f);
-		gtk_entry_select_region(GTK_ENTRY(page->name), 0, -1);
+		GtkTreeIter	oiter, iter;	/* Current and new. */
+		GtkTreePath	*path;
+
+		/* Do some fairly advanced hackery-pokery to avoid having to rebuild the
+		** entire ListStore. Instead add in the proper place in both Type list and
+		** the ListStore.
+		*/
+		if((otpe = get_current_type(page, &oiter)) != NULL)
+		{
+			/* Current selection exists; add after it. */
+			page->type = typ_type_insert(page->type, otpe, tpe);
+			gtk_list_store_insert_after(page->store, &oiter, &iter);
+		}
+		else
+		{
+			/* No current selection; insert right before the UNKNOWN at the end. */
+			GtkTreeIter	unknown;
+			gint		len;
+
+			/* Figure out how many types there are. */
+			len = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(page->store), NULL);
+			/* Get an iter to the last one, which is "Unknown". This is (I guess) O(n),
+			** but that 'n' will be quite short so this is easily quick enough.
+			*/
+			gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(page->store), &unknown, NULL, len - 1);
+			page->type = typ_type_insert(page->type, NULL, tpe);
+			gtk_list_store_insert_before(page->store, &iter, &unknown);
+		}
+		/* Set the proper values in the ListStore. */
+		gtk_list_store_set(page->store, &iter, COLUMN_NAME, tpe->name, COLUMN_TYPE, tpe, -1);
+
+		/* Select the new type. */
+		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view)), &iter);
+		/* Scroll list to new type. */
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(page->store), &iter);
+		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(page->view), path, NULL, TRUE, 0.5f, 0.0f);
+		gtk_tree_path_free(path);
+
+		typ_type_set_style(page->type, tpe, cst_get_styleinfo(), NULL);
+		gtk_list_store_set(page->store, &iter, COLUMN_NAME, tpe->name, COLUMN_TYPE, tpe, -1);
+
+		/* Put focus in the Name field, highly likely a rename is in order. */
 		gtk_widget_grab_focus(page->name);
+
 		page->modified = TRUE;
 	}
 	return TRUE;
@@ -494,17 +558,17 @@ static gint evt_add_clicked(GtkWidget *wid, gpointer user)
 */
 static gint evt_del_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Types	*page = user;
-	gint	row, len;
+	P_Types		*page = user;
+	GtkTreeIter	iter;
 
-	if(page->curr_type != NULL)
+	/* Don't rebuild ListStore; surgically remove just the proper row. */
+	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view)), NULL, &iter))
 	{
-		page->type = typ_type_remove(page->type, page->curr_type);
-		len = g_list_length(page->type);
-		row = (page->row > len) ? len - 1 : page->row;
-		update_clist(page);
-		gtk_clist_select_row(GTK_CLIST(page->clist), row, 0);
-		gtk_clist_moveto(GTK_CLIST(page->clist), row, -1, 0.5f, 0.0f);
+		FType	*type;
+
+		gtk_tree_model_get(GTK_TREE_MODEL(page->store), &iter, COLUMN_TYPE, &type, -1);
+		page->type = typ_type_remove(page->type, type);
+		gtk_list_store_remove(page->store, &iter);
 		page->modified = TRUE;
 	}
 	return TRUE;
@@ -512,34 +576,58 @@ static gint evt_del_clicked(GtkWidget *wid, gpointer user)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-12-13 -	Move selected type up one position (increases its priority). */
+/* 2009-03-16 -	Rewritten to be a bit more efficient, while using the new GTK+ 2 tree stuff, too. */
 static gint evt_up_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Types	*page = user;
+	P_Types		*page = user;
+	GtkTreeIter	iter, piter;
+	FType		*type;
 
-	if(page->curr_type != NULL)
+	if((type = get_current_type(page, &iter)) != NULL)
 	{
-		page->type = typ_type_move(page->type, page->curr_type, -1);
-		update_clist(page);
-		gtk_clist_select_row(GTK_CLIST(page->clist), page->row - 1, 0);
-		gtk_clist_moveto(GTK_CLIST(page->clist), page->row - 1, -1, 0.0f, 0.0f);
-		page->modified = TRUE;
+		GtkTreePath	*path;
+
+		/* Moving up in the list requires a path, for the prev() stepping. */
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(page->store), &iter);
+		if(gtk_tree_path_prev(path))
+		{
+			/* Do the move. */
+			page->type = typ_type_move(page->type, page->curr_type, -1);
+			gtk_tree_model_get_iter(GTK_TREE_MODEL(page->store), &piter, path);
+			gtk_list_store_move_before(page->store, &iter, &piter);
+			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(page->view), path, NULL, TRUE, 0.5f, 0.0f);
+			page->modified = TRUE;
+			set_movement_widgets(page, &iter);
+		}
+		gtk_tree_path_free(path);
 	}
 	return TRUE;
 }
 
-/* 1998-12-13 -	Move selected type down, decreasing its priority. */
+/* 2009-03-16 -	Rewritten to be a bit more efficient, while using the new GTK+ 2 tree stuff, too. */
 static gint evt_down_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Types	*page = user;
+	P_Types		*page = user;
+	GtkTreeIter	iter, niter;
+	FType		*type;
 
-	if(page->curr_type != NULL)
+	if((type = get_current_type(page, &iter)) != NULL)
 	{
+		GtkTreePath	*path;
+
+		/* Scrolling requires a path, so let's do it that way. */
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(page->store), &iter);
+		gtk_tree_path_next(path);
+		/* Then do the move. */
 		page->type = typ_type_move(page->type, page->curr_type, 1);
-		update_clist(page);
-		gtk_clist_select_row(GTK_CLIST(page->clist), page->row + 1, 0);
-		gtk_clist_moveto(GTK_CLIST(page->clist), page->row + 1, -1, 1.0f, 0.0f);
-		page->modified = TRUE;
+		if(gtk_tree_model_get_iter(GTK_TREE_MODEL(page->store), &niter, path))
+		{
+			gtk_list_store_move_after(page->store, &iter, &niter);
+			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(page->view), path, NULL, TRUE, 0.5f, 0.0f);
+			page->modified = TRUE;
+			set_movement_widgets(page, &iter);
+		}
+		gtk_tree_path_free(path);
 	}
 	return TRUE;
 }
@@ -548,38 +636,45 @@ static gint evt_down_clicked(GtkWidget *wid, gpointer user)
 
 static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 {
-	P_Types		*page = &the_page;
-	GtkWidget	*label, *vbox, *sep;
-	const gchar	*tlab[] = { N_("File"), N_("Dir"), N_("Link"), N_("B-Dev"), N_("C-Dev"), N_("FIFO"), N_("Socket") },
-			*iplab[] = { N_("SetUID"), N_("SetGID"), N_("Sticky"),  N_("Readable"),
-					N_("Writable"), N_("Executable") },
-			*idlab[] = { N_("Require Suffix"), N_("Match Name (RE)"), N_("Match 'file' (RE)") };
-	gint		idmax[] = { FT_SUFFIX_SIZE, FT_NAMERE_SIZE, FT_FILERE_SIZE },
-			ifperm[] = { FTPM_SETUID, FTPM_SETGID, FTPM_STICKY, FTPM_READ, FTPM_WRITE, FTPM_EXECUTE }, i, y;
-	static mode_t	type[] = { S_IFREG, S_IFDIR, S_IFLNK, S_IFBLK, S_IFCHR, S_IFIFO, S_IFSOCK };
+	P_Types			*page = &the_page;
+	GtkWidget		*scwin, *label, *vbox, *sep;
+        GtkCellRenderer		*cr;
+        GtkTreeViewColumn	*vc;
+        GtkTreeSelection	*ts;
+	const gchar		*tlab[] = { N_("File"), N_("Dir"), N_("Link"), N_("B-Dev"), N_("C-Dev"), N_("FIFO"), N_("Socket") },
+				*iplab[] = { N_("SetUID"), N_("SetGID"), N_("Sticky"),  N_("Readable"),
+						N_("Writable"), N_("Executable") },
+						*idlab[] = { N_("Require Suffix"), N_("Match Name (RE)"), N_("Match 'file' (RE)") };
+	gint			idmax[] = { FT_SUFFIX_SIZE, FT_NAMERE_SIZE, FT_FILERE_SIZE },
+				ifperm[] = { FTPM_SETUID, FTPM_SETGID, FTPM_STICKY, FTPM_READ, FTPM_WRITE, FTPM_EXECUTE }, i, y;
+	static mode_t		type[] = { S_IFREG, S_IFDIR, S_IFLNK, S_IFBLK, S_IFCHR, S_IFIFO, S_IFSOCK };
 
 	page->min = min;
 
 	page->type = NULL;
 	page->curr_type = NULL;
-	page->row       = -1;
 	page->modified  = FALSE;
 
 	page->vbox = gtk_vbox_new(FALSE, 0);
 
-	page->scwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	page->clist = gtk_clist_new(1);
-	gtk_signal_connect(GTK_OBJECT(page->clist), "select_row", GTK_SIGNAL_FUNC(evt_clist_select), page);
-	gtk_signal_connect(GTK_OBJECT(page->clist), "unselect_row", GTK_SIGNAL_FUNC(evt_clist_unselect), page);
-	gtk_container_add(GTK_CONTAINER(page->scwin), page->clist);
-	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
+	scwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	page->store = gtk_list_store_new(COLUMN_COUNT, G_TYPE_STRING, G_TYPE_POINTER);
+	page->view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(page->store));
+        cr = gtk_cell_renderer_text_new();
+        vc = gtk_tree_view_column_new_with_attributes("(name)", cr, "text", COLUMN_NAME, NULL);
+        gtk_tree_view_append_column(GTK_TREE_VIEW(page->view), vc);
+        gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(page->view), FALSE);
+        ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view));
+        g_signal_connect(G_OBJECT(ts), "changed", G_CALLBACK(evt_selection_changed), page);
+        gtk_container_add(GTK_CONTAINER(scwin), page->view);
+        gtk_box_pack_start(GTK_BOX(page->vbox), scwin, TRUE, TRUE, 0);
 
 	page->ntable = gtk_table_new(2, 2, FALSE);
 	label = gtk_label_new(_("Name"));
 	gtk_table_attach(GTK_TABLE(page->ntable), label, 0, 1, 0, 1,  GTK_FILL,0,5,0);
 	page->name = gtk_entry_new_with_max_length(STL_STYLE_NAME_SIZE - 1);
-	page->sig_name = gtk_signal_connect(GTK_OBJECT(page->name), "changed", GTK_SIGNAL_FUNC(evt_name_changed), page);
+	page->sig_name = g_signal_connect(G_OBJECT(page->name), "changed", G_CALLBACK(evt_name_changed), page);
 	gtk_table_attach_defaults(GTK_TABLE(page->ntable), page->name, 1, 2, 0, 1);
 
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->ntable, FALSE, FALSE, 0);
@@ -594,21 +689,21 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 	page->itmode = type;
 	for(i = 0; i < 7; i++)
 	{
-		gtk_signal_connect(GTK_OBJECT(page->itype[i]), "clicked", GTK_SIGNAL_FUNC(evt_mode_clicked), page);
+		g_signal_connect(G_OBJECT(page->itype[i]), "clicked", G_CALLBACK(evt_mode_clicked), page);
 		gtk_object_set_user_data(GTK_OBJECT(page->itype[i]), GINT_TO_POINTER(i));
 		gtk_box_pack_start(GTK_BOX(page->ithbox), page->itype[i], TRUE, TRUE, 0);
 	}
 	gtk_table_attach(GTK_TABLE(page->itable), page->ithbox, 1, 3, 0, 1, GTK_EXPAND|GTK_FILL,0,0,0);
 
 	page->irperm = gtk_check_button_new_with_label(_("Require Protection"));
-	gtk_signal_connect(GTK_OBJECT(page->irperm), "clicked", GTK_SIGNAL_FUNC(evt_id_reqperm_clicked), page);
+	g_signal_connect(G_OBJECT(page->irperm), "clicked", G_CALLBACK(evt_id_reqperm_clicked), page);
 	gtk_table_attach(GTK_TABLE(page->itable), page->irperm, 0, 1, 1, 2,  GTK_FILL,0,0,0);
 	page->iphbox = gtk_hbox_new(FALSE, 0);
 	for(i = 0; i < 6; i++)
 	{
 		page->iperm[i] = gtk_toggle_button_new_with_label(_(iplab[i]));
 		gtk_object_set_user_data(GTK_OBJECT(page->iperm[i]), GINT_TO_POINTER(ifperm[i]));
-		gtk_signal_connect(GTK_OBJECT(page->iperm[i]), "clicked", GTK_SIGNAL_FUNC(evt_id_perm_clicked), page);
+		g_signal_connect(G_OBJECT(page->iperm[i]), "clicked", G_CALLBACK(evt_id_perm_clicked), page);
 		gtk_box_pack_start(GTK_BOX(page->iphbox), page->iperm[i], TRUE, TRUE, 0);
 	}
 	gtk_table_attach(GTK_TABLE(page->itable), page->iphbox, 1, 3, 1, 2,  GTK_EXPAND | GTK_FILL,0,0,0);
@@ -622,12 +717,12 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 
 		page->ident[i].check = gtk_check_button_new_with_label(_(idlab[i]));
 		gtk_object_set_user_data(GTK_OBJECT(page->ident[i].check), GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->ident[i].check), "clicked", GTK_SIGNAL_FUNC(evt_id_check_clicked), page);
+		g_signal_connect(G_OBJECT(page->ident[i].check), "clicked", G_CALLBACK(evt_id_check_clicked), page);
 		gtk_table_attach(GTK_TABLE(page->itable), page->ident[i].check, 0, 1, y, y+1,  GTK_FILL,0,0,0);
 
 		page->ident[i].entry = gtk_entry_new_with_max_length(idmax[i] - 1);
 		gtk_object_set_user_data(GTK_OBJECT(page->ident[i].entry), GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->ident[i].entry), "changed", GTK_SIGNAL_FUNC(evt_id_entry_changed), page);
+		g_signal_connect(G_OBJECT(page->ident[i].entry), "changed", G_CALLBACK(evt_id_entry_changed), page);
 		gtk_table_attach(GTK_TABLE(page->itable), page->ident[i].entry, 1, (i == 0) ? 3 : 2, y, y+1,  GTK_EXPAND | GTK_FILL,0,0,0);
 
 		if(i >= 1)
@@ -637,12 +732,12 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 			hbox = gtk_hbox_new(FALSE, 0);
 			page->ident[i].glob = gtk_check_button_new_with_label(_("Glob?"));
 			gtk_object_set_user_data(GTK_OBJECT(page->ident[i].glob), GINT_TO_POINTER(i));
-			gtk_signal_connect(GTK_OBJECT(page->ident[i].glob), "clicked", GTK_SIGNAL_FUNC(evt_id_glob_changed), page);
+			g_signal_connect(G_OBJECT(page->ident[i].glob), "clicked", G_CALLBACK(evt_id_glob_changed), page);
 			gtk_box_pack_start(GTK_BOX(hbox), page->ident[i].glob, FALSE, FALSE, 0);
 
 			page->ident[i].nocase = gtk_check_button_new_with_label(_("Ignore Case?"));
 			gtk_object_set_user_data(GTK_OBJECT(page->ident[i].nocase), GINT_TO_POINTER(i));
-			gtk_signal_connect(GTK_OBJECT(page->ident[i].nocase), "clicked", GTK_SIGNAL_FUNC(evt_id_nocase_changed), page);
+			g_signal_connect(G_OBJECT(page->ident[i].nocase), "clicked", G_CALLBACK(evt_id_nocase_changed), page);
 			gtk_box_pack_start(GTK_BOX(hbox), page->ident[i].nocase, FALSE, FALSE, 0);
 
 			gtk_table_attach(GTK_TABLE(page->itable), hbox, 2, 3, y, y + 1, GTK_SHRINK,0,0,0);
@@ -656,7 +751,7 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 	label = gtk_label_new(_("Style"));
 	gtk_box_pack_start(GTK_BOX(page->shbox), label, FALSE, FALSE, 5);
 	page->sbtn = gtk_button_new();
-	gtk_signal_connect(GTK_OBJECT(page->sbtn), "clicked", GTK_SIGNAL_FUNC(evt_style_clicked), page);
+	g_signal_connect(G_OBJECT(page->sbtn), "clicked", G_CALLBACK(evt_style_clicked), page);
 
 	page->sbhbox = gtk_hbox_new(FALSE, 0);
 	page->sicon  = NULL;
@@ -674,24 +769,24 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 	page->bhbox = gtk_hbox_new(FALSE, 0);
 	page->add = gtk_button_new_with_label(_("Add"));
 	gtk_object_set_user_data(GTK_OBJECT(page->add), &min->cfg);
-	gtk_signal_connect(GTK_OBJECT(page->add), "clicked", GTK_SIGNAL_FUNC(evt_add_clicked), page);
+	g_signal_connect(G_OBJECT(page->add), "clicked", G_CALLBACK(evt_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->add, TRUE, TRUE, 5);
 
 	page->up = gui_arrow_button_new(GTK_ARROW_UP);
-	gtk_signal_connect(GTK_OBJECT(page->up), "clicked", GTK_SIGNAL_FUNC(evt_up_clicked), page);
+	g_signal_connect(G_OBJECT(page->up), "clicked", G_CALLBACK(evt_up_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->up, FALSE, FALSE, 0);
 
 	page->down = gui_arrow_button_new(GTK_ARROW_DOWN);
-	gtk_signal_connect(GTK_OBJECT(page->down), "clicked", GTK_SIGNAL_FUNC(evt_down_clicked), page);
+	g_signal_connect(G_OBJECT(page->down), "clicked", G_CALLBACK(evt_down_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->down, FALSE, FALSE, 0);
 
 	page->del = gtk_button_new_with_label(_("Delete"));
-	gtk_signal_connect(GTK_OBJECT(page->del), "clicked", GTK_SIGNAL_FUNC(evt_del_clicked), page);
+	g_signal_connect(G_OBJECT(page->del), "clicked", G_CALLBACK(evt_del_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->del, TRUE, TRUE, 5);
 
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->bhbox, FALSE, FALSE, 5);
 
-	gtk_widget_show_all(page->vbox);		
+	gtk_widget_show_all(page->vbox);
 
 	cfg_tree_level_append(_("Types"), page->vbox);
 	cfg_tree_level_end();
@@ -704,7 +799,7 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 /* 1998-08-14 -	Redisplay imminent, update looks. Simplistic. */
 static void ctp_update(MainInfo *min)
 {
-	populate_clist(min, &the_page);
+	populate_list(min, &the_page);
 }
 
 /* ----------------------------------------------------------------------------------------- */
