@@ -12,6 +12,10 @@
 
 #include "gentoo.h"
 
+#if !defined _POSIX_SOURCE
+ #define _POSIX_SOURCE
+#endif
+
 #include <stdlib.h>
 #include <signal.h>
 #include <sys/types.h>
