@@ -44,6 +44,33 @@ static void timeout_remove(gint button, gpointer user)
 	g_free(body);
 }
 
+/* 2008-03-16 -	Returns a label describing the current CHARSET value (I think). */
+static GtkWidget * get_charset_label(void)
+{
+	const gchar	*charset = NULL;
+	gchar		buf[1024];
+
+	g_get_charset(&charset);
+	g_snprintf(buf, sizeof buf, _("Native CHARSET: \"%s\"."), charset);
+	return gtk_label_new(buf);
+}
+
+/* 2008-08-09 -	Returns a label describing the current filename encoding. */
+static GtkWidget * get_filename_label(void)
+{
+	const gchar	**enc = NULL;
+
+	g_get_filename_charsets(&enc);
+	if(enc != NULL && enc[0] != NULL)
+	{
+		gchar	buf[1024];
+
+		g_snprintf(buf, sizeof buf, _("Filename encoding: \"%s\"."), enc[0]);
+		return gtk_label_new(buf);
+	}
+	return NULL;
+}
+
 gint cmd_about(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gchar		buf[64];
@@ -67,7 +94,7 @@ gint cmd_about(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	}
 	w = gtk_label_new(buf);
 	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
-	w = gtk_label_new(_("(c) 1998-2008 by Emil Brink, Obsession Development.\n"));
+	w = gtk_label_new(_("(c) 1998-2009 by Emil Brink, Obsession Development.\n"));
 	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
 	w = gtk_label_new(_("This is free software, and there is ABSOLUTELY NO\n"
 			"WARRANTY. Read the file COPYING for more details.\n"));
@@ -92,14 +119,18 @@ gint cmd_about(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 #endif
 	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
 
+	w = get_charset_label();
+	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
+	w = get_filename_label();
+	if(w != NULL)
+		gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
+
 	gtk_notebook_append_page(GTK_NOTEBOOK(body->notebook), body->page[0], NULL);
 	body->page[1] = gtk_label_new(	_("The author of gentoo can be reached via Internet\n"
 					"e-mail at <emil@obsession.se>; feel free to let\n"
 					"me know what you think of this software, give\n"
 					"suggestions/bug reports, and so on.\n\n"
 
-					"Custom widgets by J. Hanson <johan@tiq.com>.\n\n"
-
 					"The latest release of gentoo can always be down-\n"
 					"loaded from the official gentoo project homepage at\n"
 					"<http://www.obsession.se/gentoo/>. New releases are\n"
