@@ -50,10 +50,10 @@ typedef struct {
 
 static void cho_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
-	gchar	label[MAXNAMLEN + 64];
+	gchar	label[2 * MAXNAMLEN + 64];
 	ChoInfo	*cho = user;
 
-	g_snprintf(label, sizeof label, _("Set Ownership for '%s':"), DP_ROW_NAME(row));
+	g_snprintf(label, sizeof label, _("Set Ownership for '%s':"), DP_ROW_NAMED(row));
 	gtk_label_set_text(GTK_LABEL(cho->label), label);
 	gtk_widget_grab_focus(GTK_COMBO(cho->row[0].combo)->entry);
 }
@@ -79,8 +79,8 @@ static gboolean parse_or_lookup(const gchar *str, long lookup(const gchar *name)
 
 static gboolean get_own(ChoInfo *cho, uid_t *owner, gid_t *group)
 {
-	gchar	*text;
-	long	tmp;
+	const gchar	*text;
+	long		tmp;
 
 	if((text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cho->row[0].combo)->entry))) == NULL)
 		return FALSE;
