@@ -122,9 +122,9 @@ static gboolean copy_attributes(const char *from, const struct stat *from_stat,
 	copy_dates(from, from_stat, to);
 	copy_mode(from,  from_stat, to);
 	copy_owner(from, from_stat, to);
-	ret = errno == 0;
 	if(copy_options.ignore_attrib_err)
 		errno = old_errno;
+	ret = errno == 0;
 	return ret;
 }
 
@@ -332,7 +332,7 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 	pgs_progress_item_begin(min, dest_name, sstat.st_size);
 	if(fut_cd(from, old_dir, sizeof old_dir))
 	{
-		if(owres == OVW_PROCEED_DIR || mkdir(dest_name, sstat.st_mode | S_IWUSR) == 0)
+		if(owres == OVW_PROCEED_DIR || (mkdir(dest_name, sstat.st_mode | S_IWUSR) == 0))
 		{
 			if((dir = opendir(".")) != NULL)
 			{
