@@ -168,10 +168,10 @@ gint cmd_dpfocusisrch(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *c
 	entry = the_isearch_info.wid->entry;
 	gtk_widget_grab_focus(entry);
 
-	the_isearch_info.sig_activate  = gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(evt_isearch_activate), NULL);
-	the_isearch_info.sig_changed   = gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(evt_isearch_changed), NULL);
-	the_isearch_info.sig_key_press = gtk_signal_connect(GTK_OBJECT(entry), "key_press_event", GTK_SIGNAL_FUNC(evt_isearch_keypress), NULL);
-	the_isearch_info.sig_focus_out = gtk_signal_connect(GTK_OBJECT(entry), "focus_out_event", GTK_SIGNAL_FUNC(evt_isearch_focus_out), NULL);
+	the_isearch_info.sig_activate  = g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(evt_isearch_activate), NULL);
+	the_isearch_info.sig_changed   = g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(evt_isearch_changed), NULL);
+	the_isearch_info.sig_key_press = g_signal_connect(G_OBJECT(entry), "key_press_event", G_CALLBACK(evt_isearch_keypress), NULL);
+	the_isearch_info.sig_focus_out = g_signal_connect(G_OBJECT(entry), "focus_out_event", G_CALLBACK(evt_isearch_focus_out), NULL);
 
 	the_isearch_info.old_focus = src->focus_row;
 
