@@ -40,9 +40,6 @@ gint cmd_dprecenter(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	else if(!win_window_size_get(min->cfg.wininfo, WIN_MAIN, &ww, NULL))
 		return 1;
 
-	/* Make sure tracking, if enabled, gets a chance to track centering. */
-	if(min->gui->sig_left_sizealloc != 0)
-		gtk_signal_handler_unblock(GTK_OBJECT(min->gui->pane[0].vbox), min->gui->sig_left_sizealloc);
 	gtk_paned_set_position(GTK_PANED(min->gui->panes), ww / 2);
 
 	return 1;
@@ -125,17 +122,16 @@ gint cmd_dpfocuspath(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca
 {
 	guint	select, clear;
 
-	gtk_widget_grab_focus(GTK_COMBO(src->path)->entry);
-
 	select = car_keyword_get_boolean(ca, "select", FALSE);
 	clear  = car_keyword_get_boolean(ca, "clear", FALSE);
 
-	if(select)
-		gtk_entry_select_region(GTK_ENTRY(GTK_COMBO(src->path)->entry), 0, -1);
-	else if(clear)
-		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(src->path)->entry), "");
+	gtk_editable_set_position(GTK_EDITABLE(DP_ENTRY(src)), -1);
 
-	gtk_entry_set_position(GTK_ENTRY(GTK_COMBO(src->path)->entry), -1);
+	if(select)
+		gtk_editable_select_region(GTK_EDITABLE(DP_ENTRY(src)), 0, 3);
+	else if(clear)
+		dp_path_clear(src);
+	dp_path_focus(src);
 
 	return 1;
 }
