@@ -126,15 +126,15 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow *ro
 			gtk_table_attach(GTK_TABLE(tab), ico, 0, 1, y, y + 1, 0,0,5,5);
 		}
 	}
-	label_pack(tab, DP_ROW_NAME(row), 1, y++, TRUE);
+	label_pack(tab, DP_ROW_NAMED(row), 1, y++, TRUE);
 
 	/* For symbolic links, display name of the target, at least. */
 	if(S_ISLNK(DP_ROW_LSTAT(row).st_mode))
 	{
 		label_pack(tab, _("Link To"), 0, y, FALSE);
 		entry = gtk_entry_new();
-		gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
-		gtk_entry_set_text(GTK_ENTRY(entry), DP_ROW_LINKNAME(row));
+		gtk_widget_set_sensitive(entry, FALSE);	/* Used to be just non-editable, but ... this looks better. */
+		gtk_entry_set_text(GTK_ENTRY(entry), DP_ROW_LINKNAMED(row));
 		gtk_table_attach(GTK_TABLE(tab), entry, 1, 2, y, y + 1,  GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL,5,0);
 		y++;
 	}
@@ -243,7 +243,7 @@ static gint information(MainInfo *min, const DirPane *dp, const DirRow *row)
 	if((fr = build_info(min, dp, row, &dsi)) != NULL)
 	{
 		gtk_widget_show_all(fr);
-		dlg_dialog_async_new(fr, DP_ROW_NAME(row), _("_OK"), info_close, dsi);
+		dlg_dialog_async_new(fr, DP_ROW_NAMED(row), _("_OK"), info_close, dsi);
 	}
 	return fr != NULL;
 }
@@ -270,7 +270,7 @@ gint cmd_information(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca
 	pgs_progress_end(min);
 
 	if(errno)
-		err_set(min, errno, CMD_ID, iter ? DP_SEL_NAME(iter) : NULL);
+		err_set(min, errno, CMD_ID, iter ? DP_SEL_NAMED(iter) : NULL);
 	else
 		err_clear(min);
 	err_show(min);
