@@ -10,6 +10,7 @@
 #include <fcntl.h>
 
 #include "cmd_delete.h"
+#include "convutil.h"
 #include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
@@ -25,12 +26,14 @@
 /* ----------------------------------------------------------------------------------------- */
 
 /* 2000-02-12 -	Do the actual joining of the selected files. Returns TRUE on success. */
-static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *clist, GtkEntry *entry, gsize total_size)
+static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkListStore *store, GtkEntry *entry, gsize total_size)
 {
-	gchar		buf[PATH_MAX], *str;
+	gchar		buf[PATH_MAX], *file;
+	const gchar	*str;
 	gsize		done_size = 0U;
 	OvwRes		owres;
 	gboolean	doit = TRUE;
+	Conv		conv;
 
 	/* Protect against empty destination name. */
 	if((str = gtk_entry_get_text(entry)) == NULL || *str == '\0')
@@ -39,8 +42,10 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *cli
 		return FALSE;
 	}
 
+	file = conv_begin(&conv, str);
+
 	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Continue With Join?"), 0U);
-	g_snprintf(buf, sizeof buf, "%s/%s", dst->dir.path, str);
+	g_snprintf(buf, sizeof buf, "%s/%s", dst->dir.path, file);
 	owres = ovw_overwrite_file(min, buf, NULL);
 	switch(owres)
 	{
@@ -63,13 +68,16 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *cli
 
 		if((fout = open(buf, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) >= 0)
 		{
-			gint	fin, i;
-			DirRow	*dr;
-			gsize	put;
+			GtkTreeIter	iter;
+			gint		fin;
+			DirRow		*dr;
+			gsize		put;
 
 			pgs_progress_begin(min, _("Joining..."), PFLG_BYTE_VISIBLE);
-			for(i = 0; (dr = gtk_clist_get_row_data(clist, i)) != NULL; i++)
+			gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+			while(gtk_list_store_iter_is_valid(store, &iter))
 			{
+				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 2, &dr, -1);
 				if((fin = open(dp_full_name(src, DP_ROW_INDEX(src, dr)), O_RDONLY)) >= 0)
 				{
 					pgs_progress_item_begin(min, DP_ROW_NAME(dr), DP_ROW_STAT(dr).st_size);
@@ -93,6 +101,8 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *cli
 					err_set(min, errno, CMD_ID, DP_ROW_NAME(dr));
 					break;
 				}
+				if(!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+					break;
 			}
 			pgs_progress_end(min);
 			close(fout);
@@ -105,6 +115,7 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *cli
 			err_set(min, errno, CMD_ID, buf);
 	}
 	ovw_overwrite_end(min);
+	conv_end(&conv);
 
 	return doit ? done_size == total_size : TRUE;
 }
@@ -136,17 +147,22 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 		*/
 		if(num > 1)
 		{
-			Dialog		*dlg;
-			GtkWidget	*scwin, *vbox, *label, *clist, *entry;
-			gchar		buf[FILENAME_MAX + 64];
-			const gchar	*text[1], *first = NULL;
-			gint		row, len;
+			Dialog			*dlg;
+			GtkWidget		*scwin, *vbox, *label, *entry;
+			GtkListStore		*store;
+			GtkWidget		*view;
+			GtkCellRenderer		*cr, *sr;
+			GtkTreeViewColumn	*vc;
+			GtkTreeSelection	*treesel;
+			gchar			buf[FILENAME_MAX + 64];
+			const gchar		*first = NULL;
+			gint			len;
 
 			vbox  = gtk_vbox_new(FALSE, 0);
 			label = gtk_label_new(_("Click and Drag Files to Reorder, Then Click Join."));
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
-			if(tot > 1 << 10)
+			if(tot > (1 << 10))
 			{
 				gchar	sbuf[16];
 
@@ -160,22 +176,40 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 			scwin = gtk_scrolled_window_new(NULL, NULL);
 			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-			clist = gtk_clist_new(1);
-			gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
-			gtk_clist_set_reorderable(GTK_CLIST(clist), TRUE);
-			gtk_widget_set_usize(clist, -1, 256);		/* Get a sensible minmum height. */
+
+			/* Create and initialize list store with selected files and their sizes. */
+			store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_POINTER);
 			for(iter = sel; iter != NULL; iter = g_slist_next(iter))
 			{
 				if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
 				{
-					text[0] = DP_ROW_NAME((DirRow *) iter->data);
-					row = gtk_clist_append(GTK_CLIST(clist), (gchar **) text);
-					gtk_clist_set_row_data(GTK_CLIST(clist), row, iter->data);
-					if(!first)			/* Remember first name, for entry. */
-						first = text[0];
+					GtkTreeIter	ti;
+
+					gtk_list_store_append(store, &ti);
+					gtk_list_store_set(store, &ti, 0, DP_ROW_NAME((DirRow *) iter->data),
+								       1, DP_ROW_STAT((DirRow *) iter->data).st_size,
+								       2, iter->data,
+								       -1);
+					if(first == NULL)
+						first = DP_ROW_NAME((DirRow *) iter->data);
 				}
 			}
-			gtk_container_add(GTK_CONTAINER(scwin), clist);
+
+			view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+			cr = gtk_cell_renderer_text_new();
+			vc = gtk_tree_view_column_new_with_attributes("(title)", cr, "text", 0, NULL);
+			gtk_tree_view_append_column(GTK_TREE_VIEW(view), vc);
+			sr = gtk_cell_renderer_text_new();
+			g_object_set(G_OBJECT(sr), "xalign", 1.0f, NULL);
+			vc = gtk_tree_view_column_new_with_attributes("(title)", sr, "text", 1, NULL);
+			gtk_tree_view_append_column(GTK_TREE_VIEW(view), vc);
+			gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
+
+			treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+			gtk_tree_selection_set_mode(treesel, GTK_SELECTION_NONE);
+			gtk_tree_view_set_reorderable(GTK_TREE_VIEW(view), TRUE);
+			gtk_widget_set_size_request(view, 0, 256);	/* Get a sensible minmum height. */
+			gtk_container_add(GTK_CONTAINER(scwin), view);
 			gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
 
 			label = gtk_label_new(_("Enter Destination File Name"));
@@ -199,7 +233,7 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 			gtk_widget_show_all(vbox);
 			gtk_widget_grab_focus(entry);
 			if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
-				ret = do_join(min, src, dst, GTK_CLIST(clist), GTK_ENTRY(entry), tot);
+				ret = do_join(min, src, dst, store, GTK_ENTRY(entry), tot);
 			dlg_dialog_sync_destroy(dlg);
 		}
 		dp_free_selection(sel);
