@@ -19,6 +19,7 @@
 #include <regex.h>
 
 #include "cmd_delete.h"
+#include "convutil.h"
 #include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
@@ -32,6 +33,10 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
+typedef enum {
+	LOWER, UPPER, INITIAL, HEADLINE
+} CaseMode;
+
 typedef struct {
 	Dialog		*dlg;
 
@@ -59,6 +64,8 @@ typedef struct {
 	/* Case-conversion mode. */
 	GtkWidget	*case_vbox;
 	GtkWidget	*case_lower;
+	GtkWidget	*case_upper;
+	GtkWidget	*case_initial;
 
 	MainInfo	*min;
 	DirPane		*src;
@@ -114,22 +121,29 @@ static gint do_rename(MainInfo *min, DirPane *dp, GSList **iter, GString *nn, gi
 
 static gint rename_simple(RenREInfo *ri)
 {
-	const gchar	*from, *to, *ptr, *base;
+	const gchar	*fromd, *tod, *from, *to, *ptr, *base;
 	gint		err = 0;
 	gsize		flen;
 	GString		*nn;
 	GSList		*iter;
 	gboolean	nocase, global;
+	Conv		conv;
 
-	if((from = gtk_entry_get_text(GTK_ENTRY(ri->r_from))) == NULL)
+	if((fromd = gtk_entry_get_text(GTK_ENTRY(ri->r_from))) == NULL)
 		return -1;
-	flen = strlen(from);
 
-	to = gtk_entry_get_text(GTK_ENTRY(ri->r_to));
+	tod = gtk_entry_get_text(GTK_ENTRY(ri->r_to));
 
 	nocase = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->r_cnocase));
 	global = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->r_cglobal));
 
+	from = conv_begin(&conv, fromd);
+	to = conv_add(&conv, tod);
+
+	flen = strlen(from);
+
+	to = gtk_entry_get_text(GTK_ENTRY(ri->r_to));
+
 	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 	nn = g_string_new("");
 	for(iter = ri->selection; !err && iter != NULL; iter = g_slist_next(iter))
@@ -158,6 +172,7 @@ static gint rename_simple(RenREInfo *ri)
 	}
 	g_string_free(nn, TRUE);
 	ovw_overwrite_end(ri->min);
+	conv_end(&conv);
 
 	return err;
 }
@@ -205,15 +220,19 @@ static gboolean interpolate(GString *str, const gchar *def, const gchar *src, co
 
 static gint rename_regexp(RenREInfo *ri)
 {
-	const gchar	*fromdef, *to;
+	const gchar	*fromdefd, *tod, *fromdef, *to;
 	gint		reerr, reflags, err = 0;
 	regex_t		fromre;
+	Conv		conv;
 
-	if((fromdef = gtk_entry_get_text(GTK_ENTRY(ri->re_from))) == NULL)
+	if((fromdefd = gtk_entry_get_text(GTK_ENTRY(ri->re_from))) == NULL)
 		return -1;
-	if((to = gtk_entry_get_text(GTK_ENTRY(ri->re_to))) == NULL)
+	if((tod = gtk_entry_get_text(GTK_ENTRY(ri->re_to))) == NULL)
 		return -1;
 
+	fromdef = conv_begin(&conv, fromdefd);
+	to = conv_add(&conv, tod);
+
 	reflags = REG_EXTENDED | gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->re_cnocase));
 	if((reerr = regcomp(&fromre, fromdef, reflags)) == 0)
 	{
@@ -242,6 +261,8 @@ static gint rename_regexp(RenREInfo *ri)
 		regerror(reerr, &fromre, buf + strlen(buf), sizeof buf - strlen(buf) - 1);
 		dlg_dialog_async_new_error(buf);
 	}
+	conv_end(&conv);
+
 	return err;
 }
 
@@ -249,11 +270,12 @@ static gint rename_regexp(RenREInfo *ri)
 
 static int rename_map(RenREInfo *ri)
 {
-	const char	*fromdef, *todef, *remdef, *p;
+	const char	*fromdef, *todef, *remdef, *map;
 	GSList		*iter;
-	GString		*nn;
+	GString		*nn, *nr;
 	gint		err = 0;
-	gsize		fl, tl, rl, i, j;
+	gsize		fl, tl, rl, i, nl;
+	Conv		conv;
 
 	if((fromdef = gtk_entry_get_text(GTK_ENTRY(ri->map_from))) == NULL)
 		return -1;
@@ -270,30 +292,49 @@ static int rename_map(RenREInfo *ri)
 
 	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 	nn = g_string_new("");
+	nr = g_string_new("");
+	conv_init(&conv);
+	conv_add(&conv, "");	/* Pre-load the Conv, so we can use set() below. */
 	for(iter = ri->selection; iter != NULL; iter = g_slist_next(iter))
 	{
-		/* Step 1: set the dynamic string to the input filename. */
-		g_string_assign(nn, DP_SEL_NAME(iter));
-		/* Step 2: perform character mapping, as needed. */
-		for(i = 0; i < nn->len; i++)
+		const char	*ptr;
+
+		/* Step 1: clear the dynamic string. */
+		g_string_truncate(nn, 0);
+		ptr = DP_SEL_NAMED(iter);
+		nl = g_utf8_strlen(ptr, -1);
+		/* Step 2: go through and do the mapping. */
+		for(i = 0; i < nl; i++, ptr = g_utf8_next_char(ptr))
 		{
-			if((p = strchr(fromdef, nn->str[i])) != NULL)
+			gunichar	ch = g_utf8_get_char(ptr);
+
+			if((map = g_utf8_strchr(fromdef, fl, ch)) != NULL)
 			{
-				j = p - fromdef;
-				if(j < tl)
-					nn->str[i] = todef[j];
+				glong	index = g_utf8_pointer_to_offset(fromdef, map);
+				const gchar	*tptr = g_utf8_offset_to_pointer(todef, index);
+
+				ch = g_utf8_get_char(tptr);	/* Map. */
 			}
+			g_string_append_unichar(nn, ch);
 		}
 		/* Step 3: remove any characters found in the 'remove' string. */
-		for(i = 0; i < nn->len;)
+		ptr = nn->str;
+		nl = g_utf8_strlen(ptr, -1);
+		g_string_truncate(nr, 0);
+		for(i = 0; i < nl; i++, ptr = g_utf8_next_char(ptr))
 		{
-			if(strchr(remdef, nn->str[i]) != NULL)
-				g_string_erase(nn, i, 1);
-			else
-				i++;
+			gunichar	ch = g_utf8_get_char(ptr);
+
+			if(g_utf8_strchr(remdef, rl, ch) == NULL)
+				g_string_append_unichar(nr, ch);
 		}
+		/* Step 4: re-use the 'nn' string with filesystem version of final string. */
+		g_string_truncate(nn, 0);
+		g_string_assign(nn, conv_set(&conv, 0, nr->str));
 		do_rename(ri->min, ri->src, &iter, nn, &err);
 	}
+	conv_end(&conv);
+	g_string_free(nr, TRUE);
 	g_string_free(nn, TRUE);
 	ovw_overwrite_end(ri->min);
 
@@ -302,26 +343,71 @@ static int rename_map(RenREInfo *ri)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2008-07-19 -	Converts the given string to have an upper-case initial character, while the
+ *		the remainder are lower-case. This is a very western idea of a useful case
+ *		conversion, but ... I'll include it anyway.
+*/
+static gchar * utf8_str_initial(const gchar *str, gssize len)
+{
+	GString		*tmp;
+	gchar		*buf;
+	gunichar	here;
+
+	tmp = g_string_sized_new(len);
+
+	here = g_utf8_get_char(str);
+	g_string_append_unichar(tmp, g_unichar_toupper(here));
+
+	while(--len)
+	{
+		str = g_utf8_next_char(str);
+		here = g_utf8_get_char(str);
+		g_string_append_unichar(tmp, g_unichar_tolower(here));
+	}
+	buf = tmp->str;
+	g_string_free(tmp, FALSE);
+
+	return buf;
+}
+
+/* 2008-04-20 -	Minor touch-ups due to GTK+ 2.0 and UTF-8 in strings. We need to make a decision:
+ *		Is the map operation happening in file-system or display space? The obvious
+ *		choice seems to be file-system, which is after all what the filenames are for.
+ *		However, we don't have a general way of downcasing if the local file system is
+ *		using UTF-8 ... So we use the display versions of the names, downcase in UTF-8,
+ *		and then convert back.
+*/
 static int rename_case(RenREInfo *ri)
 {
-	gboolean	lower = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->case_lower));
 	GString		*nn;
 	GSList		*iter;
 	gint		err = 0;
+	Conv		conv;
+	gchar *		(*func)(const gchar *, gssize), *cc;
 
 	ovw_overwrite_begin(ri->min, _("\"%s\" Already Exists - Proceed With Rename?"), 0U);
 	nn = g_string_new("");
+	/* Select the case-conversion function, once. */
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->case_lower)))
+		func = g_utf8_strdown;
+	else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->case_upper)))
+		func = g_utf8_strup;
+	else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ri->case_initial)))
+		func = utf8_str_initial;
+	else
+		func = NULL;
 	for(iter = ri->selection; iter != NULL; iter = g_slist_next(iter))
 	{
 		/* Step 1: set the dynamic string to the input filename. */
-		g_string_assign(nn, DP_SEL_NAME(iter));
+		g_string_assign(nn, DP_SEL_NAMED(iter));
 		/* Step 2: perform case conversion. */
-		if(lower)
-			g_string_down(nn);
-		else
-			g_string_up(nn);
+		cc = func(nn->str, nn->len);
+		g_string_assign(nn, conv_begin(&conv, cc));
+		g_free(cc);
 		/* Step 3: if resulting name is different, rename on disk. */
-		do_rename(ri->min, ri->src, &iter, nn, &err);
+		if(strcmp(DP_SEL_NAME(iter), nn->str) != 0)
+			do_rename(ri->min, ri->src, &iter, nn, &err);
+		conv_end(&conv);
 	}
 	g_string_free(nn, TRUE);
 	ovw_overwrite_end(ri->min);
@@ -341,7 +427,7 @@ static void evt_nbook_switchpage(GtkWidget *wid, GtkNotebookPage *page, gint pag
 
 gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	GtkWidget		*label, *table, *wid;
+	GtkWidget		*label, *table;
 	static RenREInfo	ri = { NULL };
 
 	ri.min	= min;
@@ -353,7 +439,7 @@ gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	if(ri.dlg == NULL)
 	{
 		ri.nbook = gtk_notebook_new();
-		gtk_signal_connect(GTK_OBJECT(ri.nbook), "switch-page", GTK_SIGNAL_FUNC(evt_nbook_switchpage), &ri);
+		g_signal_connect(G_OBJECT(ri.nbook), "switch-page", G_CALLBACK(evt_nbook_switchpage), &ri);
 
 		/* Build the 'Simple' mode's GUI. */
 		ri.r_vbox = gtk_vbox_new(FALSE, 0);
@@ -428,12 +514,14 @@ gint cmd_renamere(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 		/* Build the 'Case' mode's GUI. */
 		ri.case_vbox = gtk_vbox_new(FALSE, 0);
-		label = gtk_label_new(_("Convert all characters in filename to either\nall-lower or all-upper case."));
+		label = gtk_label_new(_("Changes the case (upper/lower) of the characters\nin the selected filename(s)."));
 		gtk_box_pack_start(GTK_BOX(ri.case_vbox), label, FALSE, FALSE, 0);
 		ri.case_lower = gtk_radio_button_new_with_label(NULL, _("Lower Case?"));
 		gtk_box_pack_start(GTK_BOX(ri.case_vbox), ri.case_lower, FALSE, FALSE, 0);
-		wid = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case?"));
-		gtk_box_pack_start(GTK_BOX(ri.case_vbox), wid, FALSE, FALSE, 0);
+		ri.case_upper = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case?"));
+		gtk_box_pack_start(GTK_BOX(ri.case_vbox), ri.case_upper, FALSE, FALSE, 0);
+		ri.case_initial = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(ri.case_lower)), _("Upper Case Initial?"));
+		gtk_box_pack_start(GTK_BOX(ri.case_vbox), ri.case_initial, FALSE, FALSE, 0);
 		gtk_widget_show_all(ri.map_vbox);
 		gtk_notebook_append_page(GTK_NOTEBOOK(ri.nbook), ri.case_vbox, gtk_label_new(_("Case")));
 
