@@ -28,7 +28,6 @@
 typedef struct {			/* Options for the "ViewText" command. */
 	gboolean	modified;
 	gboolean	use_mmap;			/* Use mmap() for files of non-zero length? */
-	gsize		mmap_buf_size;			/* Max # of bytes to insert from mmap(). */
 	gsize		buf_size;			/* Buffer size, when not using mmap(). */
 	gsize		check_size;			/* The ViewTextOrHex command inspects this many bytes. */
 } OptViewText;
@@ -69,7 +68,7 @@ static gint view_file_text(MainInfo *min, const gchar *full_name)
 	if((txv = txv_open(min, full_name)) != NULL)
 	{
 		txv_show(txv);
-		txv_load_text(txv, full_name, viewtext_options.use_mmap, viewtext_options.use_mmap ? viewtext_options.mmap_buf_size : viewtext_options.buf_size);
+		txv_text_load(txv, full_name, viewtext_options.use_mmap, viewtext_options.buf_size);
 		txv_enable(txv);
 	}
 	return txv != NULL;
@@ -85,7 +84,7 @@ static gint view_file_hex(MainInfo *min, const gchar *full_name)
 	if((txv = txv_open(min, full_name)) != NULL)
 	{
 		txv_show(txv);
-		txv_load_text_hex(txv, full_name);
+		txv_text_load_hex(txv, full_name);
 		txv_enable(txv);
 	}
 	return txv != NULL;
@@ -162,8 +161,17 @@ static gint view_file_text_or_hex(MainInfo *min, const gchar *full_name)
 */
 gint cmd_viewtext(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	guint	mode;
+	const gchar	*sk;
+	guint		mode, skip = 0;
 
+	if((sk = car_keyword_get_value(ca, "skip", NULL)) != NULL)
+	{
+		gchar	*endptr;
+
+		skip = strtol(sk, &endptr, 0);
+		if(!*sk || *endptr)
+			skip = 0;
+	}
 	mode = car_keyword_get_enum(ca, "mode", 1, "auto", "text", "hex");
 	switch(mode)
 	{
@@ -186,16 +194,14 @@ void cfg_viewtext(MainInfo *min)
 	if(viewtext_cmc == NULL)
 	{
 		/* Initialize options to legacy default values. */
-		viewtext_options.modified	= FALSE;
-		viewtext_options.use_mmap	= TRUE;
-		viewtext_options.mmap_buf_size	= 1 << 20;
-		viewtext_options.buf_size	= 1 << 16;
-		viewtext_options.check_size	= 1 << 9;
+		viewtext_options.modified    = FALSE;
+		viewtext_options.use_mmap    = TRUE;
+		viewtext_options.buf_size    = (1 << 18);
+		viewtext_options.check_size  = (1 << 9);
 
 		viewtext_cmc = cmc_config_new("ViewText", &viewtext_options);
 		cmc_field_add_boolean(viewtext_cmc, "modified", NULL, offsetof(OptViewText, modified));
 		cmc_field_add_boolean(viewtext_cmc, "use_mmap", _("Use mmap() to Speed Loading?"), offsetof(OptViewText, use_mmap));
-		cmc_field_add_size(viewtext_cmc, "mmap_chunk", _("Buffer Size for mmap()"), offsetof(OptViewText, mmap_buf_size), 1 << 16, 1 << 23, 1 << 12);
 		cmc_field_add_size(viewtext_cmc, "buf_size", _("Buffer Size"), offsetof(OptViewText, buf_size), 1024, (1 << 20), 1024);
 		cmc_field_add_size(viewtext_cmc, "check_size", _("Hex-Check First"), offsetof(OptViewText, check_size), 32, (1 << 16), 32);
 		cmc_config_register(viewtext_cmc);
