@@ -275,30 +275,17 @@ gint csq_cmdseq_row_append(CmdSeq *cs, CmdRow *row)
 	return -1;
 }
 
-/* 1998-09-27 -	Move the <row> (which must already be part of <cs>) either one row up
-**		(<delta> == -1) or down (1). The row will wrap around at the top and
-**		bottom positions, since I think that's fun. :) Returns new index of
-**		the row.
+/* 2009-03-20 -	Reorder the rows of the given sequence, to match the given new list.
+**		The new list is assumed to re-use the actual CmdRow pointers that are
+**		in our current list. All we need to do is free the GList part of our
+**		old list, and then replace the list with the new, taking ownership.
 */
-gint csq_cmdseq_row_move(CmdSeq *cs, CmdRow *row, gint delta)
+void csq_cmdseq_rows_relink(CmdSeq *cs, GList *rows)
 {
-	GList	*link;
-	gint	pos, np;
-
-	if(cs != NULL && row != NULL && (delta == -1 || delta == 1))
-	{
-		link = g_list_find(cs->rows, row);
-		pos  = g_list_position(cs->rows, link);
-		cs->rows = g_list_remove(cs->rows, row);
-		np = pos + delta;
-		if(np < 0)
-			np = g_list_length(cs->rows);
-		else if(np > (gint) g_list_length(cs->rows))
-			np = 0;
-		cs->rows = g_list_insert(cs->rows, row, np);
-		return np;
-	}
-	return -1;
+	if(cs == NULL || rows == NULL)
+		return;
+	g_list_free(cs->rows);	/* This doesn't touch the data, glib doesn't know how. */
+	cs->rows = rows;
 }
 
 /* 1998-09-25 -	Remove given row from command sequence. Returns the index of the
@@ -320,6 +307,23 @@ gint csq_cmdseq_row_delete(CmdSeq *cs, CmdRow *row)
 	return -1;
 }
 
+/* 2009-03-18 -	Clears the command sequence of all rows, freeing them while doing so. */
+void csq_cmdseq_row_delete_all(CmdSeq *cs)
+{
+	GList	*iter;
+
+	if(cs == NULL)
+		return;
+
+	/* Don't disassemble the list link by link. Walk through and destroy the rows,
+	** then free the entire list in one ... you-know-what. With a swoop.
+	*/
+	for(iter = cs->rows; iter != NULL; iter = g_list_next(iter))
+		csq_cmdrow_destroy(iter->data);
+	g_list_free(cs->rows);
+	cs->rows = NULL;
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-25 -	Execute a built-in command. Currently very simple.
@@ -344,10 +348,12 @@ static gint execute_builtin(MainInfo *min, const gchar *cmd)
 				ca  = car_create(argv);
 				ret = cdesc->cmd(min, src, dp_mirror(min, src), ca);
 				car_destroy(ca);
+				g_free(argv);	/* Don't forget to free the parsed arguments. */
 				return ret;
 			}
 			g_snprintf(msg, sizeof msg, _("Unable to execute unknown\ncommand \"%s\"."), cmd);
 			dlg_dialog_async_new_error(msg);
+			g_free(argv);
 		}
 		return 1;
 	}
