@@ -10,11 +10,10 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "odemilbutton.h"
-
 #include "color_dialog.h"
 #include "dialog.h"
 #include "guiutil.h"
+#include "odmultibutton.h"
 #include "sizeutil.h"
 #include "strutil.h"
 #include "xmlutil.h"
@@ -173,7 +172,12 @@ void cmc_config_base_load(CmdCfg *cmc, const XmlNode *node)
 					xml_get_uinteger(node, fld->name, (guint *) ((gchar *) cmc->base + fld->offset));
 					break;
 				case CFT_SIZE:
-					xml_get_uinteger(node, fld->name, (gsize *) ((gchar *) cmc->base + fld->offset));
+					{
+						guint	tmp;
+
+						xml_get_uinteger(node, fld->name, &tmp);
+						*(gsize *) ((gchar *) cmc->base + fld->offset) = tmp;
+					}
 					break;
 				case CFT_STRING:
 					xml_get_text_copy(node, fld->name, (gchar *) cmc->base + fld->offset, fld->field.string.size);
@@ -484,7 +488,7 @@ static gint evt_string_changed(GtkWidget *wid, gpointer user)
 
 	if((cmc != NULL) && (fld != NULL) && (instance != NULL))
 	{
-		gchar	*text;
+		const gchar	*text;
 
 		if((text = gtk_entry_get_text(GTK_ENTRY(wid))) != NULL)
 			stu_strncpy((gchar *) instance + fld->offset, text, fld->field.string.size);
@@ -521,35 +525,14 @@ static gint evt_size_changed(GtkObject *obj, gpointer user)
 	return TRUE;
 }
 
-/* 1999-05-13 -	User is editing a color field through the color dialog. Update. */
-static void color_changed(GdkColor *color, gpointer user)
+/* 2008-11-28 -	User changed the color. Write new back from widget. */
+static void evt_color_set(GtkWidget *wid, gpointer wuser, guint index, gpointer suser)
 {
 	CField		*fld;
 	gpointer	instance;
 
-	get_object_data(GTK_OBJECT(user), &fld, &instance);
-	od_emil_button_set_colors(OD_EMIL_BUTTON(user), OD_EMIL_BUTTON_PRIMARY, NULL, color);
-	*(GdkColor *) ((gchar *) instance + fld->offset) = *color;
-}
-
-/* 1999-05-13 -	User clicked the "Edit" button for a color field. Pop up the color edit
-**		dialog, and update the field.
-*/
-static void evt_color_clicked(GtkWidget *wid, gpointer wuser, guint index, gpointer suser)
-{
-	CmdCfg		*cmc = suser;
-	CField		*fld;
-	gpointer	instance;
-
 	get_object_data(GTK_OBJECT(wid), &fld, &instance);
-
-	if((cmc != NULL) && (fld != NULL) && (instance != NULL))
-	{
-		GdkColor	initial = *(GdkColor *) ((gchar *) instance + fld->offset);
-
-		if(cdl_dialog_sync_new_wait(NULL, color_changed, &initial, wid) != DLG_POSITIVE)	/* Cancel? */
-			od_emil_button_set_colors(OD_EMIL_BUTTON(wid), OD_EMIL_BUTTON_PRIMARY, NULL, &initial);
-	}
+	gtk_color_button_get_color(GTK_COLOR_BUTTON(wid), (GdkColor *) ((gchar *) instance + fld->offset));
 }
 
 /* 2002-08-01 -	Build a neatly right-aligned label, and stuff it into <table>. */
@@ -595,15 +578,15 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				wid = gtk_check_button_new_with_label(fld->desc);
 				set_object_data(GTK_OBJECT(wid), fld, instance);
 				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), *(gboolean *) ((gchar *) instance + fld->offset));
-				gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_boolean_clicked), cmc);
+				g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_boolean_clicked), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 0, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
 				break;
 			case CFT_ENUM:
 				wid = gtk_frame_new(fld->desc);
 				{
 					GtkWidget	*vbox, *radio[sizeof fld->field.fenum.label / sizeof *fld->field.fenum.label];
-					GSList	*group;
-					gint	i, current = *(gint *) ((gchar *) instance + fld->offset);
+					GSList		*group;
+					gint		i, current = *(gint *) ((gchar *) instance + fld->offset);
 
 					vbox = gtk_vbox_new(FALSE, 0);
 					group = gui_radio_group_new(fld->field.fenum.num, (const gchar **) fld->field.fenum.label, radio);
@@ -611,7 +594,7 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 					{
 						set_object_data(GTK_OBJECT(radio[i]), fld, instance);
 						gtk_object_set_data(GTK_OBJECT(radio[i]), "value", GINT_TO_POINTER(i));
-						gtk_signal_connect(GTK_OBJECT(radio[i]), "toggled", GTK_SIGNAL_FUNC(evt_enum_changed), cmc);
+						g_signal_connect(G_OBJECT(radio[i]), "toggled", G_CALLBACK(evt_enum_changed), cmc);
 						gtk_box_pack_start(GTK_BOX(vbox), radio[i], FALSE, FALSE, 0);
 						if(i == current)
 							gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio[i]), TRUE);
@@ -626,7 +609,7 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				obj = gtk_adjustment_new((gfloat) *(gint *) ((gchar *) instance + fld->offset) / fld->field.size.step,
 							 fld->field.size.min / fld->field.size.step,
 							 fld->field.size.max / fld->field.size.step,
-							 1, 1, 1);
+							 1, 1, 0);
 
 				set_object_data(obj, fld, instance);
 				wid = gtk_hscale_new(GTK_ADJUSTMENT(obj));
@@ -636,7 +619,7 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				label = gtk_label_new("");
 				gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 				gtk_object_set_data(obj, "label", label);
-				gtk_signal_connect(obj, "value_changed", GTK_SIGNAL_FUNC(evt_size_changed), cmc);
+				g_signal_connect(G_OBJECT(obj), "value_changed", G_CALLBACK(evt_size_changed), cmc);
 				gtk_adjustment_value_changed(GTK_ADJUSTMENT(obj));
 				gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
 				break;
@@ -645,15 +628,14 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				wid = gtk_entry_new_with_max_length(fld->field.string.size - 1);
 				set_object_data(GTK_OBJECT(wid), fld, instance);
 				gtk_entry_set_text(GTK_ENTRY(wid), (gchar *) instance + fld->offset);
-				gtk_signal_connect(GTK_OBJECT(wid), "changed", GTK_SIGNAL_FUNC(evt_string_changed), cmc);
+				g_signal_connect(G_OBJECT(wid), "changed", G_CALLBACK(evt_string_changed), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
 				break;
 			case CFT_COLOR:
 				label_build(fld->desc, table, 0, row);
-				wid = od_emil_button_new();
+				wid = gtk_color_button_new_with_color((const GdkColor *) ((gchar *) instance + fld->offset));
 				set_object_data(GTK_OBJECT(wid), fld, instance);
-				od_emil_button_set_colors(OD_EMIL_BUTTON(wid), OD_EMIL_BUTTON_PRIMARY, NULL, (GdkColor *) ((gchar *) instance + fld->offset));
-				gtk_signal_connect(GTK_OBJECT(wid), "activate", GTK_SIGNAL_FUNC(evt_color_clicked), cmc);
+				g_signal_connect(G_OBJECT(wid), "color_set", G_CALLBACK(evt_color_set), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1, GTK_EXPAND|GTK_FILL,0,5,0);
 				break;
 		}
@@ -684,7 +666,7 @@ void cmc_field_remove(CmdCfg *cmc, const gchar *name)
 	{
 		CField	*fld;
 
-		if((fld = field_find(cmc, name)) != NULL)		
+		if((fld = field_find(cmc, name)) != NULL)
 		{
 			cmc->fields = g_list_remove(cmc->fields, fld);
 			field_destroy(fld);
