@@ -30,8 +30,8 @@ CtrlKey *	ctrl_key_add_unique(CtrlInfo *ctrl);
 void		ctrl_key_remove(CtrlInfo *ctrl, CtrlKey *key);
 void		ctrl_key_remove_by_name(CtrlInfo *ctrl, const gchar *keyname);
 void		ctrl_key_remove_all(CtrlInfo *ctrl);
-const gchar *	ctrl_key_get_keyname(CtrlKey *key);
-const gchar *	ctrl_key_get_cmdseq(CtrlKey *key);
+const gchar *	ctrl_key_get_keyname(const CtrlKey *key);
+const gchar *	ctrl_key_get_cmdseq(const CtrlKey *key);
 void		ctrl_key_set_keyname(CtrlInfo *ctrl, CtrlKey *key, const gchar *keyname);
 void		ctrl_key_set_cmdseq(CtrlInfo *ctrl, CtrlKey *key, const gchar *cmdseq);
 gboolean	ctrl_key_has_cmdseq(CtrlInfo *ctrl, CtrlKey *key, const gchar *cmdseq);
@@ -47,9 +47,9 @@ CtrlMouse *	ctrl_mouse_add(CtrlInfo *ctrl, guint button, guint state, const gcha
 void		ctrl_mouse_set_button(CtrlMouse *mouse, guint state);
 void		ctrl_mouse_set_state(CtrlMouse *mouse, guint state);
 void		ctrl_mouse_set_cmdseq(CtrlMouse *mouse, const gchar *cmdseq);
-guint		ctrl_mouse_get_button(CtrlMouse *mouse);
-guint		ctrl_mouse_get_state(CtrlMouse *mouse);
-const gchar *	ctrl_mouse_get_cmdseq(CtrlMouse *mouse);
+guint		ctrl_mouse_get_button(const CtrlMouse *mouse);
+guint		ctrl_mouse_get_state(const CtrlMouse *mouse);
+const gchar *	ctrl_mouse_get_cmdseq(const CtrlMouse *mouse);
 void		ctrl_mouse_remove(CtrlInfo *ctrl, CtrlMouse *mouse);
 void		ctrl_mouse_remove_all(CtrlInfo *ctrl);
 const gchar *	ctrl_mouse_map(CtrlInfo *ctrl, GdkEventButton *evt);
