@@ -136,7 +136,7 @@ const gchar * dpf_get_content_title(DPContent content)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 2000-04-16 -	Build and return a menu containing all the content type names. */
-GtkWidget * dpf_get_content_menu(GtkSignalFunc sigfunc, gpointer user)
+GtkWidget * dpf_get_content_menu(GCallback sigfunc, gpointer user)
 {
 	GtkWidget	*menu, *item;
 	guint		i;
@@ -146,7 +146,7 @@ GtkWidget * dpf_get_content_menu(GtkSignalFunc sigfunc, gpointer user)
 	{
 		item = gtk_menu_item_new_with_label(_(content_info[i].name_full));
 		gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(content_info[i].id));
-		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(sigfunc), user);
+		g_signal_connect(GTK_OBJECT(item), "activate", sigfunc, user);
 		gtk_menu_append(GTK_MENU(menu), item);
 	}
 	return menu;
@@ -281,7 +281,7 @@ void dpf_init_defaults(CfgInfo *cfg)
 	cfg->dp_paning.value	   = 0.0;
 
 	cfg->dp_history.select = TRUE;
-	cfg->dp_history.save   = TRUE;	/* Slightly out of place, but hey. */
+	cfg->dp_history.save = TRUE;	/* Slightly out of place, but hey. */
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -317,7 +317,7 @@ static void put_size(gchar **out, const DC_Size *dcs, const DirRow *row)
 	if(!(S_ISDIR(DP_ROW_LSTAT(row).st_mode) && dcs->dir_show_fs_size) && !DP_ROW_FLAGS_CHK(row, DPRF_HAS_SIZE))
 		return;
 	if(dcs->type == DPSIZE_NATIVE)			/* Native gets it done right away, in a single line. */
-		g_snprintf(*out, 64, "%Lu", (guint64) DP_ROW_LSTAT(row).st_size);
+		g_snprintf(*out, 64, "%" G_GUINT64_FORMAT, (guint64) DP_ROW_LSTAT(row).st_size);
 	else
 	{
 		gdouble	size = DP_ROW_LSTAT(row).st_size;
@@ -388,13 +388,13 @@ void dpf_format_column(MainInfo *min, const DirPane *dp, const DirRow *row, guin
 
 	switch(col_fmt->content)
 	{
-		case DPC_NAME:
+		case DPC_NAME:	/* GTK2: Need more stuff here, for symlinks. */
 			if(col_fmt->extra.name.show_type)
-				format_typed_name(*out, DP_ROW_NAME(row), DP_ROW_LSTAT(row).st_mode);
+				format_typed_name(*out, DP_ROW_NAMED(row), DP_ROW_LSTAT(row).st_mode);
 			else
-				sprintf(*out, "%s", DP_ROW_NAME(row));
-			if(S_ISLNK(DP_ROW_LSTAT(row).st_mode) && col_fmt->extra.name.show_linkname && DP_ROW_LINKNAME(row)[0])
-				sprintf(strchr(*out, '\0'), " -> %s", DP_ROW_LINKNAME(row));
+				sprintf(*out, "%s", DP_ROW_NAMED(row));
+			if(S_ISLNK(DP_ROW_LSTAT(row).st_mode) && col_fmt->extra.name.show_linkname && DP_ROW_LINKNAMED(row)[0])
+				sprintf(strchr(*out, '\0'), " -> %s", DP_ROW_LINKNAMED(row));
 			break;
 		case DPC_SIZE:
 			put_size(out, &col_fmt->extra.size, row);
