@@ -10,6 +10,7 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include "convutil.h"
 #include "dirpane.h"
 #include "dialog.h"
 #include "guiutil.h"
@@ -17,6 +18,7 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* Description of the error. This is in UTF-8. */
 static gchar	error_desc[1024] = "";
 
 /* ----------------------------------------------------------------------------------------- */
@@ -51,23 +53,31 @@ gint err_set(MainInfo *min, gint code, const gchar *source, const gchar *obj)
 
 	if(source != NULL)
 	{
+		Conv	conv;
+		gchar	*src_display, *obj_display;
+
+		/* Assume that the source and object names are in the filesystem encoding, and convert. */
+		conv_init(&conv);
+		src_display = conv_begin_reverse(&conv, source);
+		obj_display = conv_add_reverse(&conv, obj);
 		if(code >= 0)
 		{
-			gchar	*desc;
+			const gchar	*desc;
 
 			desc = g_strerror(code);
 			if(obj != NULL)
-				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\": %s (code %d)"), source, obj, desc, code);
+				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\": %s (code %d)"), src_display, obj_display, desc, code);
 			else
-				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\" (code %d)"), source, obj, code);
+				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\" (code %d)"), src_display, obj_display, code);
 		}
 		else
 		{
 			if(obj != NULL)
-				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\""), source, obj);
+				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s \"%s\""), src_display, obj_display);
 			else
-				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s"), source);
+				g_snprintf(error_desc, sizeof error_desc, _("Couldn't %s"), src_display);
 		}
+		conv_end(&conv);
 	}
 	errno = old_errno;
 	return 1;
