@@ -8,28 +8,28 @@
 ** #include "me.h" (for a module named "me.c", that is).
 **
 ** The reason why this file should be the first included one is that it fixes some
-** system-dependant things which need to go before any system headers are included.
+** system-dependent things which need to go before any system headers are included.
 */
 
 #include "config.h"
 
 #if !(defined __osf__ && defined __alpha__) && !defined __NetBSD__ && !defined __FreeBSD__ && !defined __sgi
-#if !defined __EXTENSIONS__
-#define __EXTENSIONS__
-#endif
-#if !defined _POSIX_C_SOURCE
-#define _POSIX_C_SOURCE	3	/* This is for Solaris. */
-#endif
-#define	POSIX_C_SOURCE	3
+ #if !defined __EXTENSIONS__
+  #define __EXTENSIONS__
+ #endif
+ #if !defined _POSIX_C_SOURCE
+  #define _POSIX_C_SOURCE	3	/* This is for Solaris. */
+ #endif
+ #define POSIX_C_SOURCE	3
 #endif				/* !__osf__ && !__alpha */
 
 #if defined __osf__ && defined __alpha__		/* On Tru64, this should bring in mknod(). */
-#define	_XOPEN_SOURCE_EXTENDED
-#define	_OSF_SOURCE					/* For MAXNAMLEN on Tru64. */
+ #define _XOPEN_SOURCE_EXTENDED
+ #define _OSF_SOURCE					/* For MAXNAMLEN on Tru64. */
 #endif
 
 #if !defined _BSD_SOURCE
-#define	_BSD_SOURCE					/* For MAXNAMLEN on Linux. */
+ #define _BSD_SOURCE					/* For MAXNAMLEN on Linux. */
 #endif
 
 #include <dirent.h>
@@ -68,6 +68,7 @@
 
 #define	GTK_DISABLE_COMPAT_H		/* No GTK+ 1.0.6-compatibility required. */
 #include <gtk/gtk.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 /* Not all systems' <mman.h> seem to define this. */
 #if !defined MAP_FAILED
@@ -91,7 +92,6 @@
 
 #if defined ENABLE_NLS
 #include <locale.h>
-#include <langinfo.h>
 /* These two are used to mark translatable strings in the source code.
 ** _() is used in contexts where C allows a function call, N_() where
 ** it does not. Strings marked with N_() still need to be passed to _()
@@ -279,8 +279,8 @@ typedef struct {
 #define	CBAF_RESCAN_DEST	(1<<1)
 #define	CBAF_CD_SOURCE		(1<<2)		/* These save some config work. */
 #define	CBAF_CD_DEST		(1<<3)
-#define	CBAF_REQSEL_SOURCE	(1<<4)		/* Command won't run if there's no source selection when envoked. */
-#define	CBAF_REQSEL_DEST	(1<<5)		/* Command won't run if there's no destination selection when envoked. */
+#define CBAF_REQSEL_SOURCE	(1<<4)		/* Command won't run if there's no source selection when envoked. */
+#define CBAF_REQSEL_DEST	(1<<5)		/* Command won't run if there's no destination selection when envoked. */
 
 typedef struct {		/* Extra info for external commands. */
 	guint32	gflags;			/* General flags. */
@@ -377,7 +377,7 @@ typedef struct {			/* Holds all configuration info. */
 	DPFormat	dp_format[2];
 	DPColors	dp_colors;		/* Global color information used to render dirpanes. */
 	DPPaning	dp_paning;		/* Controls how the paned view acts. */
-	DPHistory	dp_history;		/* History list options. */
+	DPHistory	dp_history;		/* History list option. */
 	OptOverwrite	opt_overwrite;
 	MenuInfo	*menus;
 	ButtonInfo	buttons;
@@ -394,7 +394,7 @@ typedef struct {			/* Holds all configuration info. */
 
 typedef struct GuiInfo	GuiInfo;
 
-struct MainInfo {				/* gentoo's single most important data structure. It's everywhere. */
+struct MainInfo {				/* gentoo's single most central data structure. */
 	OptDesc		*opt;
 	gsize		opt_num;
 	CfgInfo		cfg;
@@ -405,19 +405,20 @@ struct MainInfo {				/* gentoo's single most important data structure. It's ever
 
 /* -- Directory content data structures -- */
 
-/* This is used for tab-completion of paths. Easy. */
+/* This is used for complete-as-you-type of paths. */
 typedef struct {
-	glob_t	buf;		/* Result of a glob() call from original Tab press. */
-	GList	*list;		/* List items referencing into the above. */
-	GList	*iter;		/* Iterator for current position in list. */
-	guint	change_sig;	/* Signal for change handler on path entry widget. */
+	gchar			prefix[PATH_MAX];	/* The path prefix that the GtkCompletion holds strings for. */
+	guint			change_sig;		/* Signal for change handler on path entry widget. */
+	GtkEntryCompletion	*compl;			/* A GTK+ 2.0 completion object, attached to the pane's path entry. */
 } PathComplete;
 
 /* These macros are used to access fields of a DirRow structure. */
-#define	DP_ROW_NAME(r)		((r)->dr_name)
+#define	DP_ROW_NAME(r)		((r)->dr_names.disk)
+#define	DP_ROW_NAMED(r)		((r)->dr_names.display)
 #define	DP_ROW_LSTAT(r)		((r)->dr_lstat)
 #define	DP_ROW_STAT(r)		(*((r)->dr_stat))
-#define	DP_ROW_LINKNAME(r)	((r)->dr_linkname)
+#define	DP_ROW_LINKNAME(r)	((r)->dr_linknames.disk)
+#define	DP_ROW_LINKNAMED(r)	((r)->dr_linknames.display)
 #define	DP_ROW_FLAGS(r)		((r)->dr_flags)
 #define	DP_ROW_FLAGS_SET(r,f)	(DP_ROW_FLAGS(r) |= (f))
 #define	DP_ROW_FLAGS_CLR(r,f)	(DP_ROW_FLAGS(r) &= ~(f))
@@ -427,13 +428,15 @@ typedef struct {
 
 #define	DP_ROW_SET_STAT(r,s)	(((r)->dr_stat) = (s))
 #define	DP_ROW_GET_STAT(r)	((r)->dr_stat)
-#define	DP_ROW_SET_LINKNAME(r,n) ((r)->dr_linkname = (n))
+#define	DP_ROW_SET_LINKNAME(r,n) ((r)->dr_linknames.disk = (n))
+#define	DP_ROW_SET_LINKNAMED(r,n) ((r)->dr_linknames.display = (n))
 
 /* These macros are to be used when accessing a DirRow through a selection list, which
 ** is the most frequent case. The 's' parameter is the SList iterator.
 */
 #define	DP_SEL_ROW(s)		((DirRow *) (s)->data)
 #define	DP_SEL_NAME(s)		(DP_ROW_NAME(DP_SEL_ROW(s)))
+#define	DP_SEL_NAMED(s)		(DP_ROW_NAMED(DP_SEL_ROW(s)))
 #define	DP_SEL_LSTAT(s)		(DP_ROW_LSTAT(DP_SEL_ROW(s)))
 #define	DP_SEL_STAT(s)		(DP_ROW_STAT(DP_SEL_ROW(s)))
 #define	DP_SEL_LINKNAME(s)	(DP_ROW_LINKNAME(DP_SEL_ROW(s)))
@@ -448,10 +451,16 @@ enum {	DPRF_HAS_SIZE = 1 << 0,			/* Set when a row has a "real" size (not set fo
 	DPRF_LINK_EXISTS = 1 << 1		/* Set when a symlink's target exists. */
 };
 
+typedef struct {			/* A filename, in both on-disk and display formats. */
+	const gchar	*disk;			/* On-disk name, as read during directory scanning. */
+	const gchar	*display;		/* Display, as returned by glib (generally, this will be UTF-8). */
+} DRName;
+
 typedef struct {			/* Representation of a single line in a directory (a file, typically). */
-	const gchar	*dr_name;		/* Name, points into big block in DirContents. */
+	DRName		dr_names;		/* Actual filename, in two representations. */
+	DRName		dr_linknames;		/* Link name, again in two representations. */
+	const gchar	*dr_name, *dr_linkname;	/* Backward compatibility. */
 	struct stat	dr_lstat;		/* From a call to lstat(). */
-	const gchar	*dr_linkname;		/* Contents of symbolic link (if S_ISLNK() true on dr_lstat). Else NULL. */
 	struct stat	*dr_stat;		/* If link, this holds link target stats. Else, it points at dr_lstat. */
 	guint32		dr_flags;		/* Misc. flags. */
 	const FType	*dr_type;		/* Type information. */
@@ -473,6 +482,7 @@ typedef struct {			/* Some trivial file system information. Updated on rescan. *
 
 typedef struct {
 	gchar		path[PATH_MAX];		/* Contents apply to this path. */
+	gchar		*pathd;			/* Dynamically allocated display version of the path. Beware! */
 	guint		num_rows;		/* Number of valid rows. */
 	guint		tot_dirs, tot_files;	/* Total number of entries in directory. */
 	guint64		tot_bytes, tot_blocks;	/* Sum of all sizes (in bytes & blocks). */
@@ -489,42 +499,46 @@ typedef struct {
 /* -- Main DirPane structure --------------------------------------------------------------- */
 
 struct DirPane {
-	MainInfo	*main;		/* Handy to have around. */
-	guint		index;		/* Index of *this* pane, in the grand scheme of things. */
+	MainInfo	*main;			/* Handy to have around. */
+	guint		index;			/* Index of *this* pane, in the grand scheme of things. */
 	GtkWidget	*vbox;
-	GtkWidget	*notebook;	/* Notebook holding (normally) path widgetry. */
-	GtkWidget	*path;		/* Combo widget showing path, and path history. */
-	PathComplete	complete;	/* Information for completing the path. */
-	GtkWidget	*hide;		/* A toggle button showing (and controlling) hide status. */
-	GtkWidget	*scwin;		/* Scrolling window for clist to live in. */
-	GtkCList	*list;		/* The actual clist that shows all stuff. */
-	GtkWidget	*menu_top;	/* The top-level menu, typically shown by right-clicking. */
-	GtkWidget	*mitem_action;	/* The item in the top-level menu that contains the actions submenu. */
-	GtkWidget	*menu_action;	/* The menu containing the intersection of all available actions on the selection. */
-	DirContents	dir;		/* Contents of directory (list of names etc). */
-	DirHistory	*hist;		/* Historic data about previously visited directories. */
-	gint		last_row;	/* The last clicked row. */
-	gint		last_row2;	/* The second last clicked row. */
-	gint		dbclk_row;	/* The row that was double clicked, or -1. */
-	gint		focus_row;	/* The last row to be rendered focused, or -1. */
-	gint		old_focus_row;	/* Used to remember focus when pane is deactivated. */
+	GtkWidget	*notebook;		/* Notebook holding (normally) path widgetry. */
+	GtkWidget	*path;			/* GTK+ 2.0 GtkComboBox showing path and path history. */
+	PathComplete	complete;		/* Information for completing the path. */
+	GtkWidget	*hide;			/* A toggle button showing (and controlling) hide status. Focus target! */
+	GtkWidget	*scwin;			/* Scrolling window for clist to live in. */
+	GtkCList	*list;			/* The actual clist that shows all stuff. */
+	GtkWidget	*menu_top;		/* The top-level menu, typically shown by right-clicking. */
+	GtkWidget	*mitem_action;		/* The item in the top-level menu that contains the actions submenu. */
+	GtkWidget	*menu_action;		/* The menu containing the intersection of all available actions on the selection. */
+	DirContents	dir;			/* Contents of directory (list of names etc). */
+	DirHistory	*hist;			/* Historic data about previously visited directories. */
+	gint		last_row;		/* The last clicked row. */
+	gint		last_row2;		/* The second last clicked row. */
+	gint		dbclk_row;		/* The row that was double clicked, or -1. */
+	gint		focus_row;		/* The last row to be rendered focused, or -1. */
+	gint		old_focus_row;		/* Used to remember focus when pane is deactivated. */
+	guint		sig_path_activate;	/* Signal for the "activate" event on path entry combo box. */
+	guint		sig_path_changed;	/* Signal for the "changed" event on the path entry combo box. */
 };
 
 /* -- Graphic user interface stuff --------------------------------------------------------- */
 
 struct GuiInfo {
 	GtkWidget	*window;
-	guint		win_del_evt;
+	guint		sig_main_configure;
+	guint		sig_main_delete;
+	gboolean	win_configured;
 	GtkTooltips	*main_tt;		/* Tooltips for main window. Used by dirpane widgets. */
 	KbdContext	*kbd_ctx;		/* Keyboard context, for shortcuts. */
 	GtkWidget	*vbox;			/* Vbox that contains entire GUI. */
-	GtkWidget	*top;			/* A label showing status and stuff. Handy to focus. */
+	GtkWidget	*top;			/* A label showing status (selections, free spac, etc). */
 	GtkWidget	*panes;			/* GtkPaned widget holding DirPanes. */
+	guint		sig_pane_notify;	/* Signal handler ID for notify::position. */
 	GtkWidget	*middle;		/* A box holding entire middle part. */
 	GtkWidget	*bottom;		/* Bottom part of GUI. */
 	DirPane		pane[2];
 	DirPane		*cur_pane;
-	guint		sig_left_sizealloc;	/* Handler ID for "size_allocate" signal on left pane. */
 
 	gboolean	evt_button_valid;
 	GdkEventButton	evt_button;
