@@ -69,6 +69,7 @@ static struct {
 	guint		block_cnt;		/* If set, don't rescan. */
 	guint		block_rescan;		/* Used to track what needs to be reloaded on unblock(). */
 	guint		timeout[2];		/* Used to notify of pending refresh after rate limiting. */
+	gint		next_req;		/* Index number to use for next request, increments. */
 } the_faminfo = { FALSE };
 
 /* ----------------------------------------------------------------------------------------- */
@@ -156,10 +157,7 @@ static void rescan_issue(guint rescan)
 		if(rescan & (1 << i))
 		{
 			if(rescan_rate_limited(i))
-			{
-			/*	dp_rescan(&the_faminfo.min->gui->pane[i]);*/
 				f |= the_faminfo.min->gui->cur_pane == &the_faminfo.min->gui->pane[i];
-			}
 		}
 	}
 	if(f)
@@ -172,14 +170,23 @@ static void rescan_issue(guint rescan)
 static void evt_fam_monitor(gpointer data, gint source, GdkInputCondition cond)
 {
 	FAMEvent	evt;
-	guint		rescan = 0U;
+	guint		rescan = 0U, i;
 
 	while(FAMPending(&the_faminfo.conn))
 	{
 		if(FAMNextEvent(&the_faminfo.conn, &evt) == 1)	/* Returns -1 on error, annoyingly enough. */
 		{
-			if(evt.code < FAMAcknowledge)		/* Filter out events that wouldn't show up anyway. */
-				rescan |= 1 << (evt.fr.reqnum - MAGIC_OFFSET);
+			/* Explicitly filter, keeping only events that make sense. */
+			if(evt.code == FAMChanged || evt.code == FAMDeleted ||
+			   evt.code == FAMCreated || evt.code == FAMMoved)
+			{
+				/* Figure out which pane this is from, and set bit. */
+				for(i = 0; i < sizeof the_faminfo.min->gui->pane / sizeof *the_faminfo.min->gui->pane; i++)
+				{
+					if(evt.fr.reqnum == the_faminfo.req[i].reqnum)
+						rescan |= (1 << i);
+				}
+			}
 		}
 	}
 	if(the_faminfo.block_cnt)
@@ -192,12 +199,13 @@ static void evt_fam_monitor(gpointer data, gint source, GdkInputCondition cond)
 gboolean fam_initialize(MainInfo *min)
 {
 	the_faminfo.min = min;
+	the_faminfo.next_req = 0;
 	if(FAMOpen2(&the_faminfo.conn, PACKAGE) == 0)
 	{
 		guint	i;
 
 		for(i = 0; i < sizeof the_faminfo.req / sizeof *the_faminfo.req; i++)
-			the_faminfo.req[i].reqnum = 0;
+			the_faminfo.req[i].reqnum = -1;
 
 		/* Hook us into the main GTK+ event detection system. Simple. */
 		gtk_input_add_full(FAMCONNECTION_GETFD(&the_faminfo.conn), GDK_INPUT_READ, evt_fam_monitor, NULL, NULL, NULL);
@@ -222,12 +230,15 @@ void fam_monitor(const DirPane *dp)
 	if(!the_faminfo.ok)
 		return;
 	if(the_faminfo.req[dp->index].reqnum > 0)
-		FAMCancelMonitor(&the_faminfo.conn, &the_faminfo.req[dp->index]);
-	the_faminfo.req[dp->index].reqnum = MAGIC_OFFSET + dp->index;
+	{
+		if(FAMCancelMonitor(&the_faminfo.conn, &the_faminfo.req[dp->index]) != 0)
+			g_warning(_("Couldn't cancel FAM monitor: %s (restart with --no-fam to go around, perhaps)"), FamErrlist[FAMErrno]);
+	}
+	the_faminfo.req[dp->index].reqnum = ++the_faminfo.next_req;
 	if(FAMMonitorDirectory2(&the_faminfo.conn, dp->dir.path, &the_faminfo.req[dp->index]) != 0)
 	{
 		the_faminfo.req[dp->index].reqnum = 0;
-		g_warning(_("Couldn't add FAM monitor on \"%s\", error %s (restart with --no-fam to go around, perhaps)"), dp->dir.path, FamErrlist[FAMErrno]);
+		g_warning(_("Couldn't add FAM monitor on \"%s\": %s (restart with --no-fam to go around, perhaps)"), dp->dir.path, FamErrlist[FAMErrno]);
 	}
 }
 
