@@ -8,7 +8,6 @@
 #include <string.h>
 
 #include "dialog.h"
-#include "miscutil.h"
 #include "guiutil.h"
 
 /* ----------------------------------------------------------------------------------------- */
@@ -165,7 +164,7 @@ GtkWidget * gui_build_menu(const gchar *text[], GtkSignalFunc func, gpointer use
 		if(func != NULL)
 		{
 			gtk_object_set_user_data(GTK_OBJECT(item), GUINT_TO_POINTER(i));
-			gtk_signal_connect(GTK_OBJECT(item), "activate", func, user);
+			g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(func), user);
 		}
 		gtk_menu_append(GTK_MENU(menu), item);
 		gtk_widget_show(item);
