@@ -263,7 +263,7 @@ static void menu_build_action(MainInfo *min, GtkWidget *menu)
 			{
 				item = gtk_menu_item_new_with_label(iter->data);
 				gtk_object_set_user_data(GTK_OBJECT(item), min);
-				gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_menu_action_activate), iter->data);
+				g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(evt_menu_action_activate), iter->data);
 				gtk_menu_append(GTK_MENU(menu), item);
 				gtk_widget_show(item);
 			}
@@ -334,12 +334,12 @@ static void menu_build_parents(MainInfo *min, GtkWidget *menu)
 		if(canon[0] == '\0')
 			break;
 		item = gtk_menu_item_new_with_label(canon);
-		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_menu_parents_activate), min);
+		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(evt_menu_parents_activate), min);
 		gtk_menu_append(GTK_MENU(menu), item);
 	}
 	/* Always append a root ("/") item, since the root has itself as its parent. Avoids empty menu, as bonus. */
 	item = gtk_menu_item_new_with_label("/");
-	gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(evt_menu_parents_activate), min);
+	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(evt_menu_parents_activate), min);
 	gtk_menu_append(GTK_MENU(menu), item);
 	gtk_widget_show_all(menu);
 }
@@ -351,7 +351,7 @@ static void evt_menu_activate(GtkWidget *wid, gpointer user)
 {
 	MainInfo	*min;
 
-	min = gtk_object_get_user_data(GTK_OBJECT(wid));	
+	min = gtk_object_get_user_data(GTK_OBJECT(wid));
 	csq_execute(min, user);
 }
 
@@ -377,6 +377,9 @@ static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, M
 	guint		i;
 	MenuItem	*mit;
 
+	if(!name)
+		return NULL;
+
 	/* Dynamic menus are empty, and filled-in on demand in the "show" signal handler.
 	** This isn't the most pleasing solution performance-wise, but it's still rather
 	** elegant, IMO. The "hide" handler empties the menu again.
@@ -385,8 +388,8 @@ static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, M
 	{
 		menu = gtk_menu_new();
 		gtk_object_set_user_data(GTK_OBJECT(menu), mi->min);
-		gtk_signal_connect(GTK_OBJECT(menu), "show", GTK_SIGNAL_FUNC(evt_menu_show), d);
-		gtk_signal_connect(GTK_OBJECT(menu), "hide", GTK_SIGNAL_FUNC(evt_menu_hide), d);
+		g_signal_connect(G_OBJECT(menu), "show", G_CALLBACK(evt_menu_show), d);
+		g_signal_connect(G_OBJECT(menu), "hide", G_CALLBACK(evt_menu_hide), d);
 		return menu;
 	}
 
@@ -401,8 +404,8 @@ static GtkWidget * do_menu_widget_build(const MenuInfo *mi, const gchar *name, M
 		{
 			case ITEM_SIMPLE:
 				menuitem = gtk_menu_item_new_with_label(mit->data.simple.label.label);
-				gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-						   GTK_SIGNAL_FUNC(evt_menu_activate), mit->data.simple.cmdseq->str);
+				g_signal_connect(G_OBJECT(menuitem), "activate",
+						 G_CALLBACK(evt_menu_activate), mit->data.simple.cmdseq->str);
 				gtk_object_set_user_data(GTK_OBJECT(menuitem), mi->min);
 				break;
 			case ITEM_SEPARATOR:
@@ -475,8 +478,8 @@ gboolean mnu_menu_popup(const MenuInfo *mi, const gchar *name, GtkMenuPositionFu
 
 	if((menu = mnu_menu_widget_get(mi, name)) != NULL)
 	{
-		gtk_signal_connect_after(GTK_OBJECT(menu), "hide", GTK_SIGNAL_FUNC(evt_popup_hide), NULL);
-		gtk_signal_connect(GTK_OBJECT(menu), "destroy_event", GTK_SIGNAL_FUNC(evt_popup_destroy), NULL);
+		g_signal_connect_after(G_OBJECT(menu), "hide", G_CALLBACK(evt_popup_hide), NULL);
+		g_signal_connect(G_OBJECT(menu), "destroy_event", G_CALLBACK(evt_popup_destroy), NULL);
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, pos, data, button, activate_time);
 /*		fprintf(stderr, "Popping up menu at %p\n", menu);*/
 	}
@@ -599,10 +602,10 @@ const gchar * mnu_menuinfo_select(const MenuInfo *mi)
 		ri = gtk_clist_append(GTK_CLIST(clist), row);
 		gtk_clist_set_row_data(GTK_CLIST(clist), ri, row[0]);
 	}
-	gtk_signal_connect(GTK_OBJECT(clist), "select_row", GTK_SIGNAL_FUNC(evt_select_select_row), &selected);
-	gtk_signal_connect(GTK_OBJECT(clist), "unselect_row", GTK_SIGNAL_FUNC(evt_select_unselect_row), &selected);
+	g_signal_connect(G_OBJECT(clist), "select_row", G_CALLBACK(evt_select_select_row), &selected);
+	g_signal_connect(G_OBJECT(clist), "unselect_row", G_CALLBACK(evt_select_unselect_row), &selected);
 	gtk_container_add(GTK_CONTAINER(scwin), clist);
-	gtk_widget_set_usize(scwin, 200, 384);
+	gtk_widget_set_size_request(scwin, 200, 384);
 
 	dlg = dlg_dialog_sync_new(scwin, _("Select Menu"), NULL);
 	choice = dlg_dialog_sync_wait(dlg);
