@@ -14,11 +14,12 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include "fileutil.h"
 #include "dirpane.h"
+#include "fileutil.h"
 #include "guiutil.h"
 #include "miscutil.h"
 #include "sizeutil.h"
+
 #include "progress.h"
 
 /* ----------------------------------------------------------------------------------------- */
@@ -161,10 +162,10 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		pgs_info.tot_pos = 0;
 
 		pgs_info.dlg  = gtk_dialog_new();
-		gtk_widget_set_usize(pgs_info.dlg, 384, -1);
+		gtk_widget_set_size_request(pgs_info.dlg, 384, -1);
 		pgs_info.body = gtk_label_new(op_name);
-		gtk_signal_connect(GTK_OBJECT(pgs_info.dlg), "delete_event", GTK_SIGNAL_FUNC(evt_delete), &pgs_info.cancel);
-		gtk_signal_connect(GTK_OBJECT(pgs_info.dlg), "key_press_event", GTK_SIGNAL_FUNC(evt_keypress), &pgs_info.cancel);
+		g_signal_connect(G_OBJECT(pgs_info.dlg), "delete_event", G_CALLBACK(evt_delete), &pgs_info.cancel);
+		g_signal_connect(G_OBJECT(pgs_info.dlg), "key_press_event", G_CALLBACK(evt_keypress), &pgs_info.cancel);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.body, FALSE, FALSE, 0);
 
 		hbox = gtk_hbox_new(FALSE, 0);
@@ -216,7 +217,7 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		else
 			pgs_info.eta = NULL;
 		cancel = gtk_button_new_with_label(_("Cancel"));
-		gtk_signal_connect(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(evt_cancel_clicked), &pgs_info.cancel);
+		g_signal_connect(G_OBJECT(cancel), "clicked", G_CALLBACK(evt_cancel_clicked), &pgs_info.cancel);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->action_area), cancel, TRUE, TRUE, 0);
 
 		g_get_current_time(&pgs_info.time_begin);
@@ -288,7 +289,7 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 		{
 			gint	micro;
 
-			micro = 1E6 * (time_now.tv_sec  - pgs_info.time_begin.tv_sec) + 
+			micro = 1E6 * (time_now.tv_sec  - pgs_info.time_begin.tv_sec) +
 				      (time_now.tv_usec - pgs_info.time_begin.tv_usec);
 
 			if(micro >= pgs_info.delay_time)
@@ -296,6 +297,8 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 		}
 		if(!pgs_info.delayed)
 		{
+			gint	old_errno;
+
 			if(pgs_info.eta != NULL)
 			{
 				gfloat	secs = msu_diff_timeval(&pgs_info.time_begin, &time_now);
@@ -315,9 +318,11 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 					gtk_label_set_text(GTK_LABEL(pgs_info.eta), buf);
 				}
 			}
+			old_errno = errno;
 			gtk_widget_realize(pgs_info.dlg);
 			gdk_window_set_group(pgs_info.dlg->window, pgs_info.min->gui->window->window);
 			gtk_widget_show_all(pgs_info.dlg);
+			errno = old_errno;
 		}
 		if(pgs_info.show_byte && pgs_info.byte_tot)
 			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.byte_pgs), (gfloat) (pgs_info.byte_pos + pos) / pgs_info.byte_tot);
@@ -333,8 +338,11 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 /* 1998-09-30 -	Done with an item. */
 void pgs_progress_item_end(MainInfo *min)
 {
+	gint	old_errno = errno;
+
 	pgs_info.byte_pos += pgs_info.item_size;
 	gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), (gfloat) pgs_info.tot_pos / pgs_info.tot_num);
 	gui_events_flush();
-}
 
+	errno = old_errno;
+}
