@@ -5,6 +5,8 @@
 
 #include "gentoo.h"
 
+#include <stdlib.h>
+
 #include "cmdseq.h"
 #include "strutil.h"
 #include "styles.h"
@@ -72,6 +74,17 @@ struct property_rename_data {
 			name_new[STL_PROPERTY_NAME_SIZE];
 };
 
+/* This is used when looking for a style in a GtkTreeModel. */
+struct tree_find_data {
+	const Style	*style;
+	GtkTreeIter	iter;
+};
+
+/* This is used to avoid magic numbers all over the place. */
+enum {
+	TREE_COLUMN_NAME = 0, TREE_COLUMN_STYLE
+};
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-05-20 -	Create a new, empty, styleinfo. */
@@ -181,7 +194,7 @@ static void style_update(GNode *node, gpointer user)
 }
 
 /* 1999-05-19 -	Update style tree in <si>; causes all non-overridden properties in all styles to
-**		aquire the value of the nearest overriding parent. Simple.
+**		acquire the value of the nearest overriding parent. Simple.
 */
 static void styleinfo_update(StyleInfo *si)
 {
@@ -701,7 +714,7 @@ static void load_style(const XmlNode *node, gpointer user)
 
 	xml_get_text(node, "name", &name);
 	stl = stl_style_new(name);
-	
+
 	if(xml_get_boolean(node, "expand", &btmp))
 		stl_style_set_expand(stl, btmp);
 	if((prop = xml_tree_search(node, "Properties")) != NULL)
@@ -786,8 +799,8 @@ static void build_branch(GNode *node, GtkWidget *parent, Style *ignore, gboolean
 			tree = gtk_tree_new();
 			build_branch(node->children, tree, ignore, modify_flag);
 			gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), tree);
-			gtk_signal_connect(GTK_OBJECT(item), "expand", GTK_SIGNAL_FUNC(evt_style_expand), modify_flag);
-			gtk_signal_connect(GTK_OBJECT(item), "collapse", GTK_SIGNAL_FUNC(evt_style_collapse), modify_flag);
+			g_signal_connect(G_OBJECT(item), "expand", G_CALLBACK(evt_style_expand), modify_flag);
+			g_signal_connect(G_OBJECT(item), "collapse", G_CALLBACK(evt_style_collapse), modify_flag);
 		}
 		if(((Style *) node->data)->expand)
 			gtk_tree_item_expand(GTK_TREE_ITEM(item));
@@ -818,35 +831,103 @@ GtkTree * stl_styleinfo_build_partial(StyleInfo *si, Style *ignore)
 	return GTK_TREE(tree);
 }
 
-/* ----------------------------------------------------------------------------------------- */
-
-/* 1999-05-26 -	Check if <wid> is the GtkTreeItem used to build the Style in user->style. */
-static void find_widget(GtkWidget *wid, gpointer user)
+/* 2006-03-02 -	Append a branch of the style tree to the given store. This does not collapse/expand it
+ *		properly, since that's not doable with GTK+2 at this level (it's a view property).
+*/
+static void append_branch2(GtkTreeStore *store, const GNode *node, GtkTreeIter *parent, const Style *ignore, gboolean *modify_flag)
 {
-	struct widget_find_data	*wfd = user;
-	GtkWidget	*subtree;
+	GtkTreeIter	child;
 
-	if(gtk_object_get_user_data(GTK_OBJECT(wid)) == wfd->style)
-		wfd->widget = wid;
-	else if(GTK_IS_TREE_ITEM(wid) && ((subtree = GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(wid))) != NULL))
-		gtk_container_foreach(GTK_CONTAINER(subtree), find_widget, user);
+	for(; (node != NULL); node = node->next)
+	{
+		if((ignore != NULL) && (node->data == ignore))
+			continue;
+		gtk_tree_store_append(store, &child, parent);
+		gtk_tree_store_set(store, &child, TREE_COLUMN_NAME, ((const Style *) node->data)->name,
+						  TREE_COLUMN_STYLE, node->data,
+						  -1);
+		if(node->children != NULL)
+			append_branch2(store, node->children, &child, ignore, modify_flag);
+	}
 }
 
-/* 1999-05-26 -	Return the widget (typically a GtkTreeItem) used to represent <stl> of <si> in
-**		<tree>. Of course, <tree> has been previously returned by stl_styleinfo_build_XXX().
-**		Note that because of the (IMHO) oddly limited semantics of gtk_container_foreach()
-**		when applied on a tree, we do further recursion in the callback. Messy, but OK.
-*/
-GtkWidget * stl_styleinfo_widget_find(StyleInfo *si, GtkTree *tree, Style *stl)
+/* 2006-03-02 -	Build a partial style store, ignoring any branch rooted at <ignore>. */
+GtkTreeStore * stl_styleinfo_build_partial2(const StyleInfo *si, const Style *ignore)
 {
-	struct widget_find_data	wfd = { NULL, NULL };
+	GtkTreeStore	*store;
 
-	if((si != NULL) && (tree != NULL) && (stl != NULL))
+	store = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
+	append_branch2(store, si->styles, NULL, ignore, NULL);
+
+	return store;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
+/* 2009-03-22 -	We need a way to read out a Style pointer from a constructed tree, in the config. */
+Style * stl_styleinfo_get_style_iter(const StyleInfo *si, GtkTreeStore *store, GtkTreeIter *iter)
+{
+	Style	*style = NULL;
+
+	if(si == NULL || store == NULL || iter == NULL)
+		return NULL;
+	gtk_tree_model_get(GTK_TREE_MODEL(store), iter, TREE_COLUMN_STYLE, &style, -1);
+
+	return style;
+}
+
+/* Simple bookkeeping struct used by stl_styleinfo_tree_find_style(), below. */
+struct find_info
+{
+	const Style	*style;
+	gboolean	found;
+	GtkTreeIter	iter;
+};
+
+static gboolean cb_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+{
+	struct find_info	*fi = data;
+	Style		*here;
+
+	gtk_tree_model_get(model, iter, TREE_COLUMN_STYLE, &here, -1);
+	if(here == fi->style)
 	{
-		wfd.style = stl;
-		gtk_container_foreach(GTK_CONTAINER(tree), find_widget, &wfd);
+		fi->iter = *iter;
+		fi->found = TRUE;
 	}
-	return wfd.widget;
+	return fi->found;
+}
+
+/* 2009-03-24 -	Find the given style in the given store, and set the iter accordingly.
+**		This is a straight tree search, so it takes a while, but this really
+**		shouldn't be a problem.
+*/
+gboolean stl_styleinfo_tree_find_style(const StyleInfo *si, GtkTreeStore *store, const Style *style, GtkTreeIter *iter)
+{
+	struct find_info	fi;
+
+	fi.style = style;
+	fi.found = FALSE;
+	gtk_tree_model_foreach(GTK_TREE_MODEL(store), cb_foreach, &fi);
+	if(fi.found)
+	{
+		if(iter != NULL)
+			*iter = fi.iter;
+	}
+	return fi.found;
+}
+
+/* 2008-05-11 -	Set the name of a style, based on a GtkTreeIter. This will change not only
+ *		the value in the tree model, but also the style itself.
+*/
+void stl_styleinfo_set_name_iter(const StyleInfo *si, GtkTreeStore *store, GtkTreeIter *iter, const gchar *name)
+{
+	Style	*style;
+
+	if((style = stl_styleinfo_get_style_iter(si, store, iter)) == NULL)
+		return;
+	stl_style_set_name(style, name);
+	gtk_tree_store_set(store, iter, TREE_COLUMN_NAME, name, -1);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -1055,6 +1136,8 @@ static void value_destroy(StlPVal *val)
 {
 	if(val != NULL)
 	{
+		if(val->type > 4)
+			abort();
 		switch(val->type)
 		{
 			case SPT_COLOR:
@@ -1077,21 +1160,23 @@ static StlProp * style_get_property(Style *stl, const gchar *name, StlPType type
 
 	if((pr = g_hash_table_lookup(stl->properties, name)) != NULL)
 	{
-		if(pr->value->type != type)
-			fprintf(stderr, "STYLES: Style '%s' has wrong-type property '%s' (%d, not %d)\n", stl->name, name, pr->value->type, type);
-		if(pr->override == TRUE)
-			return pr;
+		if(pr->override)
+		{
+			if(pr->value->type == type)
+				return pr;
+			/* Only destroy if overriden, else we shared pointer. */
+			value_destroy(pr->value);
+		}
 	}
 	else
 	{
 		pr = g_malloc(sizeof *pr);
 		stu_strncpy(pr->name, name, sizeof pr->name);
+		g_hash_table_insert(stl->properties, pr->name, pr);
 	}
 	pr->override = TRUE;
 	pr->value    = value_new(type);
 
-	g_hash_table_insert(stl->properties, pr->name, pr);
-
 	return pr;
 }
 
@@ -1219,9 +1304,10 @@ gboolean stl_style_property_get_override(Style *stl, const gchar *property)
 static void get_action(gpointer key, gpointer value, gpointer user)
 {
 	GList	**list = user;
+	StlProp	*prop = value;
 
-	if(((StlProp *) value)->value->type == SPT_ACTION)
-		*list = g_list_insert_sorted(*list, ((StlProp *) value)->name, (GCompareFunc) strcmp);
+	if(prop->value->type == SPT_ACTION)
+		*list = g_list_insert_sorted(*list, prop->name, (GCompareFunc) strcmp);
 }
 
 /* 1999-05-24 -	Return a list of (very read-only) action property names. The list is valid
