@@ -18,7 +18,6 @@
 
 #include <ctype.h>
 #include <fcntl.h>
-#include <regex.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -36,8 +35,6 @@
 #define	HEX_ROW_SIZE	(16)		/* Must be multiple of four! */
 #define	HEX_UNPRINTABLE	('.')
 
-#define	LABEL_LIMIT	(64)		/* Keep reasonably large. */
-
 /* ----------------------------------------------------------------------------------------- */
 
 typedef enum { MVE_UP = 0, MVE_DOWN, MVE_TOP, MVE_BOTTOM } MoveID;
@@ -64,6 +61,7 @@ typedef struct {
 	MainInfo	*min;			/* Incredibly handy. */
 	GtkWidget	*win;
 	GtkWidget	*scwin;
+	GObject		*buffer;		/* GtkTextBuffer object holding the actual text. */
 	GtkWidget	*text;
 	GtkWidget	*bhbox;			/* Horizontal box with action buttons. */
 	GtkWidget	*blabel;		/* A label that is in the action box. */
@@ -109,6 +107,7 @@ static void really_destroy(GtkWidget *wid)
 			dlg_dialog_sync_close(srch_dialog.dlg, -1);
 			srch_dialog.dlg = NULL;
 		}
+		g_object_unref(txi->buffer);
 		g_free(txi);
 		win_window_close(wid);
 	}
@@ -122,17 +121,82 @@ static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 	return TRUE;
 }
 
+/* 2008-03-15 -	Helper routine since appending text now needs a bit more thinking to handle UTF-8.
+ *		Returns FALSE is conversion failed and the caller should probably give up.
+*/
+static gboolean do_text_append(TxvInfo *txi, const gchar *text, gsize length)
+{
+	GtkTextIter	iter;
+	const gchar	*eptr;
+
+	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+	if(g_utf8_validate(text, length, &eptr))
+		gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, text, length);
+	else
+	{
+		gchar	*utf8;
+		gsize	bin, bout;
+		GError	*err = NULL;
+
+		if((utf8 = g_locale_to_utf8(text, length, &bin, &bout, &err)) != NULL)
+		{
+			gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, utf8, bout);
+			g_free(utf8);
+		}
+		if(err != NULL)
+		{
+			const gchar	*local = NULL;
+
+			g_get_charset(&local);
+			err_printf(txi->min, _("Unable to convert text from native character set (%s) to UTF-8: %s"), local, err->message);
+			g_error_free(err);
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 /* 1999-03-02 -	Users of this module no longer have access to the GtkText widget, so
 **		this function is needed for them to be able to actually display text.
 */
-void txv_put_text(GtkWidget *wid, const gchar *text, gint length)
+void txv_text_append(GtkWidget *wid, const gchar *text, gsize length)
 {
-	TxvInfo	*txi;
+	TxvInfo		*txi;
 
 	if((wid == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) == NULL) || length < 1)
 		return;
+	do_text_append(txi, text, length);
+}
 
-	gtk_text_insert(GTK_TEXT(txi->text), NULL, NULL, NULL, text, length);
+/* 2008-09-26 -	Append a block of text, and give it the given color too. Spiffy! */
+void txv_text_append_with_color(GtkWidget *wid, const gchar *text, gsize length, const GdkColor *color)
+{
+	TxvInfo		*txi;
+	GtkTextIter	start, end;
+	gchar		tname[32];
+	gint		pos;
+	GtkTextTagTable	*ttab;
+	GtkTextTag	*tag;
+
+	if(color == NULL)
+	{
+		txv_text_append(wid, text, length);
+		return;
+	}
+	if((wid == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) == NULL) || length < 1)
+		return;
+	pos = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(txi->buffer));
+	do_text_append(txi, text, length);
+	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &start, pos);
+	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &end);
+
+	/* Be clever. Build a tag name that depends on the color, to re-use tags. */
+	g_snprintf(tname, sizeof tname, "txv-%04x%04x%04x", color->red, color->green, color->blue);
+	/* If a tag by this name already exists, re-use it. */
+	ttab = gtk_text_buffer_get_tag_table(GTK_TEXT_BUFFER(txi->buffer));
+	if((tag = gtk_text_tag_table_lookup(ttab, tname)) == NULL) 
+		tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(txi->buffer), tname, "foreground-gdk", color, NULL);
+	gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(txi->buffer), tag, &start, &end);
 }
 
 /* 1998-12-18 -	Rewrote this a couple of times today. Note how this version really doesn't
@@ -146,9 +210,8 @@ void txv_put_text(GtkWidget *wid, const gchar *text, gint length)
 **		using normal file I/O if mmap() fails, which can happen on certain filesystems.
 ** 1999-03-02 -	New interface, renamed.
 ** 1999-04-06 -	Added the <buf_size> argument, which is used when mmap() isn't.
-** 2008-07-08 -	The <buf_size> argument is now used in mmap mode too, but with a different value.
 */
-gint txv_load_text(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize buf_size)
+gint txv_text_load(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize buf_size)
 {
 	gchar	*buf;
 	gint	fd, ret = 0;
@@ -167,14 +230,7 @@ gint txv_load_text(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize b
 		if(use_mmap && fut_size(name, &size) && (size > 0) &&
 		   (buf = mmap(NULL, size, PROT_READ, MAP_PRIVATE, fd, 0)) != MAP_FAILED)
 		{
-			gsize	chunk;
-
-			for(; size > 0; size -= chunk, buf += chunk)
-			{
-				chunk = size > buf_size ? buf_size : size;
-				
-				gtk_text_insert(GTK_TEXT(txi->text), NULL, NULL, NULL, buf, (gint) chunk);
-			}
+			do_text_append(txi, buf, size);
 			munmap(buf, size);
 		}
 		else
@@ -183,7 +239,10 @@ gint txv_load_text(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize b
 
 			buf = g_malloc(buf_size);
 			while((got = read(fd, buf, buf_size)) > 0)
-				gtk_text_insert(GTK_TEXT(txi->text), NULL, NULL, NULL, buf, (gint) got);
+			{
+				if(!do_text_append(txi, buf, got))
+					break;
+			}
 			g_free(buf);
 		}
 		close(fd);
@@ -233,7 +292,7 @@ static void build_hex_text(const guchar *data, gchar *text, gsize bytes)
 **		Now we do plenty of small read()s instead, which I'm sure is slower, but at
 **		least doesn't waste memory.
 */
-gint txv_load_text_hex(GtkWidget *win, const gchar *name)
+gint txv_text_load_hex(GtkWidget *win, const gchar *name)
 {
 	gchar		line[8 + 1 + (HEX_ROW_SIZE / 4) * 9 + 1 + HEX_ROW_SIZE + 1 + 1],
 			group[9 * (HEX_ROW_SIZE / 4) + 1], text[HEX_ROW_SIZE + 1];
@@ -241,6 +300,7 @@ gint txv_load_text_hex(GtkWidget *win, const gchar *name)
 	gint		fd, len, chunk;
 	gsize		offset = 0;
 	TxvInfo		*txi;
+/*	GtkTextTag	*tag;*/
 
 	if((win == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(win))) == NULL) || (name == NULL))
 		return 0;
@@ -253,9 +313,12 @@ gint txv_load_text_hex(GtkWidget *win, const gchar *name)
 	text[sizeof text - 1]   = '\0';
 
 	err_clear(txi->min);
-	gtk_text_freeze(GTK_TEXT(txi->text));
+/*	gtk_text_freeze(GTK_TEXT(txi->text));*/
 	if((fd = open(name, O_RDONLY)) >= 0)
 	{
+		GtkTextIter	iter;
+
+		gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
 		while((chunk = read(fd, buf, sizeof buf)) > 0)
 		{
 			if(chunk < sizeof buf)
@@ -266,13 +329,27 @@ gint txv_load_text_hex(GtkWidget *win, const gchar *name)
 			build_hex_group(buf, group, chunk);
 			build_hex_text(buf, text, chunk);
 			len = g_snprintf(line, sizeof line, "%08lX %s %s\n", (unsigned long) offset, group, text);
-			gtk_text_insert(GTK_TEXT(txi->text), NULL, NULL, NULL, line, len);
+			gtk_text_buffer_insert(GTK_TEXT_BUFFER(txi->buffer), &iter, line, len);
 			offset += chunk;
 		}
 		close(fd);
 	}
-	gtk_text_thaw(GTK_TEXT(txi->text));
+/*	gtk_text_thaw(GTK_TEXT(txi->text));*/
 
+/*	if((tag = gtk_text_tag_new("fixed")) != NULL)
+	{
+		GtkTextIter	start, end;
+		GtkTextTagTable	*tt;
+
+		g_object_set(G_OBJECT(tag), "font", "fixed", NULL);
+		tt = gtk_text_buffer_get_tag_table(GTK_TEXT_BUFFER(txi->buffer));
+		gtk_text_tag_table_add(tt, tag);
+		gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &start);
+		gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &end);
+		gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(txi->buffer), tag, &start, &end);
+		g_object_unref(G_OBJECT(tag));
+	}
+*/
 	if(errno != 0)
 		err_set(txi->min, errno, _("File reading"), (gchar *) name);
 	return errno != 0;
@@ -285,7 +362,10 @@ static void reset_search(TxvInfo *txi)
 {
 	if(txi != NULL)
 	{
-		gtk_editable_select_region(GTK_EDITABLE(txi->text), 0, 0);
+		GtkTextIter	iter;
+
+		gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+		gtk_text_buffer_select_range(GTK_TEXT_BUFFER(txi->buffer), &iter, &iter);
 		txi->search_pos = 0;
 	}
 }
@@ -313,14 +393,6 @@ static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 	return TRUE;
 }
 
-/* 2004-04-09 -	Close on right-button press. */
-static gint evt_button_press(GtkWidget *wid, GdkEventButton *evt, gpointer user)
-{
-	if(evt->type == GDK_BUTTON_PRESS && evt->button == 3)
-		really_destroy(wid);
-	return FALSE;
-}
-
 /* 1999-01-07 -	This handler gets called when the vertical range (scrollbar) changes. This
 **		should probably also include the number of lines available, but that is a
 **		rather fuzzy quantity, so I'll just leave that for now.
@@ -328,13 +400,10 @@ static gint evt_button_press(GtkWidget *wid, GdkEventButton *evt, gpointer user)
 static gint evt_pos_changed(GtkAdjustment *adj, gpointer user)
 {
 	TxvInfo		*txi = user;
-	GtkStyle	*stl;
 	gchar		buf[64];
-	gint		fh = 10;
 	gfloat		p;
-
-	if((stl = gtk_widget_get_style(txi->text)) != NULL)
-		fh = stl->font->ascent + stl->font->descent;
+	GtkTextIter	iter;
+	gint		bx, by;
 
 	if(adj->upper - adj->page_size > 0.0f)
 	{
@@ -344,7 +413,11 @@ static gint evt_pos_changed(GtkAdjustment *adj, gpointer user)
 	}
 	else
 		p = 0.0f;
-	g_snprintf(buf, sizeof buf, _("Line %d (%.0f%%)"), (gint) adj->value / fh, p);
+	
+	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(txi->text), GTK_TEXT_WINDOW_WIDGET, 0, 0, &bx, &by);
+	gtk_text_view_get_line_at_y(GTK_TEXT_VIEW(txi->text), &iter, by, NULL);
+	by = gtk_text_iter_get_line(&iter) + 1;
+	g_snprintf(buf, sizeof buf, _("Line %d (%.0f%%)"), by, p);
 	gtk_label_set_text(GTK_LABEL(txi->bplabel), buf);
 
 	return TRUE;
@@ -357,28 +430,32 @@ static gint evt_move_clicked(GtkWidget *wid, gpointer user)
 {
 	TxvInfo		*txi = user;
 	GtkAdjustment	*adj;
-	GtkStyle	*stl;
-	gint		fh = 10;
+	GtkTextIter	iter;
+	gint		bx, by;
+
+	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(txi->text), GTK_TEXT_WINDOW_WIDGET, 0, 0, &bx, &by);
+	gtk_text_view_get_line_at_y(GTK_TEXT_VIEW(txi->text), &iter, by, NULL);
 
-	if((stl = gtk_widget_get_style(txi->text)) != NULL)
-		fh = stl->font->ascent + stl->font->descent;
 	if((adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))) != NULL)
 	{
 		switch((MoveID) GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid))))
 		{
 			case MVE_UP:
-				gtk_adjustment_set_value(adj, adj->value - fh);
+				gtk_text_view_backward_display_line(GTK_TEXT_VIEW(txi->text), &iter);
+				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
 				break;
 			case MVE_DOWN:
-				if(adj->value < adj->upper - adj->page_size)
-					gtk_adjustment_set_value(adj, adj->value + fh);
+				gtk_text_view_forward_display_line(GTK_TEXT_VIEW(txi->text), &iter);
+				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
 				break;
 			case MVE_TOP:
-				gtk_adjustment_set_value(adj, adj->lower);
+				gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
 				reset_search(txi);
 				break;
 			case MVE_BOTTOM:
-				gtk_adjustment_set_value(adj, adj->upper - adj->page_size);
+				gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+				gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
 				reset_search(txi);
 				break;
 		}
@@ -404,29 +481,37 @@ static void evt_goto_clicked(GtkWidget *wid, gpointer user)
 	if((dlg_dialog_sync_wait(goto_dialog.dlg) == DLG_POSITIVE) &&
 	   (adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))) != NULL)
 	{
-		GtkStyle	*stl;
-		gchar		*txt, unit;
-		gint		value, fh = 10;
-		gfloat		pos = -1.0f;
+		const gchar	*txt;
+		gchar		unit;
+		gint		value;
+		GtkTextIter	iter;
 
 		txt = gtk_entry_get_text(GTK_ENTRY(goto_dialog.entry));
-		if((stl = gtk_widget_get_style(txi->text)) != NULL)
-			fh = stl->font->ascent + stl->font->descent;
 		if(sscanf(txt, "0x%x", &value) == 1)			/* Hexadecimal offset? */
-			pos = (gfloat) (fh * value / 16);
+			gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &iter, value);
 		else if(sscanf(txt, "%d%c", &value, &unit) == 2)
 		{
-			if(unit == '%')					/* Percentage? */
-				pos = value * (adj->upper - adj->page_size) / 100.0;
-			else if(unit == 'K')					/* Kilobytes? Silly if not hex-view. */
-				pos = (gfloat) fh * value * (1024 / 16);
-			else if(unit == 'M')					/* Someone might view huge files. */
-				pos = (gfloat) fh * value * (1024 * 1024 / 16);
+			/* Kind of overly clever this part, perhaps. */
+			switch(unit)
+			{
+			case '%':
+				value = (adj->upper * value) / 100;
+				gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(txi->buffer), &iter, value);
+				break;
+			case 'M':
+			case 'm':
+				value <<= 10;	/* Yes, only 10, 10 more below. */
+				/* Fall-through, re-use get_iter call. */
+			case 'K':
+			case 'k':
+				value <<= 10;
+				gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &iter, value);
+				break;
+			}
 		}
 		else if(sscanf(txt, "%d", &value) == 1)			/* Line number? */
-			pos = (gfloat) fh * value;
-		if(pos >= 0.0 && pos <= (adj->upper - adj->page_size))
-			gtk_adjustment_set_value(adj, pos);
+			gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(txi->buffer), &iter, value);
+		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
 	}
 	if(goto_dialog.dlg != NULL)
 	{
@@ -456,73 +541,66 @@ static gint evt_close_clicked(GtkWidget *wid, gpointer user)
 **		want to dig *that* deep into the GtkText widget. There is an easy way to
 **		accomplish this that eliminates this problem: use a insert/delete pair.
 **		However, that causes the widget to *scroll* to the required position. Yawn.
+** 2008-03-09 -	Then came GTK+, TextIters, and life became simpler in some ways.
 */
 static void show_text(TxvInfo *txi, gint pos)
 {
-	GtkAdjustment	*adj;
-	GtkStyle	*stl;
-	gchar		*base, *text;
-	gfloat		nv;
-	gint		line, fh = 10;
+	GtkTextIter	iter;
 
-	if((base = gtk_editable_get_chars(GTK_EDITABLE(txi->text), 0, -1)) != NULL)
-	{
-		for(line = 0, text = base; *text && pos; pos--, text++)
-		{
-			if(*text == '\n')		/* Probably breaks on non-Unix systems. So? */
-				line++;
-		}
-		if((stl = gtk_widget_get_style(txi->text)) != NULL)
-		{
-			fh = stl->font->ascent + stl->font->descent;
-			if((adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))) != NULL)
-			{
-				nv = fh * line;
-				if(nv >= 0 && nv <= (adj->upper - adj->page_size))
- 					gtk_adjustment_set_value(adj, nv);
-			}
-		}
-		g_free(base);
-	}
+	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &iter, pos);
+	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(txi->text), &iter, 0.0, TRUE, 0.0, 0.0);
 }
 
 /* 1999-01-07 -	Do the actual search, using <re_src> as the regular expression source. This
 **		should really begin with '(', end with ')', and don't contain any non-escaped
 **		parentheses within!
+** 2008-03-15 -	Ported to use glib's GRegex data structures, for fun.
 */
 static gboolean do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboolean nolf)
 {
-	regex_t		re;
-	regmatch_t	rm[1];
-	gchar		*text, ebuf[1024];
-	gint		found = FALSE, cerr, cflags = REG_EXTENDED;
+	GRegex		*re;
+	GError		*err = NULL;
+	gchar		*text;
+	gint		found = FALSE, cflags = G_REGEX_EXTENDED | G_REGEX_MULTILINE;
 
 	if(nocase)
-		cflags |= REG_ICASE;
+		cflags |= G_REGEX_CASELESS;
 	if(nolf)
-		cflags |= REG_NEWLINE;
+		cflags |= G_REGEX_DOTALL;
 
-	if((cerr = regcomp(&re, re_src, cflags)) == 0)
+	if((re = g_regex_new(re_src, cflags, G_REGEX_MATCH_NOTBOL, &err)) != NULL)
 	{
-		if((text = gtk_editable_get_chars(GTK_EDITABLE(txi->text), txi->search_pos, -1)) != NULL)
+		GtkTextIter	start, end;
+
+		gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &start, txi->search_pos);
+		gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(txi->buffer), &end);
+		if((text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(txi->buffer), &start, &end, TRUE)) != NULL)
 		{
-			if(regexec(&re, text, sizeof rm / sizeof rm[0], rm, 0) == 0)
+			GMatchInfo	*match = NULL;
+
+			if(g_regex_match(re, text, 0, &match))
 			{
-				show_text(txi, txi->search_pos + rm[0].rm_so);
-				gtk_editable_select_region(GTK_EDITABLE(txi->text), txi->search_pos + rm[0].rm_so,
-								txi->search_pos + rm[0].rm_eo);
-				txi->search_pos += rm[0].rm_eo;
+				gint	startpos, endpos;
+
+				if(g_match_info_fetch_pos(match, 0, &startpos, &endpos))
+				{
+					show_text(txi, txi->search_pos + startpos);
+					gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &start, txi->search_pos + startpos);
+					gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(txi->buffer), &end, txi->search_pos + endpos);
+					gtk_text_buffer_select_range(GTK_TEXT_BUFFER(txi->buffer), &start, &end); 
+					txi->search_pos += endpos;
+				}
 				found = TRUE;
 			}
+			g_match_info_free(match);
 			g_free(text);
 		}
-		regfree(&re);
+		g_regex_unref(re);
 	}
-	else
+	if(err != NULL)
 	{
-		stu_strncpy(ebuf, _("Regular expression error:\n"), sizeof ebuf);
-		regerror(cerr, &re, ebuf + strlen(ebuf), sizeof ebuf - strlen(ebuf) - 1);
-		dlg_dialog_async_new_error(ebuf);
+		dlg_dialog_async_new_error(err->message);
+		g_error_free(err);
 	}
 	return found;
 }
@@ -553,7 +631,7 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 	gtk_widget_grab_focus(srch_dialog.entry);
 	if(dlg_dialog_sync_wait(srch_dialog.dlg) == DLG_POSITIVE)
 	{
-		gchar	*txt;
+		const gchar	*txt;
 		GString	*tmp;
 
 		txt = gtk_entry_get_text(GTK_ENTRY(srch_dialog.entry));
@@ -610,10 +688,9 @@ static void do_search_repeat(TxvInfo *txi)
 */
 GtkWidget * txv_open(MainInfo *min, const gchar *label)
 {
-	GtkWidget	*vbox, *btn, *algn, *bhbox;
+	GtkWidget	*vbox, *btn, *bhbox;
 	TxvInfo		*txi;
 	GtkAccelGroup	*accel;
-	guint		key;
 
 	txi = g_malloc(sizeof *txi);
 
@@ -623,86 +700,81 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 
 	txi->win = win_window_open(min->cfg.wininfo, WIN_TEXTVIEW);
 	gtk_object_set_user_data(GTK_OBJECT(txi->win), txi);
-	gtk_signal_connect(GTK_OBJECT(txi->win), "delete_event", GTK_SIGNAL_FUNC(evt_delete), txi);
-	gtk_signal_connect(GTK_OBJECT(txi->win), "button_press_event", GTK_SIGNAL_FUNC(evt_button_press), txi);
+	g_signal_connect(G_OBJECT(txi->win), "delete_event", G_CALLBACK(evt_delete), txi);
 	txv_connect_keypress(txi->win, NULL, NULL);
 
 	vbox = gtk_vbox_new(FALSE, 0);
 
-	txi->text = gtk_text_new(NULL, NULL);
-	gtk_text_set_word_wrap(GTK_TEXT(txi->text), FALSE);
-	gtk_text_set_editable(GTK_TEXT(txi->text), FALSE);
+	txi->buffer = G_OBJECT(gtk_text_buffer_new(NULL));
+	txi->text = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(txi->buffer));
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(txi->text), FALSE);
 	gtk_widget_set_name(txi->text, "txvText");
-	gtk_signal_connect(GTK_OBJECT(GTK_TEXT(txi->text)->vadj), "value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), txi);
 
 	txi->scwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(txi->scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_signal_connect(GTK_OBJECT(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))),
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(txi->scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	g_signal_connect_after(GTK_OBJECT(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin))),
 				"value_changed", GTK_SIGNAL_FUNC(evt_pos_changed), txi);
 	gtk_container_add(GTK_CONTAINER(txi->scwin), txi->text);
 
 	gtk_box_pack_start(GTK_BOX(vbox), txi->scwin, TRUE, TRUE, 0);
 
 	txi->bhbox = gtk_hbox_new(FALSE, 0);
-	algn = gtk_alignment_new(0.01, 0.5, 0, 0);
 	if(label != NULL)
 		txi->blabel = gtk_label_new(label);
 	else
 		txi->blabel = gtk_label_new("");
-	gtk_container_add(GTK_CONTAINER(algn), txi->blabel);
-	gtk_box_pack_start(GTK_BOX(txi->bhbox), algn, TRUE, TRUE, 10);
+	gtk_label_set_ellipsize(GTK_LABEL(txi->blabel), PANGO_ELLIPSIZE_MIDDLE);
+	gtk_misc_set_alignment(GTK_MISC(txi->blabel), 0.0f, 0.5f);
+	gtk_box_pack_start(GTK_BOX(txi->bhbox), txi->blabel, TRUE, TRUE, 10);
 
-	algn = gtk_alignment_new(0.99, 0.5, 0, 0);
 	txi->bplabel = gtk_label_new("");
-	gtk_container_add(GTK_CONTAINER(algn), txi->bplabel);
-	gtk_box_pack_start(GTK_BOX(txi->bhbox), algn, TRUE, TRUE, 0);
+	gtk_misc_set_alignment(GTK_MISC(txi->bplabel), 1.0f, 0.5f);
+	gtk_box_pack_start(GTK_BOX(txi->bhbox), txi->bplabel, FALSE, FALSE, 0);
 	
 	accel = gtk_accel_group_new();
 
-	btn = gui_arrow_button_new(GTK_ARROW_UP);
+	btn = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_UP));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gui_arrow_button_new(GTK_ARROW_DOWN);
+	btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_DOWN));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_with_label(_("Top"));
+	btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_TOP);
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_TOP));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_Home, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_with_label(_("Bottom"));
+	btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_BOTTOM);
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_BOTTOM));
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_End, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 
 	bhbox = gtk_hbox_new(FALSE, 0);
-	btn = gtk_button_new_with_label("");
-	key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(btn)->child), _("_Goto..."));
-	gtk_widget_add_accelerator(btn, "clicked", accel, key, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_goto_clicked), txi);
+	btn = gtk_button_new_from_stock(GTK_STOCK_JUMP_TO);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_goto_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_with_label("");
-	key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(btn)->child), _("_Search..."));
-	gtk_widget_add_accelerator(btn, "clicked", accel, key, 0, 0);
+
+	btn = gtk_button_new_from_stock(GTK_STOCK_FIND);
 	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_F, GDK_CONTROL_MASK, 0);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_search_clicked), txi);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_search_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), bhbox, FALSE, FALSE, 5);
 
-	btn = gtk_button_new_with_label("");
-	key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(btn)->child), _("_Quit"));
-	gtk_widget_add_accelerator(btn, "clicked", accel, key, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_close_clicked), txi);
+	btn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_close_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, TRUE, 0);
 	gtk_widget_set_sensitive(txi->bhbox, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), txi->bhbox, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(txi->win), vbox);
 	gtk_window_add_accel_group(GTK_WINDOW(txi->win), accel);
-	gtk_widget_show_all(vbox);
 
+	/* Now that all is built, initialize position-displaying label. */
+	evt_pos_changed(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin)), txi);
+
+	gtk_widget_show_all(vbox);
 	gtk_widget_grab_focus(txi->text);
 
 	return txi->win;
@@ -736,7 +808,7 @@ guint txv_connect_keypress(GtkWidget *wid, GtkSignalFunc func, gpointer user)
 		func = GTK_SIGNAL_FUNC(evt_keypress);
 		user = wid;
 	}
-	return txi->keypress_handler = gtk_signal_connect(GTK_OBJECT(wid), "key_press_event", func, user);
+	return txi->keypress_handler = g_signal_connect(G_OBJECT(wid), "key_press_event", G_CALLBACK(func), user);
 }
 
 /* 1999-01-05 -	Set the label. This is kind of redundant, but true to the original Opus look. :)
@@ -748,49 +820,11 @@ void txv_set_label(GtkWidget *wid, const gchar *text)
 
 	if(wid != NULL && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL && text != NULL)
 	{
-		guint	tl = strlen(text);
-
-		/* If wanted label is very long, cut it down to avoid window resizing. */
-		if(tl > LABEL_LIMIT)
-		{
-			static gchar	short_name[LABEL_LIMIT];
-			gint		h = sizeof short_name / 2, i;
-
-			/* Copy tail of original string. Includes terminator. */
-			for(i = 0; i < h; i++)
-				short_name[sizeof short_name - 1 - i] = text[tl - i];
-			/* Now copy the beginning. */
-			for(i = 0; i < h - 3; i++)
-				short_name[i] = text[i];
-			/* Insert periods to indicate shortened version. */
-			short_name[i++] = '.';
-			short_name[i++] = '.';
-			short_name[i++] = '.';
-			text = short_name;
-		}
 		win_window_set_title(wid, text);
 		gtk_label_set_text(GTK_LABEL(txi->blabel), text);
 	}
 }
 
-/* 1999-04-03 -	Freeze the text viewing part of the txv. Handy before massive insertions. */
-void txv_freeze(GtkWidget *wid)
-{
-	TxvInfo	*txi;
-
-	if((wid != NULL) && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
-		gtk_text_freeze(GTK_TEXT(txi->text));
-}
-
-/* 1999-04-03 -	Thaw the text viewer up. */
-void txv_thaw(GtkWidget *wid)
-{
-	TxvInfo	*txi;
-
-	if((wid != NULL) && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
-		gtk_text_thaw(GTK_TEXT(txi->text));
-}
-
 /* 1999-03-02 -	Enable the text widget for viewing. Called once all insertions are done. */
 void txv_enable(GtkWidget *wid)
 {
@@ -799,7 +833,7 @@ void txv_enable(GtkWidget *wid)
 	if((wid != NULL) && ((txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL))
 	{
 		gtk_widget_show(wid);	/* Just in case. */
-		gtk_adjustment_set_value(GTK_ADJUSTMENT(GTK_TEXT(txi->text)->vadj), 0.0f);
+/*		gtk_adjustment_set_value(GTK_ADJUSTMENT(GTK_TEXT(txi->text)->vadj), 0.0f);*/
 		gtk_widget_set_sensitive(txi->bhbox, TRUE);
 	}
 	else
