@@ -23,12 +23,10 @@ extern guint	txv_connect_keypress(GtkWidget *wid, GtkSignalFunc func, gpointer u
 
 extern void	txv_set_label(GtkWidget *wid, const gchar *text);
 
-extern void	txv_freeze(GtkWidget *wid);
-extern void	txv_thaw(GtkWidget *wid);
-
-extern void	txv_put_text(GtkWidget *wid, const gchar *text, gint length);
-extern gint	txv_load_text(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize buf_size);
-extern gint	txv_load_text_hex(GtkWidget *wid, const gchar *name);
+extern void	txv_text_append(GtkWidget *wid, const gchar *text, gsize length);
+extern void	txv_text_append_with_color(GtkWidget *wid, const gchar *text, gsize length, const GdkColor *color);
+extern gint	txv_text_load(GtkWidget *wid, const gchar *name, gboolean use_mmap, gsize buf_size);
+extern gint	txv_text_load_hex(GtkWidget *wid, const gchar *name);
 
 extern void	txv_enable(GtkWidget *wid);
 
