@@ -30,6 +30,7 @@
 
 #include <stddef.h>
 
+#include "errors.h"
 #include "strutil.h"
 #include "styles.h"
 #include "fileutil.h"
@@ -37,12 +38,14 @@
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* This list is used between typ_identify_begin() and typ_identify_end() to hold directory
-** lines that seem to need to be shown to the external 'file' command.
-*/
-static GList	*file_list = NULL;
-/* Another global (shudder), this one signals if there *are* any 'file'-using types around. */
-static gboolean	file_used = FALSE;
+/* Collected variables that deal with 'file' into a struct, for clarity. */
+static struct
+{
+	gboolean		file_used;		/* Any types using 'file' active? */
+	GSList			*files;			/* Current list of files to inspect. */
+	gboolean		sigpipe_installed;	/* SIGPIPE handler installed? */
+	volatile sig_atomic_t	sigpipe_occured;	/* Did we just catch SIGPIPE? */
+} file_info;
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -122,7 +125,7 @@ void typ_type_destroy(FType *type)
 {
 	if(type->name_re != NULL)
 	{
-		regfree(type->name_re);		
+		regfree(type->name_re);
 		g_free(type->name_re);
 	}
 	if(type->file_re != NULL)
@@ -135,6 +138,18 @@ void typ_type_destroy(FType *type)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2009-03-13 -	Return the Unknown type. */
+FType * typ_type_get_unknown(const CfgInfo *cfg)
+{
+	const GList	*iter;
+
+	for(iter = cfg->type; iter != NULL; iter = g_list_next(iter))
+	{
+		if(TYP_IS_UNKNOWN((const FType *) iter->data))
+			return iter->data;
+	}
+	return NULL;
+}
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -271,14 +286,16 @@ static gint check_type(FType *type, DirRow *row, const gchar *fout)
 */
 static gint identify(MainInfo *min, DirRow *row)
 {
-	GList	*here;
+	const GList	*here;
 
 	for(here = min->cfg.type; here != NULL; here = g_list_next(here))
 	{
 		if(check_type((FType *) here->data, row, NULL))
+		{
+			DP_ROW_TYPE(row) = here->data;
 			break;
+		}
 	}
-	DP_ROW_TYPE(row) = here->data;
 	return !TYP_IS_UNKNOWN((FType *) here->data);
 }
 
@@ -294,7 +311,7 @@ void typ_identify_begin(MainInfo *min)
 {
 	GList	*here;
 
-	if(file_list != NULL)
+	if(file_info.files != NULL)
 		fprintf(stderr, "**TYPES: Attempted to nest calls to typ_identify_begin()!\n");
 
 	for(here = min->cfg.type; here != NULL; here = g_list_next(here))
@@ -302,7 +319,7 @@ void typ_identify_begin(MainInfo *min)
 		if(((FType *) here->data)->flags & FTFL_FILEMATCH)
 			break;
 	}
-	file_used = (here != NULL);
+	file_info.file_used = (here != NULL);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -313,8 +330,8 @@ void typ_identify_begin(MainInfo *min)
 */
 void typ_identify(MainInfo *min, DirRow *row)
 {
-	if((identify(min, row) == FALSE) && (file_used == TRUE))
-		file_list = g_list_prepend(file_list, row);
+	if(!identify(min, row) && file_info.file_used)
+		file_info.files = g_slist_prepend(file_info.files, row);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -335,87 +352,118 @@ static FType * match_file(GList *list, DirRow *row, const gchar *fout)
 	return NULL;
 }
 
+/* 2009-03-11 -	Trivial SIGPIPE handler, that just sets a global flag. */
+static void sigpipe_handler(int sig)
+{
+	if(sig != SIGPIPE)
+		return;
+	file_info.sigpipe_occured = 1;
+}
+
+/* 2009-03-11 -	Installs a signal handler for SIGPIPE, so we can catch broken 'file' runs. */
+static gboolean install_handler(void)
+{
+	struct sigaction	act;
+
+	if(file_info.sigpipe_installed)
+		return TRUE;
+
+	act.sa_handler = sigpipe_handler;
+	sigemptyset(&act.sa_mask);
+	act.sa_flags	= 0;
+
+	return file_info.sigpipe_installed = (sigaction(SIGPIPE, &act, NULL) == 0);
+}
+
 /* 1998-08-26 -	End of batched identification. If any unknown's were found *and* we know that
 **		there are 1 or more types using the 'file' recognition support, we need to
 **		call 'file'. The idea here is that we only want to execute the 'file' command
 **		once, no matter how many files need identification. Also, we use redirection
 **		of stdin for 'file' to supply it with file names to check (via its "-f -" option),
 **		since writing out the names to disk seems inefficient.
+** 2009-03-08 -	Totally rewritten, now uses glib's fancy spawning to run the subprocess, and
+**		keeps it all going in pipes, without touching disk. This removes the need for
+**		a temporary filename, and should improve performance.
 */
 void typ_identify_end(MainInfo *min, const gchar *path)
 {
-	const GList	*here;
-	gchar		buf[MAXNAMLEN + 2], *temp_name;
-	gint		fd[2], len, status;
-	pid_t		file_pid;
-	FType		*type;
+	gchar	*argv[] = { "file", "-n", "-f", "-", NULL };
+	GPid	file_pid;
+	gint	file_stdin, file_stdout;
+	GError	*error = NULL;
 
-	if(file_list == NULL)
-		return;
-	if((temp_name = tmpnam(NULL)) == NULL)
-		return;
-	if(pipe(fd) != 0)
+	if(file_info.files == NULL)
 		return;
 
-	file_pid = fork();
-	if(file_pid == 0)
-	{
-		guint	bits = 0U;
+	install_handler();
 
-		if(close(STDIN_FILENO) == 0)
-		{
-			if(dup(fd[STDIN_FILENO]) == STDIN_FILENO)
-				bits |= (close(fd[STDIN_FILENO]) == 0);
-		}
-		if(close(STDOUT_FILENO) == 0)
-		{
-			if(open(temp_name, O_CREAT | O_RDWR, S_IRWXU) == STDOUT_FILENO)
-				bits |= (close(fd[STDOUT_FILENO]) == 0) << 1;
-		}
-		if(bits == 3U && chdir(path) == 0)
-			execlp("file", "file", "-f", "-", NULL);
-		_exit(EXIT_FAILURE);
-	}
-	else if(file_pid < 0)
-	{
-		g_warning("Couldn't fork() 'file' for file typing");
-		return;
-	}
-	/* We don't need the input part of the pipe, so close it. */
-	close(fd[STDIN_FILENO]);
-	/* Now, we have file listening on pipe, so write all filenames to it. */
-	for(here = file_list; here != NULL; here = g_list_next(here))
-	{
-		len = g_snprintf(buf, sizeof buf, "%s\n", DP_SEL_NAME(here));
-		write(fd[STDOUT_FILENO], buf, len);
-	}
-	close(fd[STDOUT_FILENO]);
-	waitpid(file_pid, &status, 0);
-
-	if(WIFEXITED(status))
+	if(g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+				    NULL, NULL, &file_pid, &file_stdin, &file_stdout, NULL, &error))
 	{
 		FILE	*in;
 
-		if((in = fopen(temp_name, "rt")) != NULL)
+		if((in = fdopen(file_stdout, "rt")) != NULL)
 		{
-			const gchar	*fout;
-			gchar		line[MAXNAMLEN + 256];
+			GSList	*here;
 
-			for(here = file_list; here != NULL && fgets(line, sizeof line, in); here = g_list_next(here))
+			for(here = file_info.files; here != NULL; here = g_slist_next(here))
 			{
-				if((fout = strchr(line, ':')) == NULL)
+				gchar	buf[FILENAME_MAX + 256];
+				size_t	len, to_go;
+				ssize_t	wrote;
+
+				len = g_snprintf(buf, sizeof buf, "%s\n", DP_SEL_NAME(here));
+				if(len >= sizeof buf)	/* Handle overflow by just skipping that file. */
 					continue;
-				for(fout++; isspace((guchar) *fout); fout++)
-					;
-				if((type = match_file(min->cfg.type, DP_SEL_ROW(here), fout)) != NULL)
-					DP_SEL_TYPE(here) = type;
+				for(to_go = len; to_go > 0; to_go -= wrote)
+				{
+					file_info.sigpipe_occured = 0;
+					err_clear(min);
+					wrote = write(file_stdin, buf + (len - to_go), to_go);
+					if(errno == EPIPE)
+						file_info.sigpipe_occured = 1;
+					if(wrote < 0)
+						break;
+					else if(file_info.sigpipe_occured)
+						break;
+				}
+				if(file_info.sigpipe_occured)
+					break;
+				if(fgets(buf, sizeof buf, in) != NULL)
+				{
+					const char	*fout;
+					FType		*type;
+
+					if((fout = strchr(buf, ':')) == NULL)
+						continue;
+					for(fout++; isspace(*fout); fout++)
+						;
+					if((type = match_file(min->cfg.type, DP_SEL_ROW(here), fout)) != NULL)
+						DP_SEL_TYPE(here) = type;
+				}
 			}
 			fclose(in);
 		}
+		close(file_stdin);
+		close(file_stdout);
+		g_spawn_close_pid(file_pid);
+		err_clear(min);
+		if(file_info.sigpipe_occured)
+		{
+			/* FIXME: This is problematic; the nice printf() text is often lost due to free space printing. */
+			err_printf(min, _("Got SIGPIPE when writing to 'file' process (%s), it seems to have terminated prematurely."), argv[0]);
+			/* FIXME: As a last resort, this at leas "sticks" so an error gets shown when we get out. */
+			errno = EPIPE;
+		}
 	}
-	g_list_free(file_list);
-	file_list = NULL;
-	remove(temp_name);
+	else
+	{
+		g_prefix_error(&error, _("Unable to run spawn 'file' command: "));
+		err_printf(min, "%s", error->message);
+		g_error_free(error);
+	}
+	g_slist_free(file_info.files);
+	file_info.files = NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
