@@ -6,11 +6,11 @@
 
 /* ------------------------------------------------------------------------------------------ */
 
-int		usr_init(void);
+gboolean	usr_init(void);
 
 const gchar *	usr_lookup_uname(uid_t uid);
 long		usr_lookup_uid(const gchar *name);
-const gchar *	usr_lookup_uhome(const char *uname);
+const gchar *	usr_lookup_uhome(const gchar *uname);
 const gchar *	usr_lookup_gname(gid_t gid);
 long		usr_lookup_gid(const gchar *name);
 
