@@ -1,8 +1,8 @@
 /*
-** 1999-12-23 -	Hm, perhaps this is an unfortunate choice of module title. It might bloat,
-**		or something. :) Anyway, the purpose of this module is to help handle
-**		(toplevel) windows, specifically remembering their sizes and positions.
-*/
+ ** 1999-12-23 -	Hm, perhaps this is an unfortunate choice of module title. It might bloat,
+ **		or something. :) Anyway, the purpose of this module is to help handle
+ **		(toplevel) windows, specifically remembering their sizes and positions.
+ */
 
 #include "gentoo.h"
 
@@ -10,44 +10,49 @@
 #include <stdlib.h>
 
 #include "guiutil.h"
+
 #include "window.h"
 
 /* ----------------------------------------------------------------------------------------- */
 
-typedef struct {
-	guint32		id;		/* Unique identifier for this windata. */
-	WinType		type;		/* What type of window? */
-	gboolean	persistent;	/* Makes close() actually do a hide(). */
-	const gchar	*title;		/* Text for window's title bar. */
-	const gchar	*label;		/* Short descriptive label (like "main", "config" etc). */
-	gboolean	pos_grab;
-	gboolean	size_grab;
+typedef struct
+{
+	guint32 id; /* Unique identifier for this windata. */
+	WinType type; /* What type of window? */
+	gboolean persistent; /* Makes close() actually do a hide(). */
+	const gchar *title; /* Text for window's title bar. */
+	const gchar *label; /* Short descriptive label (like "main", "config" etc). */
+	gboolean pos_grab;
+	gboolean size_grab;
 	/* Data below here is stored in config file. */
-	gint		x, y;		/* Window's opening position. */
-	gint		width, height;	/* The desired size of this window. */
-	gboolean	pos_use;	/* Should we set use the recorded position? */
-	gboolean	pos_update;	/* Update recorded position when window closes? */
-	gboolean	size_use;	/* Use recorded size? */
-	gboolean	size_update;	/* Update recorded size with current size on close? */
+	gint x, y; /* Window's opening position. */
+	gint width, height; /* The desired size of this window. */
+	gboolean pos_use; /* Should we set use the recorded position? */
+	gboolean pos_update; /* Update recorded position when window closes? */
+	gboolean size_use; /* Use recorded size? */
+	gboolean size_update; /* Update recorded size with current size on close? */
 } WinDef;
 
-struct WinInfo {
-	MainInfo	*min;		/* The handiness of this pointer cannot be overstated. */
-	GList		*windows;	/* List of WinDef:s as per above. There's no rush. */
+struct WinInfo
+{
+	MainInfo *min;			/* The handiness of this pointer cannot be overstated. */
+	GList *windows;			/* List of WinDef:s as per above. There's no rush. */
+	gint	border_width;		/* User's idea of her window manager's border size. Hackish. */
+	gint	border_height;
 };
 
 /* ----------------------------------------------------------------------------------------- */
 
-static WinDef *	window_find(const WinInfo *wi, guint32 id);
+static WinDef * window_find(const WinInfo *wi, guint32 id);
 
 /* ----------------------------------------------------------------------------------------- */
 
 WinInfo * win_wininfo_new(MainInfo *min)
 {
-	WinInfo	*wi;
+	WinInfo *wi;
 
 	wi = g_malloc(sizeof *wi);
-	wi->min	    = min;
+	wi->min = min;
 	wi->windows = NULL;
 
 	return wi;
@@ -56,24 +61,32 @@ WinInfo * win_wininfo_new(MainInfo *min)
 /* 1999-12-23 -	Create a new WinInfo, initialized with the "default" (i.e., legacy) windows. */
 WinInfo * win_wininfo_new_default(MainInfo *min)
 {
-	WinInfo	*wi;
+	WinInfo *wi;
 
 	wi = win_wininfo_new(min);
 
-	win_window_new(wi, WIN_MAIN,		WIN_TYPE_SIMPLE_TOPLEVEL, FALSE, geteuid() == 0 ? "gentoo [root]" : "gentoo", "main");
+	/* This is what seems to be right on my stock GNOME desktop.
+	** Mileage will vary, wildly.
+	*/
+	win_borders_set(wi, 4, 24);
+
+	win_window_new(wi, WIN_MAIN, WIN_TYPE_SIMPLE_TOPLEVEL, FALSE, geteuid()
+			== 0 ? "gentoo [root]" : "gentoo", "main");
 	win_window_pos_grab_set(wi, WIN_MAIN, TRUE);
 	win_window_size_set(wi, WIN_MAIN, 764, 932);
 	win_window_size_use_set(wi, WIN_MAIN, TRUE);
 	win_window_size_update_set(wi, WIN_MAIN, TRUE);
 	win_window_size_grab_set(wi, WIN_MAIN, TRUE);
 
-	win_window_new(wi, WIN_CONFIG,		WIN_TYPE_COMPLEX_DIALOG,  TRUE, _("Configure gentoo"), "config");
+	win_window_new(wi, WIN_CONFIG, WIN_TYPE_COMPLEX_DIALOG, TRUE,
+			_("Configure gentoo"), "config");
 	win_window_pos_use_set(wi, WIN_CONFIG, TRUE);
 	win_window_size_set(wi, WIN_CONFIG, -1, 464);
 	win_window_size_use_set(wi, WIN_CONFIG, TRUE);
 	win_window_size_update_set(wi, WIN_CONFIG, TRUE);
 
-	win_window_new(wi, WIN_TEXTVIEW,	WIN_TYPE_SIMPLE_TOPLEVEL, FALSE, _("Text Viewer"), "textview");
+	win_window_new(wi, WIN_TEXTVIEW, WIN_TYPE_SIMPLE_TOPLEVEL, FALSE,
+			_("Text Viewer"), "textview");
 	win_window_pos_use_set(wi, WIN_TEXTVIEW, TRUE);
 	win_window_size_set(wi, WIN_TEXTVIEW, 640, 480);
 	win_window_size_use_set(wi, WIN_TEXTVIEW, TRUE);
@@ -85,9 +98,9 @@ WinInfo * win_wininfo_new_default(MainInfo *min)
 /* 1999-12-23 -	Create a copy of <wi>, sharing no memory with it. */
 WinInfo * win_wininfo_copy(const WinInfo *wi)
 {
-	WinInfo	*nwi;
-	WinDef	*win;
-	GList	*iter;
+	WinInfo *nwi;
+	WinDef *win;
+	GList *iter;
 
 	nwi = win_wininfo_new(wi->min);
 
@@ -95,7 +108,8 @@ WinInfo * win_wininfo_copy(const WinInfo *wi)
 	{
 		win = iter->data;
 
-		win_window_new(nwi, win->id, win->type, win->persistent, win->title, win->label);
+		win_window_new(nwi, win->id, win->type, win->persistent,
+				win->title, win->label);
 		win_window_pos_grab_set(nwi, win->id, win->pos_grab);
 		win_window_pos_set(nwi, win->id, win->x, win->y);
 		win_window_pos_use_set(nwi, win->id, win->pos_use);
@@ -105,12 +119,14 @@ WinInfo * win_wininfo_copy(const WinInfo *wi)
 		win_window_size_use_set(nwi, win->id, win->size_use);
 		win_window_size_update_set(nwi, win->id, win->size_update);
 	}
+	nwi->border_width = wi->border_width;
+	nwi->border_height = wi->border_height;
 	return nwi;
 }
 
 void win_wininfo_destroy(WinInfo *wi)
 {
-	GList	*iter;
+	GList *iter;
 
 	for(iter = wi->windows; iter != NULL; iter = g_list_next(iter))
 		g_free(iter->data);
@@ -120,12 +136,15 @@ void win_wininfo_destroy(WinInfo *wi)
 
 /* ----------------------------------------------------------------------------------------- */
 
-enum { SUBFRAME_POSITION, SUBFRAME_SIZE };
+enum
+{
+	SUBFRAME_POSITION, SUBFRAME_SIZE
+};
 
 static void evt_boolean_clicked(GtkWidget *wid, gpointer user)
 {
-	gboolean	*flag = user;
-	gboolean	*modified;
+	gboolean *flag = user;
+	gboolean *modified;
 
 	*flag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
 	if((modified = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
@@ -134,8 +153,8 @@ static void evt_boolean_clicked(GtkWidget *wid, gpointer user)
 
 static void evt_integer_changed(GtkWidget *wid, gpointer user)
 {
-	gint		*value = user;
-	gboolean	*modified;
+	gint *value = user;
+	gboolean *modified;
 
 	*value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wid));
 	if((modified = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL)
@@ -143,64 +162,73 @@ static void evt_integer_changed(GtkWidget *wid, gpointer user)
 }
 
 /* Grab position. We implicitly *know* that the window in question is the main
-** gentoo window, and therefore access it directly through the MainInfo. We
-** then put the grabbed position into the editing copy of the WinDef, and update
-** the spin buttons to match. Of course, we don't forget setting that pesky old
-** modified-flag, either. :) What would people think?
-*/
+ ** gentoo window, and therefore access it directly through the MainInfo. We
+ ** then put the grabbed position into the editing copy of the WinDef, and update
+ ** the spin buttons to match. Of course, we don't forget setting that pesky old
+ ** modified-flag, either. :) What would people think?
+ */
 static void evt_pos_grab_clicked(GtkWidget *wid, gpointer user)
 {
-	MainInfo	*min;
-	WinDef		*wd = user;
+	MainInfo *min;
+	WinDef *wd = user;
 
 	min = gtk_object_get_data(GTK_OBJECT(wid), "min");
 	gdk_window_get_position(min->gui->window->window, &wd->x, &wd->y);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin1")), wd->x);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin2")), wd->y);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
+			GTK_OBJECT(wid), "spin1")), wd->x);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
+			GTK_OBJECT(wid), "spin2")), wd->y);
 	*(gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified") = TRUE;
 }
 
 /* Grab size. Works only for main gentoo window. See comment for pos_grab above. */
 static void evt_size_grab_clicked(GtkWidget *wid, gpointer user)
 {
-	MainInfo	*min;
-	WinDef		*wd = user;
+	MainInfo *min;
+	WinDef *wd = user;
 
 	min = gtk_object_get_data(GTK_OBJECT(wid), "min");
 	gdk_window_get_size(min->gui->window->window, &wd->width, &wd->height);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin1")), wd->width);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin2")), wd->height);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
+			GTK_OBJECT(wid), "spin1")), wd->width);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(
+			GTK_OBJECT(wid), "spin2")), wd->height);
 	*(gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified") = TRUE;
 }
 
 static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gboolean *modified)
 {
-	const gchar	*ltext[] = { N_("Position"), N_("Size") }, *etext[] = { N_("X"), N_("Y"), N_("Width"), N_("Height") };
-	GtkWidget	*label, *frame, *cbtn, *vbox, *table, *spin1, *spin2;
-	GtkObject	*adj;
-	gint		val1 = 0, val2 = 0;
+	const gchar *ltext[] =
+	{ N_("Position"), N_("Size") }, *etext[] =
+	{ N_("X"), N_("Y"), N_("Width"), N_("Height") };
+	GtkWidget *label, *frame, *cbtn, *vbox, *table, *spin1, *spin2;
+	GtkObject *adj;
+	gint val1 = 0, val2 = 0;
 
 	frame = gtk_frame_new(_(ltext[type]));
 	vbox = gtk_vbox_new(FALSE, 0);
 	cbtn = gtk_check_button_new_with_label(_("Set on Open?"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type == SUBFRAME_POSITION) ?
-								win_window_pos_use_get(wi, win->id) :
-								win_window_size_use_get(wi, win->id));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type
+			== SUBFRAME_POSITION) ? win_window_pos_use_get(wi,
+			win->id) : win_window_size_use_get(wi, win->id));
 	gtk_object_set_user_data(GTK_OBJECT(cbtn), modified);
-	gtk_signal_connect(GTK_OBJECT(cbtn), "clicked", GTK_SIGNAL_FUNC(evt_boolean_clicked),
-								(type == SUBFRAME_POSITION) ? &win->pos_use : &win->size_use);
+	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(
+			evt_boolean_clicked),
+			(type == SUBFRAME_POSITION) ? &win->pos_use
+					: &win->size_use);
 	gtk_box_pack_start(GTK_BOX(vbox), cbtn, FALSE, FALSE, 0);
 
 	cbtn = gtk_check_button_new_with_label(_("Update on Close?"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type == SUBFRAME_POSITION) ?
-								win_window_pos_update_get(wi, win->id) :
-								win_window_size_update_get(wi, win->id));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type
+			== SUBFRAME_POSITION) ? win_window_pos_update_get(wi,
+			win->id) : win_window_size_update_get(wi, win->id));
 	gtk_object_set_user_data(GTK_OBJECT(cbtn), modified);
-	gtk_signal_connect(GTK_OBJECT(cbtn), "clicked", GTK_SIGNAL_FUNC(evt_boolean_clicked),
-								(type == SUBFRAME_POSITION) ? &win->pos_update : &win->size_update);
+	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(
+			evt_boolean_clicked),
+			(type == SUBFRAME_POSITION) ? &win->pos_update
+					: &win->size_update);
 	gtk_box_pack_start(GTK_BOX(vbox), cbtn, FALSE, FALSE, 0);
 
-
 	table = gtk_table_new(2, 3, FALSE);
 
 	if(type == SUBFRAME_POSITION)
@@ -208,25 +236,30 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 	else
 		win_window_size_get(wi, win->id, &val1, &val2);
 
-	label  = gtk_label_new(_(etext[2 * type]));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
-	adj    = gtk_adjustment_new(val1, -1.0, 65535.0, 1.0, 25.0, 25.0);
-	spin1  = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
+	label = gtk_label_new(_(etext[2 * type]));
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 0, 0, 0, 0);
+	adj = gtk_adjustment_new(val1, -1.0, 65535.0, 1.0, 25.0, 0.0);
+	spin1 = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
 	gtk_object_set_user_data(GTK_OBJECT(spin1), modified);
-	gtk_signal_connect(GTK_OBJECT(spin1), "changed", GTK_SIGNAL_FUNC(evt_integer_changed),
-								type == SUBFRAME_POSITION ? &win->x : &win->width);
-	gtk_table_attach(GTK_TABLE(table), spin1, 1, 2, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	g_signal_connect(G_OBJECT(spin1), "changed", G_CALLBACK(
+			evt_integer_changed),
+			type == SUBFRAME_POSITION ? &win->x : &win->width);
+	gtk_table_attach(GTK_TABLE(table), spin1, 1, 2, 0, 1, GTK_EXPAND
+			| GTK_FILL, 0, 0, 0);
 	label = gtk_label_new(_(etext[2 * type + 1]));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
-	adj   = gtk_adjustment_new(val2, -1.0, 65535.0, 1.0, 25.0, 25.0);
-	spin2  = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 0, 0);
+	adj = gtk_adjustment_new(val2, -1.0, 65535.0, 1.0, 25.0, 0.0);
+	spin2 = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
 	gtk_object_set_user_data(GTK_OBJECT(spin2), modified);
-	gtk_signal_connect(GTK_OBJECT(spin2), "changed", GTK_SIGNAL_FUNC(evt_integer_changed),
-								type == SUBFRAME_POSITION ? &win->y : &win->height);
-	gtk_table_attach(GTK_TABLE(table), spin2, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-	if((type == SUBFRAME_POSITION && win->pos_grab) || (type == SUBFRAME_SIZE && win->size_grab))
+	g_signal_connect(G_OBJECT(spin2), "changed", G_CALLBACK(
+			evt_integer_changed),
+			type == SUBFRAME_POSITION ? &win->y : &win->height);
+	gtk_table_attach(GTK_TABLE(table), spin2, 1, 2, 1, 2, GTK_EXPAND
+			| GTK_FILL, 0, 0, 0);
+	if((type == SUBFRAME_POSITION && win->pos_grab) || (type
+			== SUBFRAME_SIZE && win->size_grab))
 	{
-		GtkWidget	*grab;
+		GtkWidget *grab;
 
 		grab = gtk_button_new_with_label(_("Grab"));
 
@@ -234,10 +267,11 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 		gtk_object_set_data(GTK_OBJECT(grab), "spin1", spin1);
 		gtk_object_set_data(GTK_OBJECT(grab), "spin2", spin2);
 		gtk_object_set_data(GTK_OBJECT(grab), "modified", modified);
-		gtk_signal_connect(GTK_OBJECT(grab), "clicked", type == SUBFRAME_POSITION ?
-								GTK_SIGNAL_FUNC(evt_pos_grab_clicked) :
-								GTK_SIGNAL_FUNC(evt_size_grab_clicked), win);
-		gtk_table_attach(GTK_TABLE(table), grab, 2, 3, 0, 2,  0,0,0,0);
+		g_signal_connect(G_OBJECT(grab), "clicked", type
+				== SUBFRAME_POSITION ? G_CALLBACK(
+				evt_pos_grab_clicked) : G_CALLBACK(
+				evt_size_grab_clicked), win);
+		gtk_table_attach(GTK_TABLE(table), grab, 2, 3, 0, 2, 0, 0, 0, 0);
 	}
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -247,10 +281,10 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 
 GtkWidget * win_wininfo_build(const WinInfo *wi, gboolean *modified)
 {
-	GtkWidget	*vbox, *frame, *hbox, *sframe;
-	GList		*iter;
-	gchar		tmp[64];
-	WinDef		*win;
+	GtkWidget *vbox, *frame, *hbox, *sframe;
+	GList *iter;
+	gchar tmp[64];
+	WinDef *win;
 
 	vbox = gtk_vbox_new(FALSE, 0);
 
@@ -258,9 +292,10 @@ GtkWidget * win_wininfo_build(const WinInfo *wi, gboolean *modified)
 	{
 		win = iter->data;
 
-		g_snprintf(tmp, sizeof tmp, "%c%s", toupper(*win->label), win->label + 1);
-		frame  = gtk_frame_new(tmp);
-		hbox   = gtk_hbox_new(FALSE, 0);
+		g_snprintf(tmp, sizeof tmp, "%c%s", toupper(*win->label),
+				win->label + 1);
+		frame = gtk_frame_new(tmp);
+		hbox = gtk_hbox_new(FALSE, 0);
 		sframe = subframe_build(wi, win, SUBFRAME_POSITION, modified);
 		gtk_box_pack_start(GTK_BOX(hbox), sframe, TRUE, TRUE, 5);
 		sframe = subframe_build(wi, win, SUBFRAME_SIZE, modified);
@@ -276,8 +311,8 @@ GtkWidget * win_wininfo_build(const WinInfo *wi, gboolean *modified)
 /* 2000-02-24 -	Write WinInfo out to given file <out>, at its current location. */
 void win_wininfo_save(const WinInfo *wi, FILE *out)
 {
-	GList	*iter;
-	WinDef	*wd;
+	GList *iter;
+	WinDef *wd;
 
 	xml_put_node_open(out, "Windows");
 	for(iter = wi->windows; iter != NULL; iter = g_list_next(iter))
@@ -296,60 +331,98 @@ void win_wininfo_save(const WinInfo *wi, FILE *out)
 		xml_put_boolean(out, "size_update", wd->size_update);
 		xml_put_node_close(out, "Window");
 	}
+	/* This goes INSIDE <Windows>, since there is no outer structure. :| */
+	xml_put_node_open(out, "Borders");
+	xml_put_integer(out, "width", wi->border_width);
+	xml_put_integer(out, "height", wi->border_height);
+	xml_put_node_close(out, "Borders");
 	xml_put_node_close(out, "Windows");
 }
 
-static void window_load(const XmlNode *window, gpointer user)
+static void window_load(const XmlNode *node, gpointer user)
 {
-	guint32	id;
+	WinInfo	*wi = user;
 
-	if(xml_get_uinteger(window, "id", &id))
+	if(xml_node_has_name(node, "Window"))
 	{
-		WinDef	*wd;
+		guint32 id;
 
-		if((wd = window_find(user, id)) != NULL)
+		if(xml_get_uinteger(node, "id", &id))
 		{
-			xml_get_integer(window, "x", &wd->x);
-			xml_get_integer(window, "y", &wd->y);
-			xml_get_integer(window, "w", &wd->width);
-			xml_get_integer(window, "h", &wd->height);
-			xml_get_boolean(window, "pos_use", &wd->pos_use);
-			xml_get_boolean(window, "pos_update", &wd->pos_update);
-			xml_get_boolean(window, "size_use", &wd->size_use);
-			xml_get_boolean(window, "size_update", &wd->size_update);
+			WinDef *wd;
+
+			if((wd = window_find(wi, id)) != NULL)
+			{
+				xml_get_integer(node, "x", &wd->x);
+				xml_get_integer(node, "y", &wd->y);
+				xml_get_integer(node, "w", &wd->width);
+				xml_get_integer(node, "h", &wd->height);
+				xml_get_boolean(node, "pos_use", &wd->pos_use);
+				xml_get_boolean(node, "pos_update", &wd->pos_update);
+				xml_get_boolean(node, "size_use", &wd->size_use);
+				xml_get_boolean(node, "size_update", &wd->size_update);
+			}
 		}
 	}
+	else if(xml_node_has_name(node, "Borders"))
+	{
+		xml_get_integer(node, "width", &wi->border_width);
+		xml_get_integer(node, "height", &wi->border_height);
+	}
 }
 
 /* 2000-02-24 -	Fill in <wi> with details for windows, loaded from <node>. */
 void win_wininfo_load(WinInfo *wi, const XmlNode *node)
 {
+	/* Visit all children. Since the XML structure, for the usual raisins, is a bit
+	** messed up, the Window elements are at the same level as the Borders.
+	*/
 	xml_node_visit_children(node, window_load, wi);
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
+void win_borders_set(WinInfo *wi, gint width, gint height)
+{
+	if(wi == NULL)
+		return;
+	wi->border_width = width;
+	wi->border_height = height;
+}
+
+void win_borders_get(const WinInfo *wi, gint *width, gint *height)
+{
+	if(wi == NULL)
+		return;
+	if(width != NULL)
+		*width = wi->border_width;
+	if(height != NULL)
+		*height = wi->border_height;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
 void win_window_new(WinInfo *wi, guint32 id, WinType type, gboolean persistent, const gchar *title, const gchar *label)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	wd = g_malloc(sizeof *wd);
-	wd->id    = id;
-	wd->type  = type;
+	wd->id = id;
+	wd->type = type;
 	wd->persistent = persistent;
 	wd->title = title;
 	wd->label = label;
 	wd->pos_grab = FALSE;
 	wd->size_grab = FALSE;
 
-	wd->x	   = 32;
-	wd->y	   = 32;
-	wd->width  = 32;
+	wd->x = 32;
+	wd->y = 32;
+	wd->width = 32;
 	wd->height = 32;
 
-	wd->pos_use     = FALSE;
-	wd->pos_update  = FALSE;
-	wd->size_use    = FALSE;
+	wd->pos_use = FALSE;
+	wd->pos_update = FALSE;
+	wd->size_use = FALSE;
 	wd->size_update = TRUE;
 
 	wi->windows = g_list_append(wi->windows, wd);
@@ -357,7 +430,7 @@ void win_window_new(WinInfo *wi, guint32 id, WinType type, gboolean persistent,
 
 static WinDef * window_find(const WinInfo *wi, guint32 id)
 {
-	GList	*iter;
+	GList *iter;
 
 	if(wi == NULL)
 		return NULL;
@@ -372,7 +445,7 @@ static WinDef * window_find(const WinInfo *wi, guint32 id)
 
 void win_window_pos_grab_set(const WinInfo *wi, guint32 id, gboolean grab)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		wd->pos_grab = grab;
@@ -380,7 +453,7 @@ void win_window_pos_grab_set(const WinInfo *wi, guint32 id, gboolean grab)
 
 void win_window_pos_set(const WinInfo *wi, guint32 id, gint x, gint y)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
@@ -391,7 +464,7 @@ void win_window_pos_set(const WinInfo *wi, guint32 id, gint x, gint y)
 
 gboolean win_window_pos_get(const WinInfo *wi, guint32 id, gint *x, gint *y)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
@@ -406,7 +479,7 @@ gboolean win_window_pos_get(const WinInfo *wi, guint32 id, gint *x, gint *y)
 
 void win_window_pos_use_set(const WinInfo *wi, guint32 id, gboolean enabled)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		wd->pos_use = enabled;
@@ -414,7 +487,7 @@ void win_window_pos_use_set(const WinInfo *wi, guint32 id, gboolean enabled)
 
 gboolean win_window_pos_use_get(const WinInfo *wi, guint32 id)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		return wd->pos_use;
@@ -423,7 +496,7 @@ gboolean win_window_pos_use_get(const WinInfo *wi, guint32 id)
 
 void win_window_pos_update_set(const WinInfo *wi, guint32 id, gboolean enabled)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		wd->pos_update = enabled;
@@ -431,7 +504,7 @@ void win_window_pos_update_set(const WinInfo *wi, guint32 id, gboolean enabled)
 
 gboolean win_window_pos_update_get(const WinInfo *wi, guint32 id)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		return wd->pos_update;
@@ -440,7 +513,7 @@ gboolean win_window_pos_update_get(const WinInfo *wi, guint32 id)
 
 void win_window_size_grab_set(const WinInfo *wi, guint32 id, gboolean grab)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		wd->size_grab = grab;
@@ -448,18 +521,19 @@ void win_window_size_grab_set(const WinInfo *wi, guint32 id, gboolean grab)
 
 void win_window_size_set(const WinInfo *wi, guint32 id, gint width, gint height)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
-		wd->width    = width;
-		wd->height   = height;
+		wd->width = width;
+		wd->height = height;
 	}
 }
 
-gboolean win_window_size_get(const WinInfo *wi, guint32 id, gint *width, gint *height)
+gboolean win_window_size_get(const WinInfo *wi, guint32 id, gint *width,
+		gint *height)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
@@ -474,7 +548,7 @@ gboolean win_window_size_get(const WinInfo *wi, guint32 id, gint *width, gint *h
 
 void win_window_size_use_set(const WinInfo *wi, guint32 id, gboolean enabled)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		wd->size_use = enabled;
@@ -482,7 +556,7 @@ void win_window_size_use_set(const WinInfo *wi, guint32 id, gboolean enabled)
 
 gboolean win_window_size_use_get(const WinInfo *wi, guint32 id)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		return wd->size_use;
@@ -491,7 +565,7 @@ gboolean win_window_size_use_get(const WinInfo *wi, guint32 id)
 
 void win_window_size_update_set(const WinInfo *wi, guint32 id, gboolean enabled)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		wd->size_update = enabled;
@@ -499,7 +573,7 @@ void win_window_size_update_set(const WinInfo *wi, guint32 id, gboolean enabled)
 
 gboolean win_window_size_update_get(const WinInfo *wi, guint32 id)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 		return wd->size_update;
@@ -510,7 +584,7 @@ gboolean win_window_size_update_get(const WinInfo *wi, guint32 id)
 
 void win_window_destroy(WinInfo *wi, guint32 id)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
@@ -524,68 +598,74 @@ void win_window_destroy(WinInfo *wi, guint32 id)
 /* 1999-12-23 -	Open the window described by <key> in <wi>. The window will not be shown yet. */
 GtkWidget * win_window_open(const WinInfo *wi, guint32 id)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
 	{
-		GtkWidget	*wid = NULL;
+		GtkWidget *wid = NULL;
+		gint old_errno;
 
-		switch(wd->type)
+		old_errno = errno;
+
+		switch (wd->type)
 		{
-			case WIN_TYPE_SIMPLE_TOPLEVEL:
-				wid = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-				break;
-			case WIN_TYPE_SIMPLE_DIALOG:
-				wid = gtk_window_new(GTK_WINDOW_DIALOG);
-				break;
-			case WIN_TYPE_SIMPLE_POPUP:
-				wid = gtk_window_new(GTK_WINDOW_POPUP);
-				break;
-			case WIN_TYPE_COMPLEX_DIALOG:
-				wid = gtk_dialog_new();
-				break;
+		case WIN_TYPE_SIMPLE_TOPLEVEL:
+		case WIN_TYPE_SIMPLE_DIALOG:
+			wid = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+			break;
+		case WIN_TYPE_SIMPLE_POPUP:
+			wid = gtk_window_new(GTK_WINDOW_POPUP);
+			break;
+		case WIN_TYPE_COMPLEX_DIALOG:
+			wid = gtk_dialog_new();
+			break;
 		}
-		gtk_object_set_data(GTK_OBJECT(wid), "windef", wd);
+		gtk_object_set_data(GTK_OBJECT(wid), "wininfo", (gpointer) wi);
+		gtk_object_set_data(GTK_OBJECT(wid), "windef", (gpointer) wd);
 		if(!wd->pos_use)
 			gtk_window_set_position(GTK_WINDOW(wid), GTK_WIN_POS_MOUSE);
 		gtk_widget_realize(wid);
-		if(wi->min->gui != NULL && wi->min->gui->window != NULL)
-			gdk_window_set_group(wid->window, wi->min->gui->window->window);
+		if(wi->min->gui != NULL && id != WIN_MAIN)
+			gtk_window_set_transient_for(wid, wi->min->gui->window);
 		if(wd->title != NULL)
 			win_window_set_title(wid, wd->title);
+		errno = old_errno;
 		return wid;
 	}
 	return NULL;
 }
 
 /* 2000-02-19 -	Refresh link between <wi> and <window>. Handy after <wi> has been replaced
-**		while window was open, as happens in the config window case.
-*/
+ **		while window was open, as happens in the config window case.
+ */
 void win_window_relink(const WinInfo *wi, guint32 id, GtkWidget *window)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = window_find(wi, id)) != NULL)
+	{
+		gtk_object_set_data(GTK_OBJECT(window), "wininfo", (gpointer) wi);
 		gtk_object_set_data(GTK_OBJECT(window), "windef", wd);
+	}
 }
 
 /* 2002-08-10 -	Set title of a window. Since it seems to break if locale is set, we simply
-**		un-set the locale temporarily. Hopefully, noone sets window titles in inner
-**		loops around here.
-** NOTE NOTE	This works for *all* windows, not just those created through this module.
-**		That might surprise, but it felt more sensical to stick it in here anyway.
-*/
+ **		un-set the locale temporarily. Hopefully, noone sets window titles in inner
+ **		loops around here.
+ ** NOTE NOTE	This works for *all* windows, not just those created through this module.
+ **		That might surprise, but it felt more sensical to stick it in here anyway.
+ */
 void win_window_set_title(GtkWidget *win, const gchar *title)
 {
 #if defined ENABLE_NLS
-	gchar		obuf[256];	/* Buffer, subsequent setlocale() call kills it otherwise. */
-	const gchar	*op;
-	int		old_errno = errno;	/* Buffer the errno, so any errors here don't leak out. */
+	gchar obuf[256]; /* Buffer, subsequent setlocale() call kills it otherwise. */
+	const gchar *op;
+	int old_errno = errno; /* Buffer the errno, so any errors here don't leak out. */
 
 	/* Get current locale, before slamming "C" in there. */
 	if((op = setlocale(LC_ALL, NULL)) != NULL)
 	{
-		const gchar	*sl;
+		const gchar *sl;
 
 		g_snprintf(obuf, sizeof obuf, "%s", op);
 		sl = setlocale(LC_ALL, "C");
@@ -597,23 +677,26 @@ void win_window_set_title(GtkWidget *win, const gchar *title)
 
 #if defined ENABLE_NLS
 	if(op != NULL)
-		setlocale(LC_ALL, obuf);
-	errno = old_errno;	/* Restore the errno, ignoring any errors that might have happened here. */
+	setlocale(LC_ALL, obuf);
+	errno = old_errno; /* Restore the errno, ignoring any errors that might have happened here. */
 #endif
 }
 
 /* 1999-12-23 -	Use this rather than gtk_widget_show() when you want to the window to appear, since
-**		this gives us a chance to set the window's size and position according to preference.
-*/
+ **		this gives us a chance to set the window's size and position according to preference.
+ */
 void win_window_show(GtkWidget *window)
 {
+	WinInfo	*wi;
 	WinDef	*wd;
+	gint	old_errno;
 
 	if(GTK_WIDGET_VISIBLE(window))
 		return;
 
+	wi = gtk_object_get_data(GTK_OBJECT(window), "wininfo");
 	wd = gtk_object_get_data(GTK_OBJECT(window), "windef");
-	if((wd != NULL) && wd->size_use && (wd->width >= 0))
+	if((wd != NULL) && wd->size_use && wd->width >= 0)
 		gtk_window_set_default_size(GTK_WINDOW(window), wd->width, wd->height);
 
 	/* This causes the window to appear at some more or less random position. We then reposition
@@ -621,18 +704,30 @@ void win_window_show(GtkWidget *window)
 	** but unfortunately I haven't been able to figure out how to fix it. If you move the show()
 	** call to below the if, it works, but not all the time. :( Tips welcome.
 	*/
-	gtk_widget_show(window);
+	old_errno = errno;
+	gtk_widget_show_all(window);
+	errno = old_errno;
 
-	if((wd != NULL) && wd->pos_use && (wd->x >= 0))
+	/* If user wants to set position, check where it ended up and move it home. */
+	if((wd != NULL) && wd->pos_use && wd->x >= 0)
 	{
+		gint	cx, cy;
+
+		gtk_window_get_position(GTK_WINDOW(window), &cx, &cy);
+		if(wi != NULL)	/* Any borders to adjust for? */
+		{
+			cx += wi->border_width;
+			cy += wi->border_height;
+		}
+		gtk_window_move(GTK_WINDOW(window), wd->x - cx, wd->y - cy);
 		gui_events_flush();
-		gtk_widget_set_uposition(GTK_WIDGET(window), wd->x, wd->y);
+		gtk_window_get_position(GTK_WINDOW(window), &cx, &cy);
 	}
 }
 
 /* 2000-03-04 -	If <a_new> is different from *<a>, update *<a> to <a_new> and return TRUE, else
-**		return FALSE. Likewise for <b>, whatever that would mean.
-*/
+ **		return FALSE. Likewise for <b>, whatever that would mean.
+ */
 static gboolean update_pair(gint *a, gint *b, gint a_new, gint b_new)
 {
 	if((*a != a_new) || (*b != b_new))
@@ -645,16 +740,16 @@ static gboolean update_pair(gint *a, gint *b, gint a_new, gint b_new)
 }
 
 /* 2000-03-18 -	Do an update for the given window, changing the config as needed. Returns TRUE
-**		if the config was indeed changed.
-*/
+ **		if the config was indeed changed.
+ */
 gboolean win_window_update(GtkWidget *win)
 {
-	WinDef		*wd;
+	WinDef *wd;
 
 	if((wd = gtk_object_get_data(GTK_OBJECT(win), "windef")) != NULL)
 	{
-		gboolean	ret = FALSE;
-		gint		ta, tb;
+		gboolean ret = FALSE;
+		gint ta, tb;
 
 		if(wd->pos_update)
 		{
@@ -672,18 +767,18 @@ gboolean win_window_update(GtkWidget *win)
 }
 
 /* 1999-12-23 -	Close the given window. Use this rather than a direct gtk_widget_destroy(), since
-**		we need a chance to update the position and size fields if needed. Also, if the
-**		window is classified as "persistent", we don't actually destroy it, but only hide
-**		it. Returns TRUE if the config was changed, i.e. if one of the update flags were
-**		set AND the relevant property actually had changed.
-*/
+ **		we need a chance to update the position and size fields if needed. Also, if the
+ **		window is classified as "persistent", we don't actually destroy it, but only hide
+ **		it. Returns TRUE if the config was changed, i.e. if one of the update flags were
+ **		set AND the relevant property actually had changed.
+ */
 gboolean win_window_close(GtkWidget *win)
 {
-	WinDef	*wd;
+	WinDef *wd;
 
 	if((wd = gtk_object_get_data(GTK_OBJECT(win), "windef")) != NULL)
 	{
-		gboolean	ret;
+		gboolean ret;
 
 		ret = win_window_update(win);
 		if(wd->persistent)
